@@ -30,6 +30,7 @@
 #endif
 
 #include "ADM_pp.h"
+#include "ADM_toneMapper.h"
 
 /**
     \fn recalibrateSigned
@@ -213,16 +214,26 @@ uint64_t tail;
     
         // Decode image...
         _SEGMENT *seg=_segments.getSegment(_currentSegment);
+        ADM_assert(seg);
         // Search it in the cache...
         _VIDEOS *vid=_segments.getRefVideo(seg->_reference);
-        
+        ADM_assert(vid);
+
         uint64_t refPts;
-        _segments.LinearToRefTime(_currentSegment,_currentPts,&refPts);
-        ADMImage *cached=vid->_videoCache->getAfter(refPts);
+
+        if(false == _segments.LinearToRefTime(_currentSegment,_currentPts,&refPts))
+        {
+            ADM_error("_currentSegment or _currentPts invalid or mismatched.\n");
+            return false;
+        }
+
+        EditorCache *cache = vid->_videoCache;
+        ADM_assert(cache);
+        ADMImage *cached = cache->getAfter(seg->_reference, refPts);
         if(cached)
         {
-            uint64_t delta=cached->Pts-seg->_refStartTimeUs;
-            if(delta<seg->_durationUs)
+            int64_t delta = cached->Pts - seg->_refStartTimeUs;
+            if(delta < (int64_t)seg->_durationUs)
             {
                 // Got it
                 image->duplicate(cached);
@@ -249,7 +260,7 @@ uint64_t tail;
                 if(frame && DecodePictureUpToIntra(seg->_reference,frame))
                 {
                     ADM_info("Successfully skipped to the next keyframe\n");
-                    cached=vid->_videoCache->getByPts(nkfPts);
+                    cached = cache->getByPts(seg->_reference, nkfPts);
                     if(cached)
                         image->duplicate(cached);
                 }else
@@ -291,6 +302,7 @@ np_nextSeg:
             }
             ADM_info("Switched to next segment\n");
             seg=_segments.getSegment(_currentSegment);
+            ADM_assert(seg);
             samePictureInternal(seg->_reference,image);
             updateImageTiming(seg,image);
             SET_CURRENT_PTS(image->Pts);
@@ -366,12 +378,17 @@ bool        ADM_Composer::previousPicture(ADMImage *image)
         uint64_t targetPts=_currentPts;
         // Decode image...
         _SEGMENT *seg=_segments.getSegment(_currentSegment);
+        ADM_assert(seg);
         // Search it in the cache...
         _VIDEOS *vid=_segments.getRefVideo(seg->_reference);
-        
-        uint64_t refPts;
+        ADM_assert(vid);
 
-        _segments.LinearToRefTime(_currentSegment,_currentPts,&refPts);
+        uint64_t refPts;
+        if(false == _segments.LinearToRefTime(_currentSegment,_currentPts,&refPts))
+        {
+            ADM_error("_currentSegment or _currentPts invalid or mismatched.\n");
+            return false;
+        }
 
         if(refPts<=vid->firstFramePts) // at the first frame of segment too
         {
@@ -385,8 +402,10 @@ bool        ADM_Composer::previousPicture(ADMImage *image)
             // and the segment boundary. Call a taxi then.
             targetPts=seg->_startTimeUs;
         }
-        ADMImage *cached=vid->_videoCache->getBefore(refPts);
-       
+        EditorCache *cache = vid->_videoCache;
+        ADM_assert(cache);
+        ADMImage *cached = cache->getBefore(seg->_reference, refPts);
+
         if(cached && cached->Pts<refPts)
         {
             if(cached->Pts>=seg->_refStartTimeUs) // It might be in the cache but belonging to the previous seg
@@ -398,8 +417,8 @@ bool        ADM_Composer::previousPicture(ADMImage *image)
                 return true;
             }
         }
-        ADM_info("while looking for frame %" PRIu64"\n",_currentPts);
-        vid->_videoCache->dump();
+        ADM_info("while looking for frame at PTS %s (%" PRIu64")\n",ADM_us2plain(_currentPts),_currentPts);
+        cache->dump();
         // The previous is not available
         // either it is in the same segment but we have decoded later in that segment
         // , or it is in the previous segment
@@ -415,17 +434,16 @@ bool        ADM_Composer::previousPicture(ADMImage *image)
         // 1- we are still in the same segment, in that case, we have to go back
         // and decode forward 
         // 2- We are at a jum, i.e. what we want is the last image from the previous segment
-        seg=_segments.getSegment(segNo);
         if(segNo==_currentSegment) // Still in the same segment..
         {
                 switch(decodeTillPictureAtPts(targetPts,image))
                 {
                     case 1:
-                        cached=vid->_videoCache->getBefore(refPts);
+                        cached = cache->getBefore(seg->_reference, refPts);
                         break;
                     case ADM_IGN:
                         ADM_warning("Cannot decode till our current pts\n");
-                        cached=vid->_videoCache->getLast();
+                        cached = cache->getLast(seg->_reference);
                         break;
                     default:
                         return false;
@@ -443,7 +461,7 @@ bool        ADM_Composer::previousPicture(ADMImage *image)
                     ADM_warning("The image found is before refStartTime ???\n");
                 }
                 ADM_error("Find our frame and its predecessor (%" PRIu64"), but it is out of range\n",refPts);
-                vid->_videoCache->dump();
+                cache->dump();
                 return false;
         }
         ADM_assert(segNo+1==_currentSegment);
@@ -455,20 +473,26 @@ bool        ADM_Composer::previousPicture(ADMImage *image)
             ADM_error("Cannot switch to previous segment to get previous frame\n");
             return false;
         }
-        
         seg=_segments.getSegment(_currentSegment);
+        ADM_assert(seg);
         vid=_segments.getRefVideo(seg->_reference);
-        
+        ADM_assert(vid);
 
-        if(false==decodeTillPictureAtPts(targetPts,image))
+        if(0 == decodeTillPictureAtPts(targetPts,image))
         {
             ADM_error("Cannot decode to get a candidate.\n");
             return false;
         }
         _currentSegment=segNo;
         // We may have overshot...
-        _segments.LinearToRefTime(_currentSegment,targetPts,&refPts);
-        ADMImage *candidate=vid->_videoCache->getLast();
+        if(false == _segments.LinearToRefTime(_currentSegment,targetPts,&refPts))
+        {
+            ADM_error("targetPts out of range for the ref video in segment %" PRIu32"\n",_currentSegment);
+            return false;
+        }
+        cache = vid->_videoCache;
+        ADM_assert(cache);
+        ADMImage *candidate = cache->getLast(seg->_reference);
         if(!candidate)
         {
             ADM_error("No frame in cache !\n");
@@ -481,7 +505,7 @@ bool        ADM_Composer::previousPicture(ADMImage *image)
                 break;
             }
             // Try to go before...
-            ADMImage *img=vid->_videoCache->getBefore(candidate->Pts);
+            ADMImage *img = cache->getBefore(seg->_reference, candidate->Pts);
             if(!img) break;
             candidate=img;
         }
@@ -512,7 +536,7 @@ uint32_t segNo;
     }
       ADM_assert(_currentSegment==segNo);
       refPts=segTime+seg->_refStartTimeUs;
-      ADMImage *last=ref->_videoCache->getByPts(refPts);
+      ADMImage *last=ref->_videoCache->getByPts(seg->_reference, refPts);
       if(last)
       {
             image->duplicate(last);
@@ -531,12 +555,12 @@ uint32_t segNo;
 */
 uint8_t ADM_Composer::dupe(ADMImage *src,ADMImage *dst,_VIDEOS *vid)
 {
-    if(src->_colorspace==ADM_COLOR_YV12)
+    if(src->_pixfrmt==ADM_PIXFRMT_YV12)
         return dst->duplicate(src);
     // We need to do some colorspace conversion
     // Is there already one ?
     if(!vid->color)
-        vid->color=new ADMColorScalerSimple(src->_width,src->_height,src->_colorspace,ADM_COLOR_YV12);
+        vid->color=new ADMColorScalerSimple(src->_width,src->_height,src->_pixfrmt,ADM_PIXFRMT_YV12);
     // Since it is not YV12 it MUST be a ref
     ADM_assert(src->isRef());
     dst->copyInfo(src);
@@ -568,6 +592,27 @@ uint8_t ADM_Composer::getPostProc( uint32_t *type, uint32_t *strength, bool *swa
 	*swapuv=_pp->swapuv;
 	return 1;
 }
+/**
+    \fn setHDRConfig
+*/
+uint8_t ADM_Composer::setHDRConfig( uint32_t toneMappingMethod, float saturationAdjust, float boostAdjust)
+{
+    if(!_segments.getNbRefVideos()) return 0;
+    if(!_hdrConfig) return 0;
+    _hdrConfig->setConfig(toneMappingMethod, saturationAdjust, boostAdjust);
+    return 1;
+}
+/**
+    \fn getHDRConfig
+*/
+
+uint8_t ADM_Composer::getHDRConfig( uint32_t * toneMappingMethod, float * saturationAdjust, float * boostAdjust)
+{
+    if(!_segments.getNbRefVideos()) return 0;
+    if(!_hdrConfig) return 0;
+    _hdrConfig->getConfig(toneMappingMethod, saturationAdjust, boostAdjust);
+    return 1;
+}
 /**
     \fn switchToNextSegment
     \brief Switch to the next segment
