@@ -77,6 +77,13 @@ void ADM_ScriptGenerator::generateScript(std::iostream& stream,const GeneratorTy
         {
             this->_scriptWriter->setPostProcessing(pptype, ppstrength, ppswap);
         }
+        
+        // HDR config
+        printf("Scripting HDR tone mapping config\n");
+        uint32_t toneMappingMethod;
+        float saturationAdjust, boostAdjust;
+        this->_editor->getHDRConfig(&toneMappingMethod, &saturationAdjust, &boostAdjust);
+        this->_scriptWriter->setHDRConfig(toneMappingMethod, saturationAdjust, boostAdjust);
 
         // Video codec
     }
@@ -143,13 +150,13 @@ void ADM_ScriptGenerator::generateScript(std::iostream& stream,const GeneratorTy
 
                     // --------- Normalize ----------------
             ADM_GAINMode mode;
-            uint32_t gain;
+            int32_t gain, maxlevel;
 
-            track->audioEncodingConfig.audioFilterGetNormalize(&mode, &gain);
+            track->audioEncodingConfig.audioFilterGetNormalize(&mode, &gain, &maxlevel);
 
             if (mode != ADM_NO_GAIN)
             {
-                            this->_scriptWriter->setAudioGain(i, mode, gain);
+                            this->_scriptWriter->setAudioGain(i, mode, gain, maxlevel);
             }
         }
     }
@@ -169,6 +176,7 @@ bool ADM_ScriptGenerator::saveVideoFilters()
     for (int i = 0; i < nbFilter; i++)
     {
 		ADM_VideoFilterElement *element = &ADM_VideoFilters[i];
+		if(!element->enabled) continue;
 		this->_scriptWriter->addVideoFilter(ADM_vf_getPluginFromTag(element->tag), element);
     }
     return true;
@@ -190,4 +198,4 @@ bool ADM_ScriptGenerator::end()
 {
         this->_scriptWriter->disconnectStream();
         return true;
-}
\ No newline at end of file
+}
