@@ -80,7 +80,6 @@ class dxvaRender: public VideoRenderBase,public ADM_QvideoDrawer
                         GUI_WindowInfo  info;
                         IDirect3DSurface9 *mySurface;
                         IDirect3DSurface9 *myYV12Surface;
-                        uint8_t           *videoBuffer;
                         D3DDISPLAYMODE     displayMode;
                         IDirect3DDevice9  *d3dDevice;
                         IDirect3D9        *d3dHandle;
@@ -112,7 +111,6 @@ dxvaRender::dxvaRender()
     useYV12=false;
     mySurface=NULL;
     myYV12Surface=NULL;
-    videoBuffer=NULL;
     videoWidget=NULL;
     d3dHandle=admD3D::getHandle();
 }
@@ -322,10 +320,9 @@ bool dxvaRender::setup()
   }
 
   scaler=new ADMColorScalerFull(ADM_CS_BICUBIC,imageWidth,imageHeight,displayWidth,displayHeight,
-          ADM_COLOR_YV12,
-          ADM_COLOR_RGB32A
+          ADM_PIXFRMT_YV12,
+          ADM_PIXFRMT_RGB32A
       );
-  videoBuffer=new uint8_t[displayWidth*displayHeight*4];
   panScan.left  =0;
   panScan.right =imageWidth-1;
   panScan.top   =0;
@@ -366,11 +363,6 @@ bool dxvaRender::cleanup()
        D3DCallNoArg(IDirect3DDevice9,Release,d3dDevice);
        d3dDevice=NULL;
     }
-    if(videoBuffer)
-    {
-        delete [] videoBuffer;
-        videoBuffer=NULL;
-    }
     return true;
 }
  /**
@@ -443,10 +435,10 @@ static bool  ADMImage_To_yv12Surface(ADMImage *pic, IDirect3DSurface9 *surface)
   d3dBlit(pic, PLANAR_Y,dst,dStride,width,height);
 
   dst+=height*dStride;
-  d3dBlit(pic, PLANAR_U,dst,dStride>>1,width>>1,height>>1);
+  d3dBlit(pic, PLANAR_V, dst, dStride>>1, width>>1, height>>1);
 
   dst+=(height/2)*(dStride/2);
-  d3dBlit(pic, PLANAR_V,dst,dStride>>1,width>>1,height>>1);
+  d3dBlit(pic, PLANAR_U, dst, dStride>>1, width>>1, height>>1);
 
   if (ADM_FAILED(D3DCallNoArg(IDirect3DSurface9,UnlockRect,surface)))
   {
