@@ -17,6 +17,8 @@
 
 class decoderFFVT : public ADM_acceleratedDecoderFF
 {
+protected:
+                    ADMImage    *copy;
 public:
                     bool        alive;
                                 decoderFFVT (AVCodecContext *avctx,decoderFF *parent);
