@@ -237,12 +237,16 @@ static enum AVPixelFormat ADM_DXVA2_getFormat(struct AVCodecContext *avctx,  con
                 {
                     prefs->get(FEATURES_DXVA2_OVERRIDE_BLACKLIST_VERSION,&ignore_version);
                     const char *minversion=humanReadable(INTEL_MIN_DRIVER_VERSION_FOR_HEVC);
+                    const char *driverVersion=humanReadable(drv);
                     if(!ignore_version)
                     {
-                        ADM_warning("Intel driver version %s < %s is blacklisted for HEVC decoding.\n",humanReadable(drv),minversion);
+                        ADM_warning("Intel driver version %s < %s is blacklisted for HEVC decoding.\n",driverVersion,minversion);
+                        ADM_dealloc(minversion);
+                        ADM_dealloc(driverVersion);
                         continue;
                     }
-                    ADM_warning("Overriding Intel driver version blacklist for %s\n",humanReadable(drv));
+                    ADM_warning("Overriding Intel driver version blacklist for %s\n",driverVersion);
+                    ADM_dealloc(driverVersion);
                 }else if(vid==admD3D::VENDOR_INTEL && dxvaBitDepthFromContext(avctx)==10)
                 {
                     prefs->get(FEATURES_DXVA2_OVERRIDE_BLACKLIST_PROFILE,&ignore_profile);
@@ -489,6 +493,8 @@ bool decoderFFDXVA2::uncompress (ADMCompressedImage * in, ADMImage * out)
     AVFrame *frame=_parent->getFramePointer();
     ADM_assert(frame);
 
+    int ret = 0;
+
     if(_parent->getDrainingState())
     {
         if(_parent->getDrainingInitiated()==false)
@@ -498,22 +504,30 @@ bool decoderFFDXVA2::uncompress (ADMCompressedImage * in, ADMImage * out)
         }
     }else if(!handover)
     {
-        AVPacket pkt;
-        av_init_packet(&pkt);
-        pkt.data=in->data;
-        pkt.size=in->dataLength;
-        if(in->flags&AVI_KEY_FRAME)
-            pkt.flags=AV_PKT_FLAG_KEY;
-        else
-            pkt.flags=0;
+        AVPacket *pkt = _parent->getPacketPointer();
+        ADM_assert(pkt);
+        pkt->data = in->data;
+        pkt->size = in->dataLength;
 
-        avcodec_send_packet(_context, &pkt);
+        if(in->flags&AVI_KEY_FRAME)
+            pkt->flags = AV_PKT_FLAG_KEY;
+        else
+            pkt->flags = 0;
+
+        ret = avcodec_send_packet(_context, pkt);
+        if(ret)
+        {
+            char er[AV_ERROR_MAX_STRING_SIZE]={0};
+            av_make_error_string(er, AV_ERROR_MAX_STRING_SIZE, ret);
+            ADM_warning("Ignoring error %d submitting packet to decoder (\"%s\")\n",ret,er);
+        }
+        av_packet_unref(pkt);
     }else
     {
         handover=false;
     }
 
-    int ret = avcodec_receive_frame(_context, frame);
+    ret = avcodec_receive_frame(_context, frame);
 
     if(!_parent->decodeErrorHandler(ret))
         return false;
@@ -538,6 +552,7 @@ bool     decoderFFDXVA2::readBackBuffer(AVFrame *decodedFrame, ADMCompressedImag
     uint64_t pts_opaque=(uint64_t)(decodedFrame->reordered_opaque);
     out->Pts= (uint64_t)(pts_opaque);
     out->flags=admFrameTypeFromLav(decodedFrame);
+    out->_range=(decodedFrame->color_range==AVCOL_RANGE_JPEG)? ADM_COL_RANGE_JPEG : ADM_COL_RANGE_MPEG;
     out->refType=ADM_HW_DXVA;
     out->refDescriptor.refCodec=this;
     admDx2Surface *img=(admDx2Surface *)(decodedFrame->data[0]);
@@ -657,13 +672,21 @@ bool           ADM_hwAccelEntryDxva2::canSupportThis(struct AVCodecContext *avct
         return false;
 
     outputFormat=ofmt;
-    ADM_info("This is maybe supported by DXVA2...\n");
+    int align=16;
+    if(avctx->codec_id == AV_CODEC_ID_H265)
+        align=128;
+    int width=avctx->width;
+    int height=avctx->height;
+    width=(width+align-1)&~(align-1);
+    height=(height+align-1)&~(align-1);
+    ADM_info("This is maybe supported by DXVA2, padded width: %d padded height: %d\n",width,height);
     int bits=dxvaBitDepthFromContext(avctx);
-    if(!admDxva2::supported(avctx->codec_id,bits)) // not sure either
+    if(!admDxva2::supported(avctx->codec_id, bits, width, height)) // not sure either
     {
         ADM_warning("Not supported by DXVA2\n");
         return false;
     }
+    ADM_info("This is supported by DXVA2\n");
     return true;
 }
 
