@@ -153,6 +153,7 @@ int decoderFFVDPAU::getBuffer(AVCodecContext *avctx, AVFrame *pic)
         ADM_info("[VDPAU] No more available surface, creating a new one\n");
         render=new ADM_vdpauRenderState;
         memset(render,0,sizeof( ADM_vdpauRenderState));
+        render->surface=VDP_INVALID_HANDLE;
         int widthToUse = admVdpau::dimensionRoundUp(avctx->coded_width);
         int heightToUse= admVdpau::dimensionRoundUp(avctx->coded_height);
         if(VDP_STATUS_OK!=admVdpau::surfaceCreate(widthToUse,heightToUse,&(render->surface)))
@@ -162,8 +163,6 @@ int decoderFFVDPAU::getBuffer(AVCodecContext *avctx, AVFrame *pic)
             alive=false;
             return -1;
         }
-        render->state=0;
-        vdpauMarkSurfaceUsed(&vdpau,(void *)render);
         surfaceMutex.lock();
         vdpau.fullQueue.push_back(render);
         surfaceMutex.unlock();
@@ -220,11 +219,16 @@ static enum AVPixelFormat vdpauGetFormat(struct AVCodecContext *avctx,  const en
             c=fmt[i];
             ADM_info("[vdpau]: Evaluating %d\n",c);
             if(c!=AV_PIX_FMT_VDPAU) continue;        
-    #define FMT_V_CHECK(x,y)      case AV_CODEC_ID_##x:  \
-                                         if(admVdpau::queryDecoderCapabilities(y,&maxW,&maxH)) \
-                                         { id=avctx->codec_id;\
-                                         outPix=AV_PIX_FMT_VDPAU ;}else {ADM_info(#x ":Not supported by HW\n");}\
-                                         break;
+    #define FMT_V_CHECK(x,y) case AV_CODEC_ID_##x: \
+            if(admVdpau::queryDecoderCapabilities(y,&maxW,&maxH)) \
+            { \
+                if(avctx->width > maxW) { ADM_warning(#x ": width %d exceeds max %u supported by HW\n",avctx->width,maxW); break; } \
+                if(avctx->height > maxH) { ADM_warning(#x ": height %d exceeds max %u supported by HW\n",avctx->height,maxH); break; } \
+                id=avctx->codec_id; \
+                outPix=AV_PIX_FMT_VDPAU; \
+            }else \
+            {   ADM_info(#x ":Not supported by HW\n"); } \
+            break;
             switch(avctx->codec_id)
             {
                 FMT_V_CHECK(H264,      VDP_DECODER_PROFILE_H264_HIGH)
@@ -337,7 +341,7 @@ decoderFFVDPAU::~decoderFFVDPAU()
             ADM_vdpauRenderState *r=vdpau.fullQueue[i];
             if(r)
             {
-                if(r->surface)
+                if(r->surface!=VDP_INVALID_HANDLE)
                 {
                         if(VDP_STATUS_OK!=admVdpau::surfaceDestroy((r->surface)))
                                 ADM_error("Error destroying surface %d\n",i);
@@ -360,7 +364,7 @@ bool decoderFFVDPAU::uncompress (ADMCompressedImage * in, ADMImage * out)
             out->refType=ADM_HW_NONE;
     }
 
-    if (!in->dataLength )	// Null frame, silently skipped
+    if(!_parent->getDrainingState() && !in->dataLength) // Null frame, silently skipped
     {
         out->_noPicture = 1;
         out->Pts=ADM_COMPRESSED_NO_PTS;
@@ -371,18 +375,12 @@ bool decoderFFVDPAU::uncompress (ADMCompressedImage * in, ADMImage * out)
    // Put a safe value....
     out->Pts=in->demuxerPts;
     _context->reordered_opaque=in->demuxerPts;
-    int got_picture;
-    AVPacket pkt;
-    av_init_packet(&pkt);
-    pkt.data=in->data;
-    pkt.size=in->dataLength;
-    if(in->flags&AVI_KEY_FRAME)
-        pkt.flags=AV_PKT_FLAG_KEY;
-    else
-        pkt.flags=0;
 
     AVFrame *frame=_parent->getFramePointer();
     ADM_assert(frame);
+
+    int ret = 0;
+
     if(_parent->getDrainingState())
     {
         if(_parent->getDrainingInitiated()==false)
@@ -390,12 +388,32 @@ bool decoderFFVDPAU::uncompress (ADMCompressedImage * in, ADMImage * out)
             avcodec_send_packet(_context, NULL);
             _parent->setDrainingInitiated(true);
         }
+    }else if(!handover)
+    {
+        AVPacket *pkt = _parent->getPacketPointer();
+        ADM_assert(pkt);
+        pkt->data = in->data;
+        pkt->size = in->dataLength;
+
+        if(in->flags&AVI_KEY_FRAME)
+            pkt->flags = AV_PKT_FLAG_KEY;
+        else
+            pkt->flags = 0;
+
+        ret = avcodec_send_packet(_context, pkt);
+        if(ret)
+        {
+            char er[AV_ERROR_MAX_STRING_SIZE]={0};
+            av_make_error_string(er, AV_ERROR_MAX_STRING_SIZE, ret);
+            ADM_warning("Ignoring error %d submitting packet to decoder (\"%s\")\n",ret,er);
+        }
+        av_packet_unref(pkt);
     }else
     {
-        avcodec_send_packet(_context, &pkt);
+        handover=false;
     }
 
-    int ret = avcodec_receive_frame(_context, frame);
+    ret = avcodec_receive_frame(_context, frame);
 
     if(!_parent->decodeErrorHandler(ret))
         return false;
@@ -435,6 +453,7 @@ bool     decoderFFVDPAU::readBackBuffer(AVFrame *decodedFrame, ADMCompressedImag
     uint64_t pts_opaque=(uint64_t)(decodedFrame->reordered_opaque);
     out->Pts= (uint64_t)(pts_opaque);    
     out->flags=admFrameTypeFromLav(decodedFrame);
+    out->_range=(decodedFrame->color_range==AVCOL_RANGE_JPEG)? ADM_COL_RANGE_JPEG : ADM_COL_RANGE_MPEG;
     return true;
 }
 
