@@ -21,10 +21,13 @@
 // Get list
 uint32_t                ADM_vf_getSize(void);
 ADM_coreVideoFilter     *ADM_vf_getInstance(int index);
+bool                    ADM_vf_getEnabled(int index);
 uint32_t                ADM_vf_getTag(int index);
 bool                    ADM_vf_getConfigurationFromIndex(int index,CONFcouple **c);
 
 bool                    ADM_vf_configureFilterAtIndex(int index);
 bool                    ADM_vf_moveFilterDown(int index);
 bool                    ADM_vf_moveFilterUp(int index);
+bool                    ADM_vf_partialize(int index);
+bool                    ADM_vf_absolvePartialized(int index);
 #endif
