@@ -15,6 +15,8 @@
 
 #include "ADM_default.h"
 #include "ADM_coreVideoFilter.h"
+#include "ADM_coreVideoFilterFunc.h"
+#include "ADM_videoFilterApi.h"
 #include "DIA_factory.h"
 #include "DIA_coreToolkit.h"
 #include "ADM_vidMisc.h"
@@ -22,8 +24,7 @@
 #include "partial_desc.cpp"
 #include "avi_vars.h"
 
-extern ADM_coreVideoFilter *ADM_vf_createFromTag(uint32_t tag, ADM_coreVideoFilter *last, CONFcouple *couples);
-extern uint32_t    ADM_vf_getTagFromInternalName(const char *name);
+#define DESC_MAX_LENGTH 2048
 
 ADM_coreVideoFilter *createPartialFilter(const char *internalName,CONFcouple *couples);
 /**
@@ -55,11 +56,12 @@ protected:
                 ADM_coreVideoFilter *sonFilter;
                 partial      configuration;
                 bool         byPass;
-                char         description[2048];
+                char         description[DESC_MAX_LENGTH];
                 ADMImage     *intermediate;
                 bool         hasIntermediate;
                 uint32_t     intermediateFn;
                 bool         sonFilterPreview;
+                FilterInfo   previewFilterInfo;
 
 
                 bool        isInRange(uint64_t tme);
@@ -71,6 +73,7 @@ public:
         virtual const char   *getConfiguration(void);                   /// Return  current configuration as a human readable string
         virtual bool         getNextFrame(uint32_t *fn,ADMImage *image);    /// Return the next image
         virtual bool         getNextFrameForSon(uint32_t *fn,ADMImage *image);    /// Return the next image
+        virtual FilterInfo   *getInfo(void);
         virtual bool         getCoupledConf(CONFcouple **couples) ;     /// Return the current filter configuration
         virtual void         setCoupledConf(CONFcouple *couples);
         virtual bool         configure(void); /// Start graphical user interface
@@ -132,8 +135,6 @@ partialFilter::partialFilter(  ADM_coreVideoFilter *in,CONFcouple *setup) : ADM_
         configuration.startBlack=configuration.endBlack;
         configuration.endBlack=swap;
     }
-    if(configuration.endBlack > info.totalDuration)
-        configuration.endBlack=info.totalDuration;
     // Ok, create trampoline & son
     trampoline=new trampolineFilter(this,NULL);
     // Create swallowed filter
@@ -227,15 +228,28 @@ bool partialFilter::getNextFrame(uint32_t *fn,ADMImage *image)
     return true;
 }
 
+/**
+    \fn getInfo
+    \brief Get FilterInfo
+*/
+FilterInfo  *partialFilter::getInfo(void)
+{
+    if (!sonFilterPreview)
+        return previousFilter->getInfo();
+    
+    previewFilterInfo = *(previousFilter->getInfo());
+    getRange(&(previewFilterInfo.markerA), &(previewFilterInfo.markerB));
+    return &previewFilterInfo;
+}
+
 /**
  */
 bool partialFilter::isInRange(uint64_t tme)
 {
-    uint64_t first=previousFilter->getAbsoluteStartTime();
-    uint64_t from=1000LL*configuration.startBlack;
-    uint64_t to=1000LL*configuration.endBlack;
+    tme += previousFilter->getAbsoluteStartTime();
+    uint32_t tmeMs = tme / 1000LL;	// we need to comprare at ms resolution, otherwise if usec part not zero, would mess up things
 
-    if(tme+first<from || tme+first>to)
+    if(tmeMs<configuration.startBlack || tmeMs>=configuration.endBlack)
     {
         return false;
     }
@@ -312,11 +326,39 @@ void partialFilter::setCoupledConf(CONFcouple *couples)
 */
 const char *partialFilter::getConfiguration(void)
 {
-  sprintf(description,"Partial : %s -- ",ADM_us2plain((uint64_t)(configuration.startBlack)*1000));
-  strcat(description,ADM_us2plain((uint64_t)(configuration.endBlack)*1000));
-  strcat(description," ");
-  strcat(description,sonFilter->getConfiguration());
-  return description;
+    uint32_t id = ADM_vf_getTagFromInternalName(configuration.filterName.c_str());
+    int len = DESC_MAX_LENGTH;
+
+    snprintf(description,len,"%s: ",ADM_vf_getDisplayNameFromTag(id));
+
+    len -= strlen(description);
+    if(len < 1)
+        return description;
+
+    const char *str = ADM_us2plain((uint64_t)(configuration.startBlack)*1000);
+    len -= strlen(str)+4;
+    if(len < 1)
+        return description;
+
+    strcat(description,str);
+    strcat(description," -- ");
+
+    str = ADM_us2plain((uint64_t)(configuration.endBlack)*1000);
+    len -= strlen(str)+1;
+    if(len < 1)
+        return description;
+
+    strcat(description,str);
+    strcat(description,"\n");
+
+    str = sonFilter->getConfiguration();
+    len -= strlen(str);
+    if(len < 1)
+        return description;
+
+    strcat(description,str);
+
+    return description;
 }
 /**
  */
@@ -339,13 +381,19 @@ void partialFilter::reconfigureSon(void)
 */
 bool partialFilter::configure( void)
 {
-        uint32_t mx=(uint32_t)(previousFilter->getInfo()->totalDuration/1000);
-        diaElemTimeStamp start(&(configuration.startBlack),QT_TRANSLATE_NOOP("partial","_Start time:"),0,mx);
-        diaElemTimeStamp end(&(configuration.endBlack),QT_TRANSLATE_NOOP("partial","_End time:"),0,mx);
-        diaElemButton    son(QT_TRANSLATE_NOOP("partial", "Configure filter"), partialFilter::reconfigureCallback,this);
+    uint32_t mx = (uint32_t)(previousFilter->getInfo()->totalDuration/1000);
+    uint32_t id = ADM_vf_getTagFromInternalName(configuration.filterName.c_str());
+    char str[256];
+    str[0] = '\0';
+    snprintf(str,256,QT_TRANSLATE_NOOP("partial","Partialize \"%s\""),ADM_vf_getDisplayNameFromTag(id));
+    str[255] = '\0';
 
-        diaElem *elems[3]={&start,&end,&son};
-        return diaFactoryRun(QT_TRANSLATE_NOOP("partial","Partial Filter"),3,elems);
+    diaElemTimeStamp start(&configuration.startBlack, QT_TRANSLATE_NOOP("partial","_Start time:"), 0, mx);
+    diaElemTimeStamp end(&configuration.endBlack, QT_TRANSLATE_NOOP("partial","_End time:"), 0, mx);
+    diaElemButton son(QT_TRANSLATE_NOOP("partial", "Configure filter"), partialFilter::reconfigureCallback, this);
+
+    diaElem *elems[3]={&start,&end,&son};
+    return diaFactoryRun(str,3,elems);
 }
 
 /**
@@ -414,16 +462,12 @@ ADM_coreVideoFilter *createPartialFilter(const char *internalName,CONFcouple *co
   CONFcouple tmp(3+sonNbItems);
 
   uint32_t start, end;
-  if(source->getInfo()->totalDuration == video_body->getVideoDuration())
-  {
-      start=video_body->getMarkerAPts()/1000;
-      end=video_body->getMarkerBPts()/1000;
-  }else
-  {
-      double f=(double)(source->getInfo()->totalDuration) / video_body->getVideoDuration();
-      start=(uint32_t)(video_body->getMarkerAPts() * f / 1000);
-      end=(uint32_t)(video_body->getMarkerBPts() * f / 1000);
-  }
+  uint64_t startPts, endPts;
+  startPts = source->getInfo()->markerA;
+  endPts = source->getInfo()->markerB;
+
+  start = startPts/1000;
+  end = endPts/1000;
 
   tmp.writeAsString("filterName",internalName);
   tmp.writeAsUint32 ("startBlack",start);
