@@ -101,6 +101,7 @@ void brokenAct(void);
 void HandleAction (Action action);
 void HandleAction_Navigate(Action action);
 void HandleAction_Save(Action action);
+void HandleAction_Staged(Action action);
 
 extern void call_scriptEngine(const char *scriptFile);
 
@@ -337,11 +338,10 @@ void HandleAction (Action action)
           }
         ADM_info("Closing ui\n");
         UI_closeGui();
-        if(video_body && video_body->canUndo())
-        {
+        if(video_body && avifileinfo)
             A_saveSession();
+        if(video_body && video_body->canUndo())
             video_body->clearUndoQueue();
-        }
         return;
     case ACT_RESTORE_SESSION:
         if(playing) break;
@@ -395,6 +395,10 @@ void HandleAction (Action action)
   {
     return HandleAction_Save(action);
   }
+  if(action > ACT_STAGED_BEGIN && action < ACT_STAGED_END)
+  {
+    return HandleAction_Staged(action);
+  }
 
   switch (action)
     {
@@ -437,7 +441,7 @@ void HandleAction (Action action)
         {
                 float currentZoom=(float)(2<<(action-ACT_ZOOM_1_4))/8;
                 UI_setBlockZoomChangesFlag(true);
-                changePreviewZoom(currentZoom);
+                admPreview::changePreviewZoom(currentZoom);
                 UI_setBlockZoomChangesFlag(false);
                 UI_resetZoomThreshold();
                 admPreview::samePicture();
@@ -466,7 +470,7 @@ void HandleAction (Action action)
 #define TOGGLE_PREVIEW ADM_PREVIEW_OUTPUT
     case ACT_PreviewChanged:
     {
-        ADM_PREVIEW_MODE oldpreview=getPreviewMode(),newpreview=(ADM_PREVIEW_MODE)UI_getCurrentPreview();
+        ADM_PREVIEW_MODE oldpreview = admPreview::getPreviewMode(),newpreview=(ADM_PREVIEW_MODE)UI_getCurrentPreview();
           printf("Old preview %d, New preview mode : %d\n",oldpreview,newpreview);
 
           if(oldpreview==newpreview)
@@ -474,7 +478,7 @@ void HandleAction (Action action)
             return;
           }
             admPreview::stop();
-            setPreviewMode(newpreview);
+            admPreview::setPreviewMode(newpreview);
             admPreview::start();
 //            admPreview::update(curframe);
       }
@@ -489,23 +493,32 @@ void HandleAction (Action action)
     case ACT_MarkA:
     case ACT_MarkB:
     {
-      bool swapit=0;
-      uint64_t markA,markB;
-      uint64_t pts=admPreview::getCurrentPts();
-      if( prefs->get(FEATURES_SWAP_IF_A_GREATER_THAN_B, &swapit) != RC_OK )     swapit = 1;
+        bool swapit = false;
+        uint64_t pts,markA,markB;
+        pts = admPreview::getCurrentPts();
+        if(!prefs->get(FEATURES_SWAP_IF_A_GREATER_THAN_B, &swapit))
+            swapit = true;
 
-      markA=video_body->getMarkerAPts();
-      markB=video_body->getMarkerBPts();
-      if (action == ACT_MarkA)
+        markA = video_body->getMarkerAPts();
+        markB = video_body->getMarkerBPts();
+        if(action == ACT_MarkA)
             markA=pts;
-      else
+        else
             markB=pts;
-      if (markA>markB && swapit )    // auto swap
+        if(markA > markB)
         {
-          uint64_t y;
-          y = markA;
-          markA=markB;
-          markB=y;
+            if(swapit) // auto swap
+            {
+                uint64_t y = markA;
+                markA = markB;
+                markB = y;
+            }else
+            {
+                if(action == ACT_MarkA)
+                    markB = video_body->getVideoDuration(); // reset B
+                else
+                    markA = 0; // reset A
+            }
         }
         video_body->addToUndoQueue();
         video_body->setMarkerAPts(markA);
@@ -513,13 +526,29 @@ void HandleAction (Action action)
         UI_setMarkers (markA, markB);
       break;
     }
+    case ACT_ResetMarkerA:
+        if(0 == video_body->getMarkerAPts())
+            break;
+        video_body->addToUndoQueue();
+        video_body->setMarkerAPts(0);
+        UI_setMarkers(0, video_body->getMarkerBPts());
+        break;
+    case ACT_ResetMarkerB:
+    {
+        uint64_t end = video_body->getVideoDuration();
+        if(end == video_body->getMarkerBPts())
+            break;
+        video_body->addToUndoQueue();
+        video_body->setMarkerBPts(end);
+        UI_setMarkers(video_body->getMarkerAPts(), end);
+        break;
+    }
     case ACT_ResetMarkers:
     {
         if(!video_body->getMarkerAPts() && video_body->getMarkerBPts()==video_body->getVideoDuration())
             break; // do nothing if the markers were not moved
         video_body->addToUndoQueue();
         A_ResetMarkers();
-        A_Resync();
         break;
     }
     case ACT_Copy:
@@ -622,7 +651,19 @@ void HandleAction (Action action)
               video_body->setMarkerAPts(markA);
               video_body->setMarkerBPts(markB);
               A_Resync();
-              GUI_GoToTime(currentPts);
+              if(!lastFrame)
+                  currentPts+=d;
+              if(!video_body->goToTimeVideo(currentPts))
+              {
+                  // If seek fails, we may crash in admPreview::samePicture()
+                  // due to _currentSegment and _currentPts going out of sync.
+                  // Rewind to get on firm ground again.
+                  A_Rewind();
+              }else
+              {
+                  admPreview::samePicture();
+                  GUI_setCurrentFrameAndTime();
+              }
             }
             break;
 
@@ -646,7 +687,14 @@ void HandleAction (Action action)
                 A_Resync();
                 A_Rewind();
 
-                if(currentPts<=video_body->getVideoDuration()) GUI_GoToTime(currentPts);
+                if(currentPts<=video_body->getVideoDuration())
+                    r = GUI_GoToTime(currentPts);
+                else
+                    r = false;
+
+                EditableAudioTrack *ed=video_body->getDefaultEditableAudioTrack();
+                if(ed && ed->edTrack)
+                    ed->edTrack->goToTime(r? currentPts : 0); // update audio segment
             }
         }
         break;
@@ -804,6 +852,20 @@ void HandleAction (Action action)
                         current-=b-a+c;
                 }
             }
+            if(current>=after) // the current frame is gone
+            {
+                // Can we go to the last keyframe before the cut?
+                current=after;
+                // current equal segment duration does not belong to this segment or
+                // to any segment, when it matches video duration, triggering rewind.
+                if(current && current != ADM_NO_PTS)
+                    current--;
+                if(!video_body->getPKFramePTS(&current))
+                { // nope
+                    A_Rewind();
+                    break;
+                }
+            }
             if(!video_body->goToTimeVideo(current))
             {
                 // If seek fails, we may crash in admPreview::samePicture()
@@ -1039,7 +1101,7 @@ void  updateLoaded (bool resetMarker)
     UI_setAudioTrackCount( video_body->getNumberOfActiveAudioTracks() );
     ADM_info(" conf updated \n");
     UI_setDecoderName(video_body->getVideoDecoderName());
-
+    UI_displayZoomLevel();
 }
 
 //___________________________________________
@@ -1309,23 +1371,22 @@ bool A_saveSession(void)
 bool A_checkSavedSession(bool load)
 {
     std::string where=LAST_SESSION_FILE;
-    bool exists=ADM_fileExist(where.c_str());
+    if(false == ADM_fileExist(where.c_str()))
+        return false;
     if(!load)
-        return exists;
-    bool r=false;
-    if(exists)
-    {
-        IScriptEngine *engine=getPythonScriptEngine();
-        if(engine)
-        {
-            ADM_info("Restoring the last editing state from %s\n",where.c_str());
-            r=A_parseScript(engine,where.c_str());
-            A_Resync();
-            if(!ADM_eraseFile(where.c_str()))
-                ADM_warning("Could not delete %s\n",where.c_str());
-        }
-    }
-    return r;
+        return true;
+    IScriptEngine *engine=getPythonScriptEngine();
+    if(!engine)
+        return false;
+    ADM_info("Restoring the last editing state from %s\n",where.c_str());
+    if(false == A_parseScript(engine,where.c_str()))
+        return false;
+    video_body->rewind();
+    admPreview::samePicture();
+    A_Resync();
+    if(false == ADM_eraseFile(where.c_str()))
+        ADM_warning("Could not delete %s\n",where.c_str());
+    return true;
 }
 /**
     Unpack all frames without displaying them to check for error
@@ -1428,6 +1489,26 @@ bool swap;
 
 }
 
+//
+extern int DIA_getHDRParams( uint32_t * toneMappingMethod, float * saturationAdjust, float * boostAdjust);
+//
+void    A_setHDRConfig( void )
+{
+    uint32_t method;
+    float saturation, boost;
+    stagedActionSuccess = 0;
+    if(!avifileinfo) return;
+
+    video_body->getHDRConfig(&method,&saturation,&boost);
+
+     if(DIA_getHDRParams( &method, &saturation,&boost))
+     {
+        video_body->setHDRConfig(method,saturation,boost);
+        stagedActionSuccess = 1;
+        return;
+     }
+}
+
 /**
 
 */
@@ -1695,17 +1776,16 @@ uint8_t GUI_close(void)
   if (avifileinfo)        // already opened ?
     {                // delete everything
       // if preview is on
-      if(getPreviewMode()!=ADM_PREVIEW_NONE)
+      if(ADM_PREVIEW_NONE != admPreview::getPreviewMode())
       {
         admPreview::stop();
-        setPreviewMode(ADM_PREVIEW_NONE);
+        admPreview::setPreviewMode(ADM_PREVIEW_NONE);
       }
       admPreview::setMainDimension(0, 0, ZOOM_1_1); // destroy preview
       UI_setNeedsResizingFlag(false);
-      uint32_t zero[6]={0};
-      UI_setVUMeter(zero);
-      if(video_body->canUndo())
-        A_saveSession();
+      int32_t inactiveVolume[8]={255,255,255,255,255,255,255,255};
+      UI_setVUMeter(inactiveVolume);
+      A_saveSession();
       delete avifileinfo;
       //delete wavinfo;
       avifileinfo = NULL;
@@ -1717,6 +1797,7 @@ uint8_t GUI_close(void)
 //      filterCleanUp ();
       UI_setTitle(NULL);
       UI_setDecoderName("XXXX");
+      UI_displayZoomLevel();
 
     A_ResetMarkers();
     ReSync();
@@ -1891,10 +1972,12 @@ void A_Rewind(void)
 {
                admPreview::stop();
                video_body->rewind();
+#if 0 /* Should not be needed anymore after ADM_Composer::DecodePictureUpToIntra has been fixed. */
                video_body->rewind(); // do it twice, for interlaced content it may fail the 1st time
+#endif
                admPreview::start();
                admPreview::samePicture();
-               admPreview::samePicture();
+               //admPreview::samePicture();
                GUI_setCurrentFrameAndTime();
 }
 void brokenAct(void)
