@@ -45,7 +45,7 @@
 //ADM_muxer               *ADM_MuxerSpawnFromIndex(int index);
 #include "ADM_muxerProto.h"
 extern ADM_audioStream  *audioCreateEncodingStream(EditableAudioTrack *ed,bool globalHeader,uint64_t startTime);
-extern ADM_audioStream  *audioCreateCopyStream(uint64_t startTime,int32_t shift,ADM_audioStream *input, bool needPerfectAudio);
+extern ADM_audioStream  *audioCreateCopyStream(uint64_t startTime, int32_t shift, ADM_edAudioTrack *input, bool needPerfectAudio);
 
 extern void A_Rewind(void);
 extern bool GUI_GoToTime(uint64_t time);
@@ -94,6 +94,8 @@ int A_Save(const char *name)
     uint64_t current=video_body->getCurrentFramePts();
     char *fullpath=ADM_PathCanonize(name);
     admSaver *save=new admSaver(fullpath);
+    delete [] fullpath;
+    fullpath=NULL;
     bool r=save->save();
     delete save;
     ADM_slaveSendResult(r);
@@ -113,9 +115,11 @@ int A_Save(const char *name)
             ADM_warning("Too much audio tracks, limiting to %d\n",ADM_MAX_AUDIO_STREAM);
             nbAudioTracks=ADM_MAX_AUDIO_STREAM;
         }
-        fileName=std::string(out);
-        logFileName=fileName;
-        logFileName+=std::string(".stats");
+        fileName = out;
+        logFileName = out;
+        logFileName += ".";
+        logFileName += videoEncoder6_GetCurrentEncoderName();
+        logFileName += ".stats";
         muxer=NULL;
         chain=NULL;
         audio=NULL;
@@ -188,6 +192,9 @@ bool abort=false;
         if(false==pass1->setup())
         {
             printf("[Save] setup failed for pass1 encoder\n");
+            GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Video"),QT_TRANSLATE_NOOP("adm","Cannot set up encoder for the first pass. "
+                "The configuration supplied to the encoder may be incompatible "
+                "or the encoder may depend on features unavailable on this system."));
             delete pass1;
             pass1=NULL;
             return NULL;
@@ -201,7 +208,9 @@ bool abort=false;
             return NULL;
         }
         muxer->createUI(videoDuration);
-        muxer->getEncoding()->setPhasis("Pass 1"); // don't make it translatable here, this is done in the encoding dialog
+        muxer->getEncoding()->setFileName(logFileName.c_str()); // just being honest, the muxer will update it later
+        muxer->getEncoding()->setLogFileName(logFileName.c_str()); // needed for cleanup afterwards
+        muxer->getEncoding()->setPhase(ADM_ENC_PHASE_FIRST_PASS,NULL);
 
         ADMBitstream bitstream;
         uint8_t *buffer=new uint8_t[BUFFER_SIZE];
@@ -209,24 +218,29 @@ bool abort=false;
         bitstream.bufferSize=BUFFER_SIZE;
         int nbFrames=0;
         uint32_t percent=0;
+#define GUI_REFRESH_DELAY 500 // in ms
+        uint32_t nextUpdate=GUI_REFRESH_DELAY;
         while(pass1->encode(&bitstream))
         {
             if(bitstream.pts!=ADM_NO_PTS)
             {
-                float f=100;
-                f/=videoDuration;
-                f*=bitstream.pts;
-                uint32_t p=(uint32_t)f;
-                if(percent<p)
-                    percent=p; // avoid progress bar going backwards
+                UI_purge();
                 if(!muxer->getEncoding()->isAlive())
                 {
                     abort=true;
                     break;
                 }
-                muxer->getEncoding()->setPercent(percent);
+                float f=100;
+                f/=videoDuration;
+                f*=bitstream.pts;
+                uint32_t p=(uint32_t)f;
+                if(percent<p)
+                {
+                    percent=p; // avoid progress bar going backwards
+                    muxer->getEncoding()->setPercent(percent);
+                }
                 uint32_t elapsed=ticktock.getElapsedMS();
-                if(percent>=1)
+                if(percent>=1 && elapsed>nextUpdate)
                 {
                     double totalTime=(100*elapsed)/percent;
                     double remaining=totalTime-elapsed;
@@ -234,6 +248,8 @@ bool abort=false;
                         remaining=0;
                     uint32_t remainingMs=(uint32_t)remaining;
                     muxer->getEncoding()->setRemainingTimeMS(remainingMs);
+                    nextUpdate=elapsed+GUI_REFRESH_DELAY;
+#undef GUI_REFRESH_DELAY
                 }
             }
             nbFrames++;
@@ -289,6 +305,7 @@ ADM_videoStreamCopy *admSaver::dealWithH26x(bool isAnnexB)
             default:
             case 0:  // Both source and target are mp4 , nothing to do
                 ADM_info("Input and output are mp4 style, nothing to do\n");
+                copy=new ADM_videoStreamCopySeiInjector(markerA,markerB);
                 break;
             case 1:  // source is mp4, target is annexB
                 ADM_info("Input is probably MP4 bitstream, target is annexB\n");
@@ -375,18 +392,14 @@ ADM_videoStream *admSaver::setupVideo(void)
         }
         if(encoder->setup()==false)
         {
-            GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Video"),QT_TRANSLATE_NOOP("adm","Cannot setup codec. Bitrate too low?"));
+            GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Video"),QT_TRANSLATE_NOOP("adm","Cannot set up encoder. "
+                "The configuration supplied to the encoder may be incompatible "
+                "or the encoder may depend on features unavailable on this system."));
             delete encoder;
             encoder=NULL;
             return NULL;
         }
         video= new ADM_videoStreamProcess(encoder);
-        if(!video)
-        {
-                GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Video"),QT_TRANSLATE_NOOP("adm","Cannot create encoder"));
-                delete encoder;
-                return NULL;
-        }
     }  
     return video;
 }
@@ -447,12 +460,12 @@ bool admSaver::save(void)
         bool ask=true;
         switch(chk)
         {
-            case ADM_EDITOR_CUT_POINT_NON_IDR:
+            case ADM_EDITOR_CUT_POINT_NON_KEY:
                 alert=QT_TRANSLATE_NOOP("adm","The video is in copy mode but the cut points are not on keyframes.\n"
                     "The video will be saved but there will be corruption at cut point(s).\n"
                     "Do you want to continue anyway ?");
                 break;
-            case ADM_EDITOR_CUT_POINT_RECOVERY:
+            case ADM_EDITOR_CUT_POINT_BAD_POC:
                 alert=QT_TRANSLATE_NOOP("adm","This video uses non-IDR recovery points instead of IDR as keyframes. "
                     "Picture reordering information in the video stream is not reset at non-IDR frames. "
                     "The choice of cut points may result in playback interruption "
@@ -498,24 +511,24 @@ bool admSaver::save(void)
         }
     if(false==setupAudio())
     {
-        if(video) delete video;
-        if(muxer) delete muxer;
+        delete video;
+        video=NULL;
+        delete muxer;
         muxer=NULL;
         return false;
     }
-   
-   
     if(!muxer->open(fileName.c_str(),video,nbAudioTracks,audioAccess))
     {
         GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Muxer"),QT_TRANSLATE_NOOP("adm","Cannot open "));
     }else
     {
         ret=muxer->save();
-        muxer->close();
+        if(false==muxer->close())
+            ret=false;
     }
-
-    if(video)
-        delete video;
+    delete muxer;
+    muxer=NULL;
+    delete video;
     video=NULL;
     for(int i=0;i<nbAudioTracks;i++)
     {
