@@ -23,16 +23,24 @@ void ADM_QCanvas::changeSize(uint32_t w,uint32_t h)
 {
 	_w=w;
 	_h=h;
+#if QT_VERSION >= QT_VERSION_CHECK(5,5,0)
+	_w=(_w*devicePixelRatioF() + 0.5);
+	_h=(_h*devicePixelRatioF() + 0.5);
+#endif
+	_l=ADM_IMAGE_ALIGN(_w*4);
 	dataBuffer=NULL;
 	resize(w,h);
 }
 
+void ADM_QCanvas::getDisplaySize(uint32_t *w,uint32_t *h)
+{
+	*w=_w;
+	*h=_h;
+}
+
 ADM_QCanvas::ADM_QCanvas(QWidget *z,uint32_t w,uint32_t h) : QWidget(z) 
 {
-	_w=w;
-	_h=h;
-	dataBuffer=NULL;
-	resize(w,h);
+	changeSize(w,h);
 }
 
 ADM_QCanvas::~ADM_QCanvas() 
@@ -48,7 +56,10 @@ void ADM_QCanvas::paintEvent(QPaintEvent *ev)
 	if(!dataBuffer)
 		return ;
 
-	QImage image(dataBuffer,_w,_h,QImage::Format_RGB32);
+	QImage image(dataBuffer,_w,_h,_l,QImage::Format_RGB32);
+#if QT_VERSION >= QT_VERSION_CHECK(5,5,0)
+	image.setDevicePixelRatio(devicePixelRatioF());
+#endif
 	QPainter painter(this);
 	painter.drawImage(QPoint(0,0),image);
 	painter.end();
