@@ -12,6 +12,7 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <QDialog>
 #include <QDialogButtonBox>
 #include <QGridLayout>
 #include <QSpacerItem>
@@ -77,10 +78,8 @@ void * qt4DiaFactoryPrepare(const char *title,uint32_t nb,diaElem **elems)
 {
   factoryCookie *cookie=new factoryCookie(title);
 
-  ADM_assert(title);
-  ADM_assert(nb);
-  ADM_assert(elems);
-  
+    if(!nb) return cookie;
+
     int currentLayout = 0;
     int  v=0;
     for(int i=0;i<nb;i++)
@@ -124,6 +123,9 @@ void * qt4DiaFactoryPrepare(const char *title,uint32_t nb,diaElem **elems)
  */
 bool  qt4DiaFactoryFinish(void *finish)
 {
+    if(!finish)
+        return false;
+
   bool  r=false;
   factoryCookie *cookie=(factoryCookie *)  finish;
   QSpacerItem *spacer = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Fixed);
@@ -183,11 +185,7 @@ const char *shortkey(const char *in)
 void  *qt4DiaFactoryTabsPrepare(const char *title,uint32_t nb,diaElemTabs **tabs)
 {
     factoryCookie *cookie=new factoryCookie(title);
-  
-    ADM_assert(title);
-    ADM_assert(nb);
-    ADM_assert(tabs);
-  
+
     cookie->layout  = new QGridLayout();
     cookie->tabWidget = new QTabWidget();
     
@@ -208,6 +206,8 @@ void  *qt4DiaFactoryTabsPrepare(const char *title,uint32_t nb,diaElemTabs **tabs
   */ 
 bool qt4DiaFactoryTabsFinish(void *f)
 {
+    if(!f) return false;
+
     bool r=false;
     factoryCookie *cookie=(factoryCookie *)f;
     
@@ -217,9 +217,9 @@ bool qt4DiaFactoryTabsFinish(void *f)
     buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
     QSpacerItem *spacer = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Fixed);
     cookie->vboxlayout->addLayout(cookie->layout);
-    cookie->vboxlayout->addWidget(cookie->tabWidget,0,0);
+    cookie->vboxlayout->addWidget(cookie->tabWidget,0);
     cookie->vboxlayout->addItem(spacer);
-    cookie->vboxlayout->addWidget(buttonBox,1,0);
+    cookie->vboxlayout->addWidget(buttonBox,1);
 
      cookie->dialog->setLayout(cookie->vboxlayout);
 
@@ -342,6 +342,8 @@ extern CREATE_BAR_T         qt4CreateBar;
 extern DELETE_DIA_ELEM_T    qt4DestroyBar;
 extern CREATE_FLOAT_T       qt4CreateFloat;
 extern DELETE_DIA_ELEM_T    qt4DestroyFloat;
+extern CREATE_FLOAT_RESETTABLE_T qt4CreateFloatResettable;
+extern DELETE_DIA_ELEM_T    qt4DestroyFloatResettable;
 extern CREATE_FRAME_T       qt4CreateFrame;
 extern DELETE_DIA_ELEM_T    qt4DestroyFrame;
 extern CREATE_HEX_T         qt4CreateHex;
@@ -403,6 +405,9 @@ static FactoryDescriptor Qt4FactoryDescriptor=
         // Float
         &qt4CreateFloat,
         &qt4DestroyFloat,
+        // Float resettable
+        &qt4CreateFloatResettable,
+        &qt4DestroyFloatResettable,
         // Integer
         &qt4CreateInteger,
         &qt4DestroyInteger,
