@@ -42,6 +42,7 @@ using std::string;
 #include "ADM_videoFilterApi.h"
 #include "ADM_videoFilters.h"
 #include "ADM_toolkitQt.h"
+#include "ADM_preview.h"
 
 #include "prefs.h"
 #include "ADM_edScriptGenerator.h"
@@ -190,6 +191,11 @@ void FilterItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &op
         fg = pal.color(QPalette::Text);
         bg = pal.color(QPalette::Base);
     }
+
+    bool disabled = index.data(DisabledRole).toBool();
+    if (disabled)
+        fg = pal.color(QPalette::Disabled, QPalette::WindowText);
+
     painter->fillRect(option.rect, bg);
     pen.setColor(fg);
     painter->setPen(pen);
@@ -197,6 +203,7 @@ void FilterItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &op
     QString filterNameText = index.data(FilterNameRole).toString();
     QString descText = index.data(DescriptionRole).toString();
 
+
     QFont filterNameFont = QApplication::font();
     QFont descFont = QApplication::font();
     int fontSize = descFont.pointSize();
@@ -267,15 +274,30 @@ void filtermainWindow::preview(bool b)
      ADM_info("Rank : %d\n",itag);
      ADM_coreVideoFilter     *filter=ADM_vf_getInstance(itag);
      ADM_assert(filter);
+    bool                      enabled = ADM_vf_getEnabled(itag);
+    uint32_t                  instanceTag=ADM_vf_getTag(itag);
+    const char               *name= ADM_vf_getDisplayNameFromTag(instanceTag);
     if (previewDialog)
     {
             delete previewDialog;
             previewDialog=NULL;
     }
-    previewDialog = new Ui_seekablePreviewWindow(this, filter, 0);
+
+    QString title = QT_TRANSLATE_NOOP("qmainfilter","Preview");
+    title += QString(" / ");
+    if (!enabled)
+        title += QT_TRANSLATE_NOOP("qmainfilter","DISABLED ");
+    title += QString::fromUtf8(name);
+
+    previewDialog = new Ui_seekablePreviewWindow(this, filter);
+    previewDialog->setWindowTitle(title);
     previewDialog->setModal(true);
     connect(previewDialog, SIGNAL(accepted()), this, SLOT(closePreview()));
+    connect(previewDialog, SIGNAL(rejected()), this, SLOT(closePreview()));
+    qtRegisterDialog(previewDialog);
     previewDialog->show();
+    previewDialog->seekablePreview->adjustCanvasPosition();
+    previewDialog->canvas->parentWidget()->setMinimumSize(30,30); // allow resizing after the dialog has settled
 }
 /**
  * \fn closePreview
@@ -284,8 +306,13 @@ void filtermainWindow::closePreview()
 {
     if (previewDialog)
     {
-        delete previewDialog;
-        previewDialog = NULL;
+        if(previewDialog->seekablePreview)
+        {
+            previewDialog->blockSignals(true); // we crash when this slot is invoked more than once
+            previewDialog->seekablePreview->play(false);
+            previewDialog->seekablePreview->goToTime(0);
+        }
+        qtUnregisterDialog(previewDialog);
     }
 }
 
@@ -334,7 +361,19 @@ void filtermainWindow::add( bool b)
         }
    }
 }
-
+/**
+    \fn duplicate
+*/
+void filtermainWindow::duplicate()
+{
+    int filterIndex=getTagForActiveSelection();
+    if(-1==filterIndex)
+        return;
+    ADM_vf_duplicateFilterAtIndex(video_body, filterIndex);
+    buildActiveFilterList ();
+    if(nb_active_filter)
+        setSelected(nb_active_filter-1);
+}
 /**
     \fn removeAction
     \brief active filter context menu item
@@ -349,6 +388,8 @@ void filtermainWindow::removeAction(void)
 */
 void filtermainWindow::remove( bool b)
 {
+    if(!nb_active_filter)
+        return;
     int itag=getTagForActiveSelection();
     if(-1==itag)
         return;
@@ -357,7 +398,10 @@ void filtermainWindow::remove( bool b)
     buildActiveFilterList ();
     if(nb_active_filter)
     {
-          setSelected(nb_active_filter-1);
+        if (itag < nb_active_filter)
+            setSelected(itag);
+        else
+            setSelected(nb_active_filter-1);
     }
 }
 /**
@@ -369,6 +413,14 @@ void filtermainWindow::configureAction(void)
     configure(true);
 }
 
+/**
+    \fn duplicateAction
+    \brief active filter context menu item
+*/
+void filtermainWindow::duplicateAction(void)
+{
+    duplicate();
+}
 /**
  * \fn getTagForActiveSelection
  */
@@ -409,10 +461,28 @@ void filtermainWindow::moveUp( )
     int itag=getTagForActiveSelection();
     if(-1==itag)
         return;
+    if(0==itag) // already at the top
+        return;
     ADM_vf_moveFilterUp(itag);
     buildActiveFilterList ();
     setSelected(itag-1);
 }
+
+/**
+ * 
+ */
+void filtermainWindow::togglePartial()
+{
+    int filterIndex=getTagForActiveSelection();
+    if(-1==filterIndex)
+        return;
+    uint32_t tag=ADM_vf_getTag(filterIndex);
+    if (tag == VF_PARTIAL_FILTER)
+        absolvePartial();
+    else
+        makePartial();
+        
+}
 /**
  * 
  */
@@ -437,11 +507,50 @@ void filtermainWindow::makePartial()
         buildActiveFilterList ();
         setSelected(filterIndex);
     }else
-      {
+    {
         ADM_info("CANCEL \n");
-      }
+    }
+}
+/**
+ * 
+ */
+void filtermainWindow::absolvePartial()
+{
+    ADM_info("Absolve Partial\n");
+    int filterIndex=getTagForActiveSelection();
+    if(-1==filterIndex)
+        return;
+    ADM_info("Absolve Partial %d\n",filterIndex);
+    // Check we can partialize it...
+    uint32_t tag=ADM_vf_getTag(filterIndex);
+    if(tag!=VF_PARTIAL_FILTER)
+    {
+        GUI_Error_HIG(QT_TRANSLATE_NOOP("qmainfilter","Partial"),QT_TRANSLATE_NOOP("qmainfilter","This filter is not partial"));
+        return;
+    }
+    // Get info about that filter..
+    if(ADM_vf_absolvePartialized(filterIndex))
+    {
+        ADM_info("Absolving partialized done, rebuilding \n");
+        buildActiveFilterList ();
+        setSelected(filterIndex);
+    }else
+    {
+        ADM_info("CANCEL \n");
+    }
+}
+/**
+ * 
+ */
+void filtermainWindow::toggleEnabled()
+{
+    int filterIndex=getTagForActiveSelection();
+    if(-1==filterIndex)
+        return;
+    ADM_vf_toggleFilterEnabledAtIndex(filterIndex);
+    buildActiveFilterList ();
+    setSelected(filterIndex);
 }
-
 /**
         \fn     down( bool b)
         \brief  Move selected filter one place down
@@ -496,8 +605,8 @@ void filtermainWindow::displayFamily(uint32_t family)
         uint32_t major,minor,patch;
         ADM_vf_getFilterInfo((VF_CATEGORY)family,i,&name, &desc,&major,&minor,&patch);
 
-        QString s1 = QString(name);
-        QString s2 = QString(desc);
+        QString s1 = QString::fromUtf8(name);
+        QString s2 = QString::fromUtf8(desc);
 
         QListWidgetItem *item;
         item=new QListWidgetItem(NULL,availableList,ALL_FILTER_BASE+i+family*100);
@@ -549,16 +658,18 @@ void filtermainWindow::buildActiveFilterList(void)
     {
         uint32_t                instanceTag=ADM_vf_getTag(i);
         ADM_coreVideoFilter     *instance=ADM_vf_getInstance(i);
+        bool                    enabled = ADM_vf_getEnabled(i);
         const char *name= ADM_vf_getDisplayNameFromTag(instanceTag);
         const char *conf=instance->getConfiguration();
         printf("%d %s\n",i,name);
 
-        QString s1 = QString(name);
-        QString s2 = QString(conf);
+        QString s1 = QString::fromUtf8(name);
+        QString s2 = QString::fromUtf8(conf);
 
         QListWidgetItem *item=new QListWidgetItem(NULL,activeList,ACTIVE_FILTER_BASE+i);
         item->setData(FilterItemDelegate::FilterNameRole, s1);
         item->setData(FilterItemDelegate::DescriptionRole, s2);
+        item->setData(FilterItemDelegate::DisabledRole, !enabled);
         printf("Active item :%p\n",item);
         item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled);
         activeList->addItem(item);
@@ -570,28 +681,59 @@ void filtermainWindow::buildActiveFilterList(void)
 */
 void filtermainWindow::activeListContextMenu(const QPoint &pos)
 {
-    QMenu *cm=new QMenu();
+    if(!nb_active_filter)
+        return;
+    if(!activeList->currentItem())
+        return;
 
-    QAction *up = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Move up")),this);
-    QAction *down = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Move down")),this);
-    QAction *configure = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Configure")),this);
-    QAction *remove = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Remove")),this);
-    QAction *partial = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Make partial")),this);
+    QMenu *cm=new QMenu(this);
+
+    QAction *up = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Move up")),cm);
+    QAction *down = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Move down")),cm);
+    QAction *configure = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Configure")),cm);
+    QAction *duplicate = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Duplicate")),cm);
+    QAction *remove = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Remove")),cm);
+    QAction *partial = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Make partial")),cm);
+    QAction *enabled = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Enable/Disable")),cm);
+
+    up->setShortcut(shortcutMoveUp);
+    down->setShortcut(shortcutMoveDown);
+    configure->setShortcut(shortcutConfigure);
+    duplicate->setShortcut(shortcutDuplicate);
+    remove->setShortcut(shortcutRemove);
+    partial->setShortcut(shortcutTogglePartial);
+    enabled->setShortcut(shortcutToggleEnabled);
+
+#if defined(__APPLE__) && QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+    up->setShortcutVisibleInContextMenu(true);
+    down->setShortcutVisibleInContextMenu(true);
+    configure->setShortcutVisibleInContextMenu(true);
+    duplicate->setShortcutVisibleInContextMenu(true);
+    remove->setShortcutVisibleInContextMenu(true);
+    partial->setShortcutVisibleInContextMenu(true);
+    enabled->setShortcutVisibleInContextMenu(true);
+#endif
 
     cm->addAction(up);
     cm->addAction(down);
     cm->addAction(configure);
+    cm->addAction(duplicate);
     cm->addAction(remove);
     cm->addAction(partial);
+    cm->addAction(enabled);
 
     connect(up,SIGNAL(triggered()),this,SLOT(moveUp()));
     connect(down,SIGNAL(triggered()),this,SLOT(moveDown()));
     connect(configure,SIGNAL(triggered()),this,SLOT(configureAction()));
+    connect(duplicate,SIGNAL(triggered()),this,SLOT(duplicateAction()));
     connect(remove,SIGNAL(triggered()),this,SLOT(removeAction()));
-    connect(partial,SIGNAL(triggered()),this,SLOT(makePartial()));
+    connect(partial,SIGNAL(triggered()),this,SLOT(togglePartial()));
+    connect(enabled,SIGNAL(triggered()),this,SLOT(toggleEnabled()));
 
     updateContextMenu(cm);
     cm->exec(activeList->viewport()->mapToGlobal(pos));
+    delete cm;
+    cm = NULL;
 }
 
 /**
@@ -607,6 +749,7 @@ void filtermainWindow::updateContextMenu(QMenu *contextMenu)
     bool canMoveUp=true;
     bool canMoveDown=true;
     bool canPartialize=true;
+    bool partialized=false;
     QListWidgetItem *item=activeList->currentItem();
     if(!item)
         return;
@@ -616,6 +759,7 @@ void filtermainWindow::updateContextMenu(QMenu *contextMenu)
     itag -= ACTIVE_FILTER_BASE;
     uint32_t tag=ADM_vf_getTag(itag);
     canPartialize=ADM_vf_canBePartialized(tag);
+    partialized=(tag==VF_PARTIAL_FILTER);
 
     int row=item->listWidget()->row(item);
     if(!row)
@@ -623,9 +767,29 @@ void filtermainWindow::updateContextMenu(QMenu *contextMenu)
     if(row==nb_active_filter-1)
         canMoveDown=false;
 
-    contextMenu->actions().at(0)->setEnabled(canMoveUp);
-    contextMenu->actions().at(1)->setEnabled(canMoveDown);
-    contextMenu->actions().at(4)->setEnabled(canPartialize);
+    const char *textEnable = ADM_vf_getEnabled(itag) ?
+        QT_TRANSLATE_NOOP("qmainfilter","Disable") :
+        QT_TRANSLATE_NOOP("qmainfilter","Enable");
+    
+    const char *textPartial = partialized ? 
+        QT_TRANSLATE_NOOP("qmainfilter","Make global") :
+        QT_TRANSLATE_NOOP("qmainfilter","Make partial");
+
+    for(int i = 0; i < contextMenu->actions().size(); i++)
+    {
+        QAction *a = contextMenu->actions().at(i);
+
+        if(a->shortcut() == shortcutTogglePartial)
+            a->setText(QString::fromUtf8(textPartial));
+
+#define MATCHME(x,y) if(a->shortcut() == shortcut##x) { a->setEnabled(y); continue; }
+        MATCHME(MoveUp,canMoveUp)
+        MATCHME(MoveDown,canMoveDown)
+        MATCHME(TogglePartial,canPartialize||partialized)
+
+        if(a->shortcut() == shortcutToggleEnabled)
+            a->setText(QString::fromUtf8(textEnable));
+    }
 }
 
 /**
@@ -650,10 +814,15 @@ filtermainWindow::filtermainWindow(QWidget* parent) : QDialog(parent)
     availableList->setItemDelegate(new FilterItemDelegate(availableList));
     activeList->setItemDelegate(new FilterItemDelegate(activeList));
 
+    availableList->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
+    activeList->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
+
     connect(ui.listFilterCategory,SIGNAL(itemDoubleClicked(QListWidgetItem *)),
                 this,SLOT(filterFamilyClick(QListWidgetItem *)));
     connect(ui.listFilterCategory,SIGNAL(itemClicked(QListWidgetItem *)),
                 this,SLOT(filterFamilyClick(QListWidgetItem *)));
+    connect(ui.listFilterCategory,SIGNAL(currentRowChanged(int)),
+                this,SLOT(filterFamilyClick(int)));
 
     connect(activeList,SIGNAL(itemDoubleClicked(QListWidgetItem *)),this,SLOT(activeDoubleClick(QListWidgetItem *)));
     connect(availableList,SIGNAL(itemDoubleClicked(QListWidgetItem *)),this,SLOT(allDoubleClick(QListWidgetItem *)));
@@ -661,6 +830,7 @@ filtermainWindow::filtermainWindow(QWidget* parent) : QDialog(parent)
     connect(ui.buttonClose, SIGNAL(clicked(bool)), this, SLOT(accept()));
     connect(ui.pushButtonPreview, SIGNAL(clicked(bool)), this, SLOT(preview(bool)));
 
+    ADM_vf_rebuildBridge(video_body);
     displayFamily(0);
     buildActiveFilterList();
     setSelected(nb_active_filter - 1);
@@ -676,10 +846,86 @@ filtermainWindow::filtermainWindow(QWidget* parent) : QDialog(parent)
     availableList->addAction(add );
     connect(add,SIGNAL(triggered(bool )),this,SLOT(addSlot()));
 
+    QAction *rem = new QAction(QString(QT_TRANSLATE_NOOP("qmainfilter","Remove")),this);
+    int keycode;
+#ifdef __APPLE__
+    keycode = Qt::Key_Backspace;
+#else
+    keycode = Qt::Key_Delete;
+#endif
+    shortcutRemove = QKeySequence(keycode);
+    bool alt = false;
+    prefs->get(KEYBOARD_SHORTCUTS_USE_ALTERNATE_KBD_SHORTCUTS,&alt);
+    if(alt)
+    {
+        std::string sc;
+        prefs->get(KEYBOARD_SHORTCUTS_ALT_DELETE,sc);
+        if(sc.size())
+        {
+            QString qs = QString::fromUtf8(sc.c_str());
+            shortcutRemove = QKeySequence::fromString(qs);
+        }
+    }
+    rem->setShortcut(shortcutRemove);
+    addAction(rem);
+    connect(rem,SIGNAL(triggered(bool)),this,SLOT(remove(bool)));
+
+    // TODO make configurable
+    shortcutMoveUp = QKeySequence(Qt::ShiftModifier | Qt::Key_Up);
+    shortcutMoveDown = QKeySequence(Qt::ShiftModifier | Qt::Key_Down);
+    shortcutConfigure = QKeySequence(Qt::Key_Return);
+    shortcutDuplicate = QKeySequence(Qt::ControlModifier | Qt::Key_D);
+    shortcutTogglePartial = QKeySequence(Qt::ShiftModifier | Qt::Key_P);
+    shortcutToggleEnabled = QKeySequence(Qt::ShiftModifier | Qt::Key_D);
+
+    QAction *movup = new QAction(this);
+    movup->setShortcut(shortcutMoveUp);
+    addAction(movup);
+    connect(movup,SIGNAL(triggered()),this,SLOT(moveUp()));
+
+    QAction *movdw = new QAction(this);
+    movdw->setShortcut(shortcutMoveDown);
+    addAction(movdw);
+    connect(movdw,SIGNAL(triggered()),this,SLOT(moveDown()));
+
+    QAction *dupl = new QAction(this);
+    dupl->setShortcut(shortcutDuplicate);
+    addAction(dupl);
+    connect(dupl,SIGNAL(triggered()),this,SLOT(duplicate()));
+
+    QAction *mkpartl = new QAction(this);
+    mkpartl->setShortcut(shortcutTogglePartial);
+    addAction(mkpartl);
+    connect(mkpartl,SIGNAL(triggered()),this,SLOT(togglePartial()));
+
+    QAction *tglenbl = new QAction(this);
+    tglenbl->setShortcut(shortcutToggleEnabled);
+    addAction(tglenbl);
+    connect(tglenbl,SIGNAL(triggered()),this,SLOT(toggleEnabled()));
+
+
     activeList->setContextMenuPolicy(Qt::CustomContextMenu);
     connect(activeList,SIGNAL(customContextMenuRequested(const QPoint &)),this,SLOT(activeListContextMenu(const QPoint &)));
 
     this->installEventFilter(this);
+    ui.pushButtonPreview->installEventFilter(this);
+    originalTime = admPreview::getCurrentPts();
+
+#if 1
+#define HINT_LENGTH 256
+    char hint[HINT_LENGTH];
+    hint[0] = '\0';
+    QKeySequence acc(Qt::ControlModifier | Qt::Key_Return);
+    snprintf(hint,
+        HINT_LENGTH,
+        QT_TRANSLATE_NOOP("qmainfilter","Press %s to accept the dialog"),
+        acc.toString(QKeySequence::NativeText).toUtf8().constData());
+    hint[HINT_LENGTH-1] = '\0';
+    ui.labelAcceptHint->setText(QString::fromUtf8(hint));
+#undef HINT_LENGTH
+#else
+    ui.labelAcceptHint->setVisible(false);
+#endif
 }
 
 /**
@@ -690,7 +936,7 @@ filtermainWindow::~filtermainWindow()
     if(previewDialog) 
         delete previewDialog;
     previewDialog=NULL;
-
+    admPreview::seekToTime(originalTime);
 }
 
 /**
@@ -703,32 +949,50 @@ bool filtermainWindow::eventFilter(QObject* watched, QEvent* event)
     if(event->type() == QEvent::KeyPress)
     {
         keyEvent = (QKeyEvent*)event;
-        switch (keyEvent->key())
+        if(keyEvent->key() == Qt::Key_Return)
         {
-#ifndef __APPLE__
-            case Qt::Key_Delete:
-#else
-            case Qt::Key_Backspace:
-#endif
-                remove(true);
+            if(keyEvent->modifiers() & Qt::ControlModifier)
+            {
+                accept();
                 return true;
-            case Qt::Key_Return:
-                if(keyEvent->modifiers() & Qt::ControlModifier)
-                {
-                    accept();
-                    return true;
-                }
-                if(ui.listFilterCategory->hasFocus())
-                    filterFamilyClick(ui.listFilterCategory->currentRow());
-                else if(availableList->hasFocus())
-                    add(true);
-                else if(activeList->hasFocus())
-                    configure(true);
-                else
-                    accept();
+            }
+            if(watched == ui.pushButtonPreview)
+                preview(true);
+            else if(ui.listFilterCategory->hasFocus())
+                filterFamilyClick(ui.listFilterCategory->currentRow());
+            else if(availableList->hasFocus())
+                add(true);
+            else if(activeList->hasFocus())
+                configure(true);
+            else
+                accept();
+            return true;
+        }
+        if(keyEvent->key() == Qt::Key_Left)
+        {
+            if(ui.listWidgetAvailable->hasFocus())
+            {
+                ui.listFilterCategory->setFocus();
                 return true;
-            default:
-                return false;
+            }
+            if(ui.listWidgetActive->hasFocus())
+            {
+                ui.listWidgetAvailable->setFocus();
+                return true;
+            }
+        }
+        if(keyEvent->key() == Qt::Key_Right)
+        {
+            if(ui.listFilterCategory->hasFocus())
+            {
+                ui.listWidgetAvailable->setFocus();
+                return true;
+            }
+            if(ui.listWidgetAvailable->hasFocus())
+            {
+                ui.listWidgetActive->setFocus();
+                return true;
+            }
         }
     }
     return QObject::eventFilter(watched, event);
@@ -768,6 +1032,8 @@ void filtermainWindow::rowsMovedSlot(void)
             ADM_vf_moveFilterDown(itag+i);
         }
     }
+    buildActiveFilterList();
+    setSelected(moved);
 }
 
 /*******************************************************/
