@@ -56,7 +56,7 @@ private:
 
 public:
     FilterItemDelegate(QWidget *parent = 0);
-    enum datarole { FilterNameRole = Qt::UserRole, DescriptionRole };
+    enum datarole { FilterNameRole = Qt::UserRole, DescriptionRole, DisabledRole };
     void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
     QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const;
     static int padding;
@@ -85,7 +85,11 @@ public slots:
     void add(bool b);
     void moveUp();
     void moveDown();
+    void togglePartial();
     void makePartial();
+    void absolvePartial();
+    void toggleEnabled();
+    void duplicate();
     void remove(bool b);
     void configure(bool b);
     void activeDoubleClick( QListWidgetItem  *item);
@@ -98,6 +102,7 @@ public slots:
     void addSlot(void);
     void removeAction(void);
     void configureAction(void);
+    void duplicateAction(void);
     void activeListContextMenu(const QPoint &pos);
     // reorder active filters using drag-n-drop
     void rowsMovedSlot(void);
@@ -110,5 +115,14 @@ private:
     void updateContextMenu(QMenu *contextMenu);
     bool eventFilter(QObject* watched, QEvent* event);
     uint64_t originalTime;
+private:
+    QKeySequence shortcutMoveUp;
+    QKeySequence shortcutMoveDown;
+    QKeySequence shortcutConfigure;
+    QKeySequence shortcutDuplicate;
+    QKeySequence shortcutRemove;
+    QKeySequence shortcutTogglePartial;
+    QKeySequence shortcutToggleEnabled;
+
 };
 
