@@ -22,7 +22,7 @@ ThumbSlider::ThumbSlider(QWidget *parent) : QAbstractSlider(parent)
 	setMinimum(-100);
 	setValue(0);
 
-	count = lock = 0;
+	timerId = count = lock = pos = stopping = 0;
 }
 
 void ThumbSlider::timerEvent(QTimerEvent *event)
@@ -51,6 +51,16 @@ void ThumbSlider::timerEvent(QTimerEvent *event)
 	lock--;
 }
 
+void ThumbSlider::stop(void)
+{
+	if (timerId)
+		killTimer(timerId);
+	setValue(0);
+	timerId = count = pos = 0;
+
+	triggerAction(SliderNoAction);
+}
+
 void ThumbSlider::paintEvent(QPaintEvent *event)
 {
 	QPainter painter;
@@ -67,7 +77,8 @@ void ThumbSlider::paintEvent(QPaintEvent *event)
 
 void ThumbSlider::mousePressEvent(QMouseEvent *event)
 {
-	if (event->buttons() && Qt::LeftButton)
+	if (stopping) return;
+	if (event->buttons() & Qt::LeftButton)
 	{
 		int value = QStyle::sliderValueFromPosition(minimum(), maximum(), event->x(), width(), false);
 
@@ -79,7 +90,8 @@ void ThumbSlider::mousePressEvent(QMouseEvent *event)
 
 void ThumbSlider::mouseMoveEvent(QMouseEvent *event)
 {
-	if (event->buttons() && Qt::LeftButton)
+	if (stopping) return;
+	if (event->buttons() & Qt::LeftButton)
 	{
 		int value = QStyle::sliderValueFromPosition(minimum(), maximum(), event->x(), width(), false);
 
@@ -90,11 +102,50 @@ void ThumbSlider::mouseMoveEvent(QMouseEvent *event)
 
 void ThumbSlider::mouseReleaseEvent(QMouseEvent *event)
 {
-	killTimer(timerId);
-	setValue(0);
-	count = 0;
+	if (event->button() & Qt::LeftButton)
+	{
+		stop();
+		stopping = 0;
+		emit valueEmitted(0);
+	}
+}
 
-	triggerAction(SliderNoAction);
+void ThumbSlider::wheelEvent(QWheelEvent *event)
+{
+	int value;
+#if QT_VERSION < QT_VERSION_CHECK(5,0,0)
+	value = event->delta();
+#else
+	value = event->angleDelta().ry();
+#endif
+	value /= 12;
+	pos += value;
+	setSliderPosition(pos);
+	if (pos)
+	{
+		if (!timerId)
+			timerId = startTimer(20);
+	}
+	else
+	{
+		if (timerId)
+			killTimer(timerId);
+		timerId = count = stopping = 0;
+		emit valueEmitted(0);
+	}
+
+	if (value)
+		triggerAction(SliderMove);
+	else
+		triggerAction(SliderNoAction);
+}
+
+void ThumbSlider::reset(void)
+{
+	stopping = 1;
+	stop();
+	setSliderPosition(0);
+	emit valueEmitted(0);
 }
 
 void ThumbSlider::drawBackground(QPainter *painter)
