@@ -8,10 +8,7 @@
 //
 //
 
-#include "config.h"
 #include "ADM_inttype.h"
-#include <QApplication>
-#include <QDesktopWidget>
 
 void UI_setAProcessToggleStatus( uint8_t status ) {}
 void UI_setVProcessToggleStatus( uint8_t status ) {}
