@@ -83,6 +83,19 @@ extern "C"
       	 myCpuCaps |= ADM_CPUCAP_SSE3;
        if (ecx & 0x00000200 )
       	 myCpuCaps |= ADM_CPUCAP_SSSE3;
+       if (ecx & (1<<19) )
+      	 myCpuCaps |= ADM_CPUCAP_SSE4;
+       if (ecx & (1<<20) )
+      	 myCpuCaps |= ADM_CPUCAP_SSE42;
+       if (ecx & (1<<28) )
+      	 myCpuCaps |= ADM_CPUCAP_AVX;
+   }
+   if(max_std_level >= 7)
+   {
+       ecx=0;
+       adm_cpu_cpuid(7, &eax, &ebx, &ecx, &edx);
+       if (ebx & (1<<5) )
+      	 myCpuCaps |= ADM_CPUCAP_AVX2;
    }
 
    adm_cpu_cpuid(0x80000000,& max_ext_level,& ebx, &ecx,& edx);
@@ -104,14 +117,18 @@ skipIt:
   #define CHECK(x) if (myCpuCaps & ADM_CPUCAP_##x) { printf("\t\t"#x" detected"); \
                        if (!(myCpuMask & ADM_CPUCAP_##x)) printf(", but disabled"); \
                        printf("\n"); }
-      CHECK(MMX);
-      CHECK(3DNOW);
-      CHECK(3DNOWEXT);
-      CHECK(MMXEXT);
-      CHECK(SSE);
-      CHECK(SSE2);
-      CHECK(SSE3);
-      CHECK(SSSE3);
+    CHECK(MMX)
+    CHECK(3DNOW)
+    CHECK(3DNOWEXT)
+    CHECK(MMXEXT)
+    CHECK(SSE)
+    CHECK(SSE2)
+    CHECK(SSE3)
+    CHECK(SSSE3)
+    CHECK(SSE4)
+    CHECK(SSE42)
+    CHECK(AVX)
+    CHECK(AVX2)
 #endif // X86
     ADM_info("[CpuCaps] End of CPU capabilities check (cpuCaps: 0x%08x, cpuMask: 0x%08x)\n",myCpuCaps,myCpuMask);
     return ;
@@ -173,6 +190,10 @@ static int Cpu2Lav(uint32_t admMask)
     	LAV_CPU_CAPS(SSE2);
     	LAV_CPU_CAPS(SSE3);
     	LAV_CPU_CAPS(SSSE3);
+    	LAV_CPU_CAPS(SSE4);
+    	LAV_CPU_CAPS(SSE42);
+    	LAV_CPU_CAPS(AVX);
+    	LAV_CPU_CAPS(AVX2);
         return out;
 }
 
@@ -187,7 +208,10 @@ bool     CpuCaps::setMask(uint32_t mask)
     myCpuMask=mask;
 
     int lavCpuMask=Cpu2Lav(myCpuMask);
-    av_set_cpu_flags_mask(lavCpuMask);
+    //av_set_cpu_flags_mask(lavCpuMask); deprecated!
+    lavCpuMask &= av_get_cpu_flags();
+    ADM_info("[CpuCaps] Forcing lav cpu flags 0x%08x\n",lavCpuMask);
+    av_force_cpu_flags(lavCpuMask);
 
     return true;
 }
@@ -200,6 +224,13 @@ uint32_t     CpuCaps::getMask( )
 {
     return myCpuMask;
 }
+/**
+ * \fn getCaps
+ */
+uint32_t CpuCaps::getCaps(void)
+{
+    return myCpuCaps;
+}
 extern "C"
 {
 extern void adm2_emms_yasm(void);
@@ -210,6 +241,8 @@ extern void adm2_emms_yasm(void);
  */
 void ADM_emms()
 {
+#ifdef ADM_CPU_X86
     adm2_emms_yasm();
+#endif
 }
 
