@@ -33,6 +33,12 @@ ADM_audioStream::ADM_audioStream(WAVHeader *header,ADM_audioAccess *access)
     lastDts=ADM_AUDIO_NO_DTS;
     lastDtsBase=0;
     sampleElapsed=0;
+    if(wavHeader.encoding==WAV_AAC)
+        samplesPerPacket=AAC_DEFAULT_FRAME_LENGTH;
+    else if(wavHeader.encoding==WAV_AAC_HE)
+        samplesPerPacket=AAC_DEFAULT_FRAME_LENGTH<<1;
+    else
+        samplesPerPacket=512; // dummy
     durationInUs=0;
     if(access && access->canGetDuration())
         durationInUs=access->getDurationInUs();
@@ -55,19 +61,19 @@ bool  ADM_audioStream::goToTime(uint64_t nbUs)
     }
     ADM_assert(true==access->canSeekOffset());
     // Convert time to offset in bytes
-    float f=nbUs*wavHeader.byterate;
+    double f=nbUs*wavHeader.byterate;
     f/=1000;
     f/=1000; // in bytes
-    if(access->setPos( (uint32_t)(f+0.5)))
+    if(access->setPos( (uint64_t)(f+0.5)))
     {
         // The seek might not be accurate, recompute the Dts
         // it is better to undershoot in most case
         uint64_t pos=access->getPos();
         // compute dts from pos & byterate
-        float r=pos;
-            r*=1000*1000;
-            r/=wavHeader.byterate;
-            setDts(r);
+        double r=pos;
+        r*=1000*1000;
+        r/=wavHeader.byterate;
+        setDts(r);
         return 1;
     }
     return false;
@@ -81,11 +87,9 @@ uint64_t dts=0;
     if(!access->getPacket(buffer,size,sizeMax,&dts)) return 0;
     // We got the packet
     // Try to guess the nbSample
-    // if it is AAC, we hardcodec 1024 samples
-
-    if(wavHeader.encoding==WAV_AAC)
+    if(wavHeader.encoding==WAV_AAC || wavHeader.encoding==WAV_AAC_HE)
     {
-        *nbSample=1024;
+        *nbSample=samplesPerPacket;
         if(dts!=ADM_AUDIO_NO_DTS)
         {
             setDts(dts);
@@ -214,6 +218,7 @@ const char *getStrFromAudioCodec( uint32_t codec)
               case WAV_MP2: return QT_TRANSLATE_NOOP("adm","MP2");
               case WAV_MP3: return QT_TRANSLATE_NOOP("adm","MP3");
               case WAV_WMAPRO:  return QT_TRANSLATE_NOOP("adm","WMAPRO");
+              case WAV_WMALOSSLESS: return QT_TRANSLATE_NOOP("adm","WMA Lossless");
               case WAV_WMA:  return QT_TRANSLATE_NOOP("adm","WMA");
               case WAV_LPCM: return QT_TRANSLATE_NOOP("adm","LPCM");
               case WAV_AC3:  return QT_TRANSLATE_NOOP("adm","AC3");
@@ -223,13 +228,15 @@ const char *getStrFromAudioCodec( uint32_t codec)
               case WAV_PCM_FLOAT:  return QT_TRANSLATE_NOOP("adm","Float PCM");
               case WAV_OGG_VORBIS: return QT_TRANSLATE_NOOP("adm","Ogg Vorbis");
               case WAV_MP4: return QT_TRANSLATE_NOOP("adm","MP4");
-              case WAV_AAC: return QT_TRANSLATE_NOOP("adm","AAC");
+              case WAV_AAC:
+              case WAV_AAC_HE: return QT_TRANSLATE_NOOP("adm","AAC");
               case WAV_QDM2: return QT_TRANSLATE_NOOP("adm","QDM2");
               case WAV_AMRNB: return QT_TRANSLATE_NOOP("adm","AMR-NB");
               case WAV_MSADPCM: return QT_TRANSLATE_NOOP("adm","MSADPCM");
               case WAV_ULAW: return QT_TRANSLATE_NOOP("adm","ULAW");
               case WAV_IMAADPCM: return QT_TRANSLATE_NOOP("adm","IMA ADPCM");
               case WAV_8BITS_UNSIGNED:return QT_TRANSLATE_NOOP("adm","8-bit PCM");
+              case WAV_TRUEHD: return QT_TRANSLATE_NOOP("adm","TrueHD");
       }
       ADM_warning("Unkown audio codec :%d (0x%x)\n",codec,codec);
       return QT_TRANSLATE_NOOP("adm","Unknown codec");
