@@ -27,8 +27,11 @@ extern "C" {
 
 #if 0
 #define xdebug ADM_info
+#include "ADM_coreUtils.h"
+#define xhdump mixDump
 #else
 #define xdebug(...) {}
+#define xhdump(...) {}
 #endif
 
 static const int aacChannels[16]=
@@ -184,8 +187,7 @@ bool ADM_latm2aac::AudioSpecificConfig(getBits &bits,int &bitsConsumed)
 #endif
     
     xdebug("Got %d extraData \n",extraLen);
-    for(int i=0;i<extraLen;i++)
-        xdebug(" %02x",extraData[i]);
+    xhdump(extraData,extraLen);
     xdebug(" \nFrequency %d, channels %d\n",fq,channels);
     conf.gotConfig=true;
     return true;
@@ -284,9 +286,9 @@ bool ADM_latm2aac::readStreamMuxConfig(getBits &bits)
 
         xdebug("NumSubFrame=%d, numSubProgram=%d\n",numSubFrames,numbSubProgram);
 
-        if(numSubFrames!=1 || numbSubProgram!=1  )
+        if(numbSubProgram!=1)
         {
-            ADM_warning("LATM: only supports subframe=1, subprogram=1\n");
+            ADM_warning("LATM: only numProgram = 1 supported, got %d\n",numbSubProgram);
             return false;
         }
 
@@ -388,6 +390,8 @@ bool  ADM_latm2aac::readAudioMux( uint64_t dts,getBits &bits )
        }
     } // streamMuxConfig
 //    if(!numSubFrames) return false;
+    if(!conf.gotConfig)
+        return false;
     if(conf.audioMuxVersionA==0)
     {
        // only 1 subFrames ATM... for(int i=0;i<numSubFrames;i++)
@@ -412,7 +416,7 @@ bool  ADM_latm2aac::readAudioMux( uint64_t dts,getBits &bits )
 */
 bool ADM_latm2aac::demuxLatm(uint64_t date,uint8_t *start,uint32_t size)
 {
-    getBits  bits(size,start);
+    getBits bits(size+AV_INPUT_BUFFER_PADDING_SIZE,start);
     return readAudioMux(date,bits);
 }
 /**
@@ -420,35 +424,78 @@ bool ADM_latm2aac::demuxLatm(uint64_t date,uint8_t *start,uint32_t size)
     \brief Check for LOAS sync word, extract LATM frames
 */
 
-bool ADM_latm2aac::pushData(int incomingLen,uint8_t *inData,uint64_t dts)
+bool ADM_latm2aac::pushData(int incomingLen,uint8_t *inData)
 {
-    // Lookup sync
-    uint8_t *end=inData+incomingLen;
-    uint8_t *start=inData;
     xdebug("Pushing data %d bytes\n",incomingLen);
-    while(start<end)
+    // Compact incoming buffer
+    if(head==tail)
+    {
+        head=tail=0;
+    }
+    if(tail>INCOMING_BUFFER_SIZE>>1)
+    {
+        int size=head-tail;
+        memmove(depot.at(0),depot.at(tail),size);
+        head=size;
+        tail=0;
+    }
+    if(head+incomingLen>INCOMING_BUFFER_SIZE)
+    {
+        ADM_error("LATM incoming buffer overflow: incoming: %d available: %d\n",incomingLen,INCOMING_BUFFER_SIZE-head);
+        return false;
+    }
+    // Add data
+    memcpy(depot.at(head),inData,incomingLen);
+    head+=incomingLen;
+    memset(depot.at(head),0,AV_INPUT_BUFFER_PADDING_SIZE);
+    return true;
+}
+/**
+    \fn convert
+    \brief find LATM frame in the incoming buffer and demux it
+*/
+ADM_latm2aac::LATM_STATE ADM_latm2aac::convert(uint64_t dts)
+{
+    if(head==tail)
+        return LATM_MORE_DATA_NEEDED;
+    // Lookup sync
+    uint8_t *start=depot.at(tail);
+    uint8_t *end=depot.at(head);
+    uint32_t len=0;
+    while(start+2<end)
     {
         int key=(start[0]<<8)+start[1];
         if((key & 0xffe0)!=0x56e0)  // 0x2b7 shifted by one bit
         {
-            ADM_warning("Sync lost\n");
-            return true;
+            start++;
+            tail++;
+            continue;
         }
-        uint32_t len=start[2]+((key & 0x1f)<<8);
+        len=start[2]+((key & 0x1f)<<8);
         start+=3;
         if(start+len>end)
         {
-            ADM_warning("Not enough data, need %d, got %d\n",len,(int)(end-start));
-            return true;
+            xdebug("Not enough data, need %d, got %d\n",len,(int)(end-start));
+            int size=head-tail;
+            memmove(depot.at(0),depot.at(tail),size);
+            tail=0;
+            head=size;
+            return LATM_MORE_DATA_NEEDED;
         }
-        xdebug("Found LATM : size %d\n",len);
-        demuxLatm(dts,start,len);
-        dts=ADM_NO_PTS;
-        // LATM demux
-        start+=len;
+        break;
     }
+    if(!len)
+    {
+        xdebug("No sync.\n");
+        return LATM_MORE_DATA_NEEDED;
+    }
+    xdebug("Found LATM : size %d\n",len);
+    // Demux one LATM frame
+    bool r = demuxLatm(dts,start,len);
+    tail+=len+3;
+    ADM_assert(head>=tail);
     xdebug("-- end of this LATM frame --\n");
-    return true;
+    return r? LATM_OK : LATM_ERROR;
 }
 /**
     \fn ctor
@@ -460,9 +507,10 @@ ADM_latm2aac::ADM_latm2aac(void)
                 channels=0;
                 extraLen=0;
                 memset(&conf,0,sizeof(conf));
-                conf.gotConfig=false;
                 for(int i=0;i<LATM_NB_BUFFERS;i++)
                     listOfFreeBuffers.pushBack(&(buffers[i]));
+                depot.setSize(INCOMING_BUFFER_SIZE + AV_INPUT_BUFFER_PADDING_SIZE);
+                head=tail=0;
 }
 /**
     \fn dtor
@@ -492,6 +540,7 @@ bool ADM_latm2aac::flush()
    listOfUsedBuffers.clear();
    for(int i=0;i<LATM_NB_BUFFERS;i++)
                     listOfFreeBuffers.pushBack(&(buffers[i]));
+   head=tail=0;
    return true;
 }
 /**
