@@ -17,6 +17,8 @@
 #include "ADM_default.h"
 #include "ADM_colorspace.h"
 #include "ADM_image.h"
+#include "ADM_rgb.h" 
+#include "prefs.h"
 
 extern "C" {
 #include "libavcodec/avcodec.h"
@@ -24,8 +26,6 @@ extern "C" {
 #include "libswscale/swscale.h"
 }
 
-#include "ADM_rgb.h" 
-#include "ADM_colorspace.h"
 
 #ifdef ADM_CPU_X86
 		#define ADD(x,y) if( CpuCaps::has##x()) flags|=SWS_CPU_CAPS_##y;
@@ -39,7 +39,6 @@ extern "C" {
 #endif
 
 #define CONTEXT (SwsContext *)context
-#define BGR32_IS_SWAPPED 1
 
 /**
     \fn swapRGB
@@ -62,54 +61,14 @@ static void swapRGB32(uint32_t w, uint32_t h, uint32_t p, uint8_t *to)
         }
     }
 }
-/**
-    \fn ADMColor2LAVColor
-    \brief Convert ADM colorspace type swscale/lavcodec colorspace name
 
-*/
-static AVPixelFormat ADMColor2LAVColor(ADM_colorspace fromColor_)
-{
-  ADM_colorspace fromColor=fromColor_;
-  int intColor=(int)fromColor;
-  intColor&=ADM_COLOR_MASK;
-  fromColor=(ADM_colorspace)intColor;
-  switch(fromColor)
-  {
-    case ADM_COLOR_YUV444: return AV_PIX_FMT_YUV444P;
-    case ADM_COLOR_YUV411: return AV_PIX_FMT_YUV411P;
-    case ADM_COLOR_YUV422: return AV_PIX_FMT_YUYV422;
-    case ADM_COLOR_UYVY422: return AV_PIX_FMT_UYVY422;
-    case ADM_COLOR_YV12: return AV_PIX_FMT_YUV420P;
-    case ADM_COLOR_NV12: return AV_PIX_FMT_NV12;
-    case ADM_COLOR_YUV422P: return AV_PIX_FMT_YUV422P;
-    case ADM_COLOR_RGB555: return AV_PIX_FMT_RGB555LE;
-    case ADM_COLOR_BGR555: return AV_PIX_FMT_BGR555LE;
-    case ADM_COLOR_RGB32A: return AV_PIX_FMT_RGBA;
-#ifdef BGR32_IS_SWAPPED
-    case ADM_COLOR_BGR32A: return AV_PIX_FMT_RGBA; // Faster that way...
-#else
-    case ADM_COLOR_BGR32A: return AV_PIX_FMT_BGRA;
-#endif
-    case ADM_COLOR_RGB24: return AV_PIX_FMT_RGB24;
-    case ADM_COLOR_BGR24: return AV_PIX_FMT_BGR24;
-    case ADM_COLOR_YUV420_10BITS: return AV_PIX_FMT_YUV420P10LE;
-    case ADM_COLOR_YUV420_12BITS: return AV_PIX_FMT_YUV420P12LE;
-    case ADM_COLOR_NV12_10BITS:  return AV_PIX_FMT_P010LE;
-    case ADM_COLOR_YUV444_10BITS: return AV_PIX_FMT_YUV444P10LE;
-    case ADM_COLOR_YUV422_10BITS: return AV_PIX_FMT_YUV422P10LE;
-    case ADM_COLOR_YUV444_12BITS: return AV_PIX_FMT_YUV444P12LE;
-    case ADM_COLOR_Y8: return AV_PIX_FMT_GRAY8;
-    default : ADM_assert(0); 
-  }
-  return AV_PIX_FMT_YUV420P;
-}
 /**
       \fn getStrideAndPointers
       \param dst=1 -> destination, =0 source
       \brief Fill in strides etc.. needed by libswscale
 */
 uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
-        uint8_t  *from,ADM_colorspace fromColor,
+        uint8_t  *from,ADM_pixelFormat fromPixFrmt,
         uint8_t **srcData,int *srcStride)
 {
     uint32_t width,height;
@@ -122,9 +81,9 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
         width=dstWidth;
         height=dstHeight;
     }
-  switch(fromColor)
+  switch(fromPixFrmt)
   {
-    case ADM_COLOR_RGB555: 
+    case ADM_PIXFRMT_RGB555: 
             srcData[0]=from;
             srcData[1]=NULL;
             srcData[2]=NULL;
@@ -132,8 +91,8 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcStride[1]=0;
             srcStride[2]=0;
             break;
-    case ADM_COLOR_RGB24:
-    case ADM_COLOR_BGR24:
+    case ADM_PIXFRMT_RGB24:
+    case ADM_PIXFRMT_BGR24:
             srcData[0]=from;
             srcData[1]=NULL;
             srcData[2]=NULL;
@@ -141,7 +100,19 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcStride[1]=0;
             srcStride[2]=0;
             break;
-    case  ADM_COLOR_YV12:
+    case ADM_PIXFRMT_GBR24P:
+            srcData[0]=from;
+            width=ADM_IMAGE_ALIGN(width);
+            height=ADM_IMAGE_ALIGN(height);
+            from+=width*height;
+            srcData[1]=from;
+            from+=width*height;
+            srcData[2]=from;
+            srcStride[0]=width;
+            srcStride[1]=width;
+            srcStride[2]=width;
+            break;
+    case  ADM_PIXFRMT_YV12:
             srcData[0]=from;
             width=ADM_IMAGE_ALIGN(width);
             height=ADM_IMAGE_ALIGN(height);
@@ -153,8 +124,8 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcStride[1]=width>>1;
             srcStride[2]=width>>1;
             break;
-    case ADM_COLOR_YUV420_10BITS:
-    case ADM_COLOR_YUV420_12BITS:
+    case ADM_PIXFRMT_YUV420_10BITS:
+    case ADM_PIXFRMT_YUV420_12BITS:
             srcData[0]=from;
             width=ADM_IMAGE_ALIGN(width*2);
             height=ADM_IMAGE_ALIGN(height);
@@ -166,7 +137,7 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcStride[1]=width>>1;
             srcStride[2]=width>>1;
             break;
-    case ADM_COLOR_NV12:
+    case ADM_PIXFRMT_NV12:
             srcData[0]=from;
             width=ADM_IMAGE_ALIGN(width);
             height=ADM_IMAGE_ALIGN(height);
@@ -177,8 +148,8 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcStride[1]=width;
             srcStride[2]=0;
             break;
-    case  ADM_COLOR_YUV422:
-    case  ADM_COLOR_UYVY422:        
+    case  ADM_PIXFRMT_YUV422:
+    case  ADM_PIXFRMT_UYVY422:        
             srcData[0]=from;
             srcData[1]=NULL;
             srcData[2]=NULL;
@@ -186,7 +157,7 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcStride[1]=0;
             srcStride[2]=0;
             break;            
-    case  ADM_COLOR_YUV422P:
+    case  ADM_PIXFRMT_YUV422P:
             srcData[0]=from;
             width=ADM_IMAGE_ALIGN(width);
             height=ADM_IMAGE_ALIGN(height);
@@ -198,8 +169,8 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcStride[1]=width>>1;
             srcStride[2]=width>>1;
             break;
-    case ADM_COLOR_RGB32A:
-    case ADM_COLOR_BGR32A:
+    case ADM_PIXFRMT_RGB32A:
+    case ADM_PIXFRMT_BGR32A:
             srcData[0]=from;
             srcData[1]=NULL;
             srcData[2]=NULL;
@@ -207,7 +178,7 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcStride[1]=0;
             srcStride[2]=0;
             break;
-    case ADM_COLOR_YUV422_10BITS:
+    case ADM_PIXFRMT_YUV422_10BITS:
             srcData[0]=from;
             width=ADM_IMAGE_ALIGN(width*2);
             height=ADM_IMAGE_ALIGN(height);
@@ -219,7 +190,7 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcStride[1]=width>>1;
             srcStride[2]=width>>1;
             break;
-    case ADM_COLOR_YUV444_12BITS:
+    case ADM_PIXFRMT_YUV444_12BITS:
             srcData[0]=from;
             width=ADM_IMAGE_ALIGN(width*2);
             height=ADM_IMAGE_ALIGN(height);
@@ -250,16 +221,16 @@ bool ADMColorScalerFull::convert(uint8_t  *from, uint8_t *to)
     int srcStride[3];
     int dstStride[3];
 
-    getStrideAndPointers(false,from,fromColor,srcData,srcStride);
-    getStrideAndPointers(true,to,toColor,dstData,dstStride);
+    getStrideAndPointers(false,from,fromPixFrmt,srcData,srcStride);
+    getStrideAndPointers(true,to,toPixFrmt,dstData,dstStride);
 
-    if(fromColor == ADM_COLOR_YV12)
+    if(fromPixFrmt == ADM_PIXFRMT_YV12)
     {
         uint8_t *p=srcData[1];
         srcData[1]=srcData[2];
         srcData[2]=p;
     }
-    if(toColor == ADM_COLOR_YV12)
+    if(toPixFrmt == ADM_PIXFRMT_YV12)
     {
         uint8_t *p=dstData[1];
         dstData[1]=dstData[2];
@@ -267,12 +238,12 @@ bool ADMColorScalerFull::convert(uint8_t  *from, uint8_t *to)
     }
 
 #ifdef BGR32_IS_SWAPPED
-    if(fromColor != toColor && fromColor == ADM_COLOR_BGR32A)
+    if(fromPixFrmt != toPixFrmt && fromPixFrmt == ADM_PIXFRMT_BGR32A)
         swapRGB32(srcWidth,srcHeight,srcStride[0],srcData[0]);
 #endif
     sws_scale(CONTEXT,srcData,srcStride,0,srcHeight,dstData,dstStride);
 #ifdef BGR32_IS_SWAPPED
-    if(fromColor != toColor && toColor==ADM_COLOR_BGR32A)
+    if(fromPixFrmt != toPixFrmt && toPixFrmt==ADM_PIXFRMT_BGR32A)
         swapRGB32(dstWidth,dstHeight,dstStride[0],dstData[0]);
 #endif
     return true;
@@ -293,12 +264,12 @@ bool ADMColorScalerFull::convertPlanes(int sourceStride[3], int destStride[3], u
         dst[i]=destData[i];
     }
 #ifdef BGR32_IS_SWAPPED
-    if(fromColor != toColor && fromColor == ADM_COLOR_BGR32A)
+    if(fromPixFrmt != toPixFrmt && fromPixFrmt == ADM_PIXFRMT_BGR32A)
         swapRGB32(srcWidth,srcHeight,xs[0],src[0]);
 #endif
     sws_scale(CONTEXT,src,xs,0,srcHeight,dst,xd);
 #ifdef BGR32_IS_SWAPPED
-    if(fromColor != toColor && toColor == ADM_COLOR_BGR32A)
+    if(fromPixFrmt != toPixFrmt && toPixFrmt == ADM_PIXFRMT_BGR32A)
         swapRGB32(dstWidth,dstHeight,xd[0],dst[0]);
 #endif
     return true;
@@ -309,6 +280,12 @@ bool ADMColorScalerFull::convertPlanes(int sourceStride[3], int destStride[3], u
 */
 bool            ADMColorScalerFull::convertImage(ADMImage *sourceImage, ADMImage *destImage)
 {
+    if (toneMapper != NULL)
+    {
+        if (toneMapper->toneMap(sourceImage, destImage))
+            return true;
+    }
+
     int xs[4];
     int xd[4];
     uint8_t *src[4];
@@ -324,20 +301,20 @@ bool            ADMColorScalerFull::convertImage(ADMImage *sourceImage, ADMImage
     src[3]=sourceImage->GetReadPtr(PLANAR_ALPHA);
     dst[3]=destImage->GetWritePtr(PLANAR_ALPHA);
 
-    if(fromColor==ADM_COLOR_YV12)
+    if(fromPixFrmt==ADM_PIXFRMT_YV12)
     {
         uint8_t *p=src[1];
         src[1]=src[2];
         src[2]=p;
     }
 
-    if(toColor==ADM_COLOR_YV12)
+    if(toPixFrmt==ADM_PIXFRMT_YV12)
     {
         uint8_t *p=dst[1];
         dst[1]=dst[2];
         dst[2]=p;
     }
-    if(fromColor != toColor)
+    if(fromPixFrmt != toPixFrmt)
     {
         int *itbl = NULL;
         int *ta = NULL;
@@ -359,7 +336,7 @@ bool            ADMColorScalerFull::convertImage(ADMImage *sourceImage, ADMImage
             }
         }
 #ifdef BGR32_IS_SWAPPED
-        if(fromColor == ADM_COLOR_BGR32A)
+        if(fromPixFrmt == ADM_PIXFRMT_BGR32A)
             swapRGB32(srcWidth,srcHeight,xs[0],src[0]);
 #endif
     }else
@@ -370,7 +347,7 @@ bool            ADMColorScalerFull::convertImage(ADMImage *sourceImage, ADMImage
     sws_scale(CONTEXT,src,xs,0,srcHeight,dst,xd);
 
 #ifdef BGR32_IS_SWAPPED
-    if(fromColor != toColor && toColor == ADM_COLOR_BGR32A)
+    if(fromPixFrmt != toPixFrmt && toPixFrmt == ADM_PIXFRMT_BGR32A)
         swapRGB32(dstWidth,dstHeight,xd[0],dst[0]);
 #endif
     return true;
@@ -388,11 +365,12 @@ bool            ADMColorScalerFull::convertImage(ADMImage *sourceImage, ADMImage
 ADMColorScalerFull::ADMColorScalerFull(ADMColorScaler_algo algo,
             int sw, int sh,
             int dw, int dh,
-            ADM_colorspace from,ADM_colorspace to)
+            ADM_pixelFormat from,ADM_pixelFormat to)
 {
-   context=NULL;
-   reset(algo,sw,sh,dw,dh,from,to);
-
+    context=NULL;
+    possibleHdrContent=false;
+    toneMapper=NULL;
+    reset(algo,sw,sh,dw,dh,from,to);
 }
 /**
     \fn  ~ADMColorScaler
@@ -400,19 +378,29 @@ ADMColorScalerFull::ADMColorScalerFull(ADMColorScaler_algo algo,
 */
 ADMColorScalerFull::~ADMColorScalerFull()
 {
-  if(context)
-  {
-     sws_freeContext(CONTEXT);
-     context=NULL;
-  }
+    if(context)
+    {
+        sws_freeContext(CONTEXT);
+        context=NULL;
+    }
+    if (toneMapper)
+    {
+        delete toneMapper;
+        toneMapper=NULL;
+    }
 }
 /**
     \fn reset
 */
-bool  ADMColorScalerFull::reset(ADMColorScaler_algo algo, int sw, int sh, int dw,int dh,ADM_colorspace from,ADM_colorspace to)
+bool  ADMColorScalerFull::reset(ADMColorScaler_algo algo, int sw, int sh, int dw,int dh,ADM_pixelFormat from,ADM_pixelFormat to)
 {
     if(context) sws_freeContext(CONTEXT);
     context=NULL;
+    if (toneMapper)
+    {
+        delete toneMapper;
+        toneMapper=NULL;
+    }
     this->algo=algo;
     int flags;
     switch(algo)
@@ -427,6 +415,7 @@ bool  ADMColorScalerFull::reset(ADMColorScaler_algo algo, int sw, int sh, int dw
     SETAL(GAUSS);
     SETAL(SINC);
     SETAL(SPLINE);
+    SETAL(POINT);    // nearest neighbor
     default: ADM_assert(0);
     }
 #if 0 // this is gone, we need to patch av_get_cpu_flags directly now
@@ -434,17 +423,22 @@ bool  ADMColorScalerFull::reset(ADMColorScaler_algo algo, int sw, int sh, int dw
         FLAGS();
     }
 #endif
-  
+    possibleHdrContent = (from >= ADM_PIXFRMT_YUV444_10BITS) && (from <= ADM_PIXFRMT_YUV444_12BITS) && (to == ADM_PIXFRMT_YV12);
+    if (possibleHdrContent)
+    {
+        toneMapper = new ADMToneMapper(flags, sw, sh, dw, dh, from, to);
+    }
+    
     srcWidth=sw;
     srcHeight=sh;
 
     dstWidth=dw;
     dstHeight=dh;
 
-    fromColor=from;
-    toColor=to;
-    AVPixelFormat lavFrom=ADMColor2LAVColor(fromColor );
-    AVPixelFormat lavTo=ADMColor2LAVColor(toColor );
+    fromPixFrmt=from;
+    toPixFrmt=to;
+    AVPixelFormat lavFrom=ADMPixFrmt2LAVPixFmt(fromPixFrmt );
+    AVPixelFormat lavTo=ADMPixFrmt2LAVPixFmt(toPixFrmt );
     
     context=(void *)sws_getContext(
                       srcWidth,srcHeight,
@@ -459,7 +453,7 @@ bool            ADMColorScalerSimple::changeWidthHeight(int newWidth, int newHei
 {
     if(newWidth==srcWidth && newHeight==srcHeight) return true; // no change
     
-     return reset(algo, newWidth,newHeight, newWidth,newHeight,fromColor,toColor);
+     return reset(algo, newWidth,newHeight, newWidth,newHeight,fromPixFrmt,toPixFrmt);
 
 }
 
@@ -474,15 +468,15 @@ bool ADMColorScalerFull::convertImage(ADMImage *img, uint8_t *to)
     int dstPitch[3];
     img->GetPitches(srcPitch);
     img->GetReadPlanes(srcPlanes);
-    getStrideAndPointers(true,to,toColor, dstPlanes, dstPitch);
+    getStrideAndPointers(true,to,toPixFrmt, dstPlanes, dstPitch);
 
-    if(fromColor==ADM_COLOR_YV12)
+    if(fromPixFrmt==ADM_PIXFRMT_YV12)
     {
         uint8_t *p=srcPlanes[1];
         srcPlanes[1]=srcPlanes[2];
         srcPlanes[2]=p;
     }
-    if(toColor==ADM_COLOR_YV12)
+    if(toPixFrmt==ADM_PIXFRMT_YV12)
     {
         uint8_t *p=dstPlanes[1];
         dstPlanes[1]=dstPlanes[2];
@@ -505,4 +499,5 @@ bool ADMColorScalerFull::convertImage(ADMImage *img, uint8_t *to)
     }
     return convertPlanes(srcPitch,dstPitch,srcPlanes,dstPlanes);
 }
+
 //EOF
