@@ -20,9 +20,11 @@
 #include "ADM_default.h"
 #include "ADM_vidMisc.h"
 #include "fourcc.h"
+#include "ADM_rgb.h"
 #include "ADM_coreMuxerFfmpeg.h"
 #include "ADM_muxerUtils.h"
 #include "ADM_coreCodecMapping.h"
+#include "ADM_audioXiphUtils.h"
 
 extern "C" {
 #include "libavformat/url.h"
@@ -87,11 +89,14 @@ muxerFFmpeg::muxerFFmpeg()
 */
 bool muxerFFmpeg::closeMuxer()
 {
+    int res=0;
     if(oc)
     {
         if(initialized==true)
         {
-            av_write_trailer(oc);
+            res=av_write_trailer(oc);
+            if(res<0)
+                ADM_warning("Error %d writing trailer.\n",res);
             avio_close(oc->pb);
         }
         avformat_free_context(oc);
@@ -103,7 +108,7 @@ bool muxerFFmpeg::closeMuxer()
         audio_st[i]=NULL;
     }
     video_st=NULL;
-    return true;
+    return res >= 0;
 }
 /**
     \fn muxerFFmpeg
@@ -125,15 +130,27 @@ bool muxerFFmpeg::setupMuxer(const char *format,const char *filename)
             return false;
     }
     oc = avformat_alloc_context();
-	if (!oc)
-	{
-       		printf("[FF] alloc format context failed\n");
-            return false;
-	}
-	oc->oformat = fmt;
-	snprintf(oc->filename,1000,"file://%s",filename);
-    // probably a memeleak here
-
+    if (!oc)
+    {
+        printf("[FF] alloc format context failed\n");
+        return false;
+    }
+    oc->oformat = fmt;
+#if defined(__APPLE__)
+ #define MAX_LEN 1024
+#else
+ #define MAX_LEN 4096
+#endif
+    uint32_t len=strlen(filename);
+    if(len>MAX_LEN)
+    {
+        ADM_error("Filename length %u exceeds limit %u\n",len,MAX_LEN);
+        return false;
+    }
+    char *url=(char *)ADM_alloc(len+8);
+    snprintf(url,len+8,"file://%s",filename);
+    url[len+7]=0;
+    oc->url=url;
 //#warning use AV METADATA
     printf("[FF] Muxer opened\n");
     return true;
@@ -155,6 +172,7 @@ static bool setAvCodec(AVCodecContext *c,enum AVCodecID id)
 bool muxerFFmpeg::initVideo(ADM_videoStream *stream)
 {
     audioDelay=stream->getVideoDelay();
+    printf("[muxerFFmpeg::initVideo] Initial audio delay: %" PRIu64" ms\n",audioDelay/1000);
     video_st = avformat_new_stream(oc, NULL);
 	if (!video_st)
 	{
@@ -166,11 +184,22 @@ bool muxerFFmpeg::initVideo(ADM_videoStream *stream)
         par->sample_aspect_ratio.num=1;
         par->sample_aspect_ratio.den=1;
         video_st->sample_aspect_ratio=par->sample_aspect_ratio;
-        par->bit_rate=9000*1000;
+        //par->bit_rate=9000*1000;
         par->codec_type = AVMEDIA_TYPE_VIDEO;
         par->width = stream->getWidth();
         par->height =stream->getHeight();
 
+        {
+        uint32_t r,p,t,m;
+        if(stream->getColorInfo(&r,&p,&t,&m))
+        {
+            par->color_range = (AVColorRange) validateColorRange(r);
+            par->color_primaries = (AVColorPrimaries) validateColorPrimaries(p);
+            par->color_trc = (AVColorTransferCharacteristic) validateColorTrC(t);
+            par->color_space = (AVColorSpace) validateColorSpace(m);
+        }
+        }
+
         uint32_t videoExtraDataSize=0;
         uint8_t  *videoExtraData;
         stream->getExtraData(&videoExtraDataSize,&videoExtraData);
@@ -272,6 +301,7 @@ bool muxerFFmpeg::initVideo(ADM_videoStream *stream)
                                                     return false;
                                                 }
                                                 par->codec_id=cid;
+                                                par->codec_tag=id;
                                             }
                                         }
                         }
@@ -326,11 +356,25 @@ bool muxerFFmpeg::initAudio(uint32_t nbAudioTrack,ADM_audioStream **audio)
           par->frame_size=1024; //For AAC mainly, sample per frame
           printf("[FF] Bitrate %u\n",(audioheader->byterate*8)/1000);
           par->sample_rate = audioheader->frequency;
+          par->channel_layout = av_get_default_channel_layout(audioheader->channels);
+          ADM_info("Using default channel layout 0x%lx for %u channels\n",par->channel_layout,audioheader->channels);
           switch(audioheader->encoding)
           {
                   case WAV_OGG_VORBIS:
                                 par->codec_id = AV_CODEC_ID_VORBIS;par->frame_size=6*256;
-                                ffmpuxerSetExtradata(par,audioextraSize,audioextraData);
+                                if(!strcmp(fmt->name,"mp4") || !strcmp(fmt->name,"psp")) // Need to translate from adm to xiph
+                                {
+                                    int xiphLen=(int)audioextraSize+(audioextraSize/255)+4+5;
+                                    uint8_t *xiph=new uint8_t[xiphLen+AV_INPUT_BUFFER_PADDING_SIZE];
+                                    memset(xiph,0,xiphLen+AV_INPUT_BUFFER_PADDING_SIZE);
+                                    xiphLen=ADMXiph::admExtraData2xiph(audioextraSize,audioextraData,xiph);
+                                    ffmpuxerSetExtradata(par,xiphLen,xiph);
+                                    delete [] xiph;
+                                    xiph=NULL;
+                                }else
+                                {
+                                    ffmpuxerSetExtradata(par,audioextraSize,audioextraData);
+                                }
                                 break;
                   case WAV_FLAC:
                                 par->codec_id = AV_CODEC_ID_FLAC;
@@ -356,23 +400,26 @@ bool muxerFFmpeg::initAudio(uint32_t nbAudioTrack,ADM_audioStream **audio)
                               break;
                   case WAV_LPCM:
                   case WAV_PCM:
-                                if(audioheader->encoding==WAV_LPCM)
+                                //par->frame_size=4; // One chunk is 10 ms (1/100 of fq)
+                                par->frame_size = 0; // so does ffmpeg
+                                par->block_align = audioheader->blockalign;
                                 {
-                                  // One chunk is 10 ms (1/100 of fq)
-                                  par->frame_size=4;
-                                  par->codec_id = AV_CODEC_ID_PCM_S16BE;break;
-                                }
-                                else
-                                {
-                                  // One chunk is 10 ms (1/100 of fq)
-                                  par->frame_size=4;
-                                  par->codec_id = AV_CODEC_ID_PCM_S16LE;break;
+                                    bool bigEndian = audioheader->encoding == WAV_LPCM;
+                                    switch(audioheader->bitspersample)
+                                    {
+                                        case 16: par->codec_id = bigEndian? AV_CODEC_ID_PCM_S16BE : AV_CODEC_ID_PCM_S16LE; break;
+                                        case 24: par->codec_id = bigEndian? AV_CODEC_ID_PCM_S24BE : AV_CODEC_ID_PCM_S24LE; break;
+                                        default: ADM_error("Unsupported bits per sample value: %u\n",audioheader->bitspersample); ADM_assert(0);
+                                    }
                                 }
+                                break;
                   case WAV_AAC:
+                  case WAV_AAC_HE:
                                   ffmpuxerSetExtradata(par,audioextraSize,audioextraData);
                                   par->codec_id = AV_CODEC_ID_AAC;
-                                  par->frame_size=1024;
+                                  par->frame_size=audio[i]->getSamplesPerPacket();
                                   break;
+                  case WAV_TRUEHD: par->codec_id = AV_CODEC_ID_TRUEHD; par->frame_size = 40; break;
                   default:
                                  printf("[FF]: Unsupported audio\n");
                                  return false;
@@ -449,6 +496,7 @@ bool muxerFFmpeg::saveLoop(const char *title)
     bool ret=true;
     uint32_t written=0;
     bool result=true;
+    bool gotVideoPacket=true;
     int missingPts=0;
     
     float f=(float)vStream->getAvgFps1000();
@@ -465,14 +513,16 @@ bool muxerFFmpeg::saveLoop(const char *title)
     encoding->setContainer(getContainerName());
     MuxAudioPacket *audioPackets=new MuxAudioPacket[nbAStreams];
     for(int i=0;i<nbAStreams;i++) // ugly...
-            audioPackets[i].clock=new audioClock(aStreams[i]->getInfo()->frequency);
+        audioPackets[i].clock=new audioClock(aStreams[i]->getInfo()->frequency);
     ADMBitstream out(bufSize);
     out.data=buffer;
 
-    while(true==vStream->getPacket(&out))
+    while(gotVideoPacket)
     {
-	AVPacket pkt;
-
+        gotVideoPacket=vStream->getPacket(&out);
+        uint64_t maxAudioDts;
+        if(gotVideoPacket)
+        {
             encoding->refresh();
             if(!encoding->isAlive())
             {
@@ -486,74 +536,29 @@ bool muxerFFmpeg::saveLoop(const char *title)
             aprintf("[FF:V] Pts: %" PRId64" DTS:%" PRId64" ms\n",xpts/1000,xdts/1000);
 
             aprintf("[FF:V] LastDts:%08" PRIu64" Dts:%08" PRIu64" (%04" PRIu64") Delta : %" PRIu64"\n",
-                        lastVideoDts,out.dts,out.dts/1000000,out.dts-lastVideoDts);
+                lastVideoDts,out.dts,out.dts/1000000,out.dts-lastVideoDts);
             rawDts=out.dts;
             if(rawDts==ADM_NO_PTS)
             {
                 ADM_warning("No DTS information for frame %" PRIu32"\n",written);
                 lastVideoDts+=videoIncrement;
-            }else if(lastVideoDts && rawDts<=lastVideoDts)
+            }else if(lastVideoDts && rawDts<lastVideoDts)
             {
-                ADM_warning("Duplicated or going back DTS for frame %" PRIu32", adding %" PRIu64" ms\n",written,(videoIncrement/2)/1000);
-                lastVideoDts+=videoIncrement/2;
+                ADM_warning("Duplicated or going back DTS for frame %" PRIu32"\n",written);
                 out.dts=lastVideoDts;
             }else
             {
                 lastVideoDts=out.dts;
             }
+            maxAudioDts=lastVideoDts+audioDelay;
+        }else
+        {
+            maxAudioDts=videoDuration;
+        }
 
-            if(out.pts==ADM_NO_PTS)
-            {
-                ADM_warning("No PTS information for frame %" PRIu32"\n",written);
-                missingPts++;
-                out.pts=lastVideoDts;
-            }
-            if(out.pts<lastVideoDts)
-            {
-                ADM_warning("Bumping PTS to keep PTS >= DTS for frame %" PRIu32"\n",written);
-                out.pts=lastVideoDts;
-            }
-
-            encoding->pushVideoFrame(out.len,out.out_quantizer,lastVideoDts);
-            muxerRescaleVideoTimeDts(&(out.dts),lastVideoDts);
-            if(!roundup && lastVideoDtsLav && out.dts==lastVideoDtsLav)
-            {
-                ADM_warning("Bumping lav DTS to avoid collision for frame %" PRIu32"\n",written);
-                out.dts++;
-            }
-            lastVideoDtsLav=out.dts;
-            muxerRescaleVideoTime(&(out.pts));
-            if(out.dts>out.pts)
-            {
-                ADM_warning("Bumping lav PTS to keep PTS >= DTS for frame %" PRIu32"\n",written);
-                out.pts=out.dts;
-            }
-            aprintf("[FF:V] RawDts:%lu Scaled Dts:%lu\n",rawDts,out.dts);
-            aprintf("[FF:V] Rescaled: Len : %d flags:%x Pts:%" PRIu64" Dts:%" PRIu64"\n",out.len,out.flags,out.pts,out.dts);
-
-            av_init_packet(&pkt);
-            pkt.dts=out.dts;
-            if(vStream->providePts()==true)
-            {
-                pkt.pts=out.pts;
-            }else
-            {
-                pkt.pts=pkt.dts;
-            }
-            pkt.stream_index=0;
-            pkt.data= buffer;
-            pkt.size= out.len;
-            if(out.flags & 0x10) // FIXME AVI_KEY_FRAME
-                        pkt.flags |= AV_PKT_FLAG_KEY;
-            ret =writePacket( &pkt);
-            aprintf("[FF]Frame:%u, DTS=%08lu PTS=%08lu\n",written,out.dts,out.pts);
-            if(false==ret)
-            {
-                printf("[FF]Error writing video packet\n");
-                break;
-            }
-            written++;
-            // Now send audio until they all have DTS > lastVideoDts+increment
+        if(written)
+        {
+            // Now send audio until they all have DTS > lastVideoDts
             for(int audio=0;audio<nbAStreams;audio++)
             {
                 MuxAudioPacket *audioTrack=&(audioPackets[audio]);
@@ -568,14 +573,14 @@ bool muxerFFmpeg::saveLoop(const char *title)
                     if(audioTrack->present==false)
                     {
                         if(false==a->getPacket(audioTrack->buffer,
-                                                &(audioTrack->size),
-                                                AUDIO_BUFFER_SIZE,
-                                                &(audioTrack->samples),
-                                                &(audioTrack->dts)))
+                                            &(audioTrack->size),
+                                            AUDIO_BUFFER_SIZE,
+                                            &(audioTrack->samples),
+                                            &(audioTrack->dts)))
                         {
-                                audioTrack->eof=true;
-                                ADM_info("No more audio packets for audio track %d\n",audio);
-                                break;
+                            audioTrack->eof=true;
+                            ADM_info("No more audio packets for audio track %d\n",audio);
+                            break;
                         }
                        // printf("Track %d , new audio packet DTS=%"PRId64" size=%"PRIu32"\n",audioTrack->dts,audioTrack->size);
                         audioTrack->present=true;
@@ -585,7 +590,7 @@ bool muxerFFmpeg::saveLoop(const char *title)
                     if(audioTrack->dts!=ADM_NO_PTS)
                     {
                         //printf("Audio PTS:%"PRId64", limit=%"PRId64"\n",audioTrack->dts,lastVideoDts+videoIncrement);
-                        if(audioTrack->dts>lastVideoDts+videoIncrement+audioDelay) break; // This packet is in the future
+                        if(audioTrack->dts>maxAudioDts) break; // This packet is in the future
                     }
                     // Write...
                     AVPacket pkt;
@@ -617,9 +622,69 @@ bool muxerFFmpeg::saveLoop(const char *title)
                     }
                    // printf("[FF] A:%"PRIu32" ms vs V: %"PRIu32" ms\n",(uint32_t)audioTrack->dts/1000,(uint32_t)(lastVideoDts+videoIncrement)/1000);
                 }
-                //if(!nb) printf("[FF] A: No audio for video frame %d\n",written);
             }
+        }
 
+        if(!gotVideoPacket) break;
+
+        if(out.pts==ADM_NO_PTS)
+        {
+            ADM_warning("No PTS information for frame %" PRIu32"\n",written);
+            missingPts++;
+            out.pts=lastVideoDts;
+        }
+        if(out.pts<lastVideoDts)
+        {
+            ADM_warning("Bumping PTS to keep PTS >= DTS for frame %" PRIu32"\n",written);
+            out.pts=lastVideoDts;
+        }
+
+        encoding->pushVideoFrame(out.len,out.out_quantizer,lastVideoDts);
+        muxerRescaleVideoTimeDts(&(out.dts),lastVideoDts);
+        if(!roundup && lastVideoDtsLav && out.dts==lastVideoDtsLav)
+        {
+            ADM_warning("Bumping lav DTS to avoid collision for frame %" PRIu32"\n",written);
+            out.dts++;
+        }
+        lastVideoDtsLav=out.dts;
+        muxerRescaleVideoTime(&(out.pts));
+        if(out.dts>out.pts)
+        {
+            ADM_warning("Bumping lav PTS to keep PTS >= DTS for frame %" PRIu32"\n",written);
+            out.pts=out.dts;
+        }
+        aprintf("[FF:V] RawDts:%lu Scaled Dts:%lu\n",rawDts,out.dts);
+        aprintf("[FF:V] Rescaled: Len : %d flags:%x Pts:%" PRIu64" Dts:%" PRIu64"\n",out.len,out.flags,out.pts,out.dts);
+
+        AVPacket pkt;
+
+        av_init_packet(&pkt);
+        pkt.dts=out.dts;
+        if(vStream->providePts()==true)
+        {
+            pkt.pts=out.pts;
+        }else
+        {
+            pkt.pts=pkt.dts;
+        }
+        pkt.stream_index=0;
+        pkt.data = buffer;
+        pkt.size = out.len;
+        if(out.flags & 0x10) // FIXME AVI_KEY_FRAME
+            pkt.flags |= AV_PKT_FLAG_KEY;
+        ret = writePacket( &pkt);
+        aprintf("[FF]Frame:%u, DTS=%08lu PTS=%08lu\n",written,out.dts,out.pts);
+        if(false==ret)
+        {
+            printf("[FF]Error writing video packet\n");
+            break;
+        }
+        if(!written)
+        {
+            audioDelay=vStream->getVideoDelay();
+            printf("[muxerFFmpeg::initVideo] Final audio delay: %" PRIu64" ms\n",audioDelay/1000);
+        }
+        written++;
     }
     delete [] buffer;
     if(false==ret)
