@@ -17,10 +17,19 @@
 #ifdef USE_VDPAU
 #include "../include/ADM_coreVdpauInternal.h"
 #include "ADM_dynamicLoading.h"
+#include <map>
 
+#if 0
+#define VDP_TRACK printf
+#else
+#define VDP_TRACK(...) {}
+#endif
 
 GUI_WindowInfo      admVdpau::myWindowInfo;
 
+
+typedef std::map<VdpVideoSurface ,bool >::iterator myIterator;
+
 namespace ADM_coreVdpau
 {
  VdpFunctions          funcs;
@@ -31,9 +40,31 @@ static ADM_LibWrapper        vdpauDynaLoader;
 static VdpDeviceCreateX11    *ADM_createVdpX11;
 static VdpGetProcAddress     *vdpProcAddress;
 static bool                  coreVdpWorking=false;
+static int                   videoSurfaceMaxWidth=0;
+static int                   videoSurfaceMaxHeight=0;
 static VdpPresentationQueueTarget  queueX11;
 
 
+
+static std::map <VdpVideoSurface ,bool >listOfAllocatedSurface;
+/**
+ * \fn admVdpau_exitCleanup
+ */
+bool admVdpau_exitCleanup()
+{
+    std::map <VdpVideoSurface ,bool > cpy=listOfAllocatedSurface;
+    int n=cpy.size();
+    printf("At exit, we have still %d surface\n",n);
+    myIterator it=cpy.begin();
+    for(;it!=cpy.end();it++)
+        admVdpau::surfaceDestroy(it->first);
+    printf("After cleanup we have  %d surface\n",(int)listOfAllocatedSurface.size());
+    
+    admVdpau::cleanup();
+    return true;
+}
+
+
 /**
     \fn getFunc
     \brief vdpau function pointers from ID
@@ -45,19 +76,25 @@ static void *getFunc(uint32_t id)
     return (void *)f;
 }
 /**
-
+    \fn getVdpDevice
 */
 void        *admVdpau::getVdpDevice(void)
 {
-        return (void *)ADM_coreVdpau::vdpDevice;
+        return (void *)(intptr_t)ADM_coreVdpau::vdpDevice;
 }
 /**
-
+    \fn getProcAddress
 */
 void        *admVdpau::getProcAddress(void)
 {
+    printf("==> GetProcAddress called\n");
     return (void *)vdpProcAddress;
 }
+VdpGetProcAddress        *admVdpau::getProcAddress2(void)
+{
+    printf("==> GetProcAddress called\n");
+    return vdpProcAddress;
+}
 
 /**
     \fn     init
@@ -65,10 +102,13 @@ void        *admVdpau::getProcAddress(void)
 */
 bool admVdpau::init(GUI_WindowInfo *x)
 {
+#define STR(x) #x
+#define MKSTRING(x) STR(x)
+#define VDPAU_LIBNAME libvdpau.so.1
     memset(&ADM_coreVdpau::funcs,0,sizeof(ADM_coreVdpau::funcs));
-    if(false==vdpauDynaLoader.loadLibrary("libvdpau.so"))
+    if(false==vdpauDynaLoader.loadLibrary(MKSTRING(VDPAU_LIBNAME)))
     {
-        ADM_info("Cannot load libvdpau.so\n");
+        ADM_info("Cannot load " MKSTRING(VDPAU_LIBNAME) "\n");
         return false;
     }
     ADM_createVdpX11=(VdpDeviceCreateX11*)vdpauDynaLoader.getSymbol("vdp_device_create_x11");
@@ -81,7 +121,12 @@ bool admVdpau::init(GUI_WindowInfo *x)
         return false;
     }
     // Now that we have the vdpProcAddress, time to get the functions....
-#define GetMe(fun,id)         ADM_coreVdpau::funcs.fun= (typeof(ADM_coreVdpau::funcs.fun))getFunc(id);ADM_assert(ADM_coreVdpau::funcs.fun); 
+#if 1 //def HAS_MOVE_SEMANTICS
+#define myTypeOf   decltype
+#else
+#define myTypeOf   typeof
+#endif
+#define GetMe(fun,id)         ADM_coreVdpau::funcs.fun= (myTypeOf(ADM_coreVdpau::funcs.fun))getFunc(id);ADM_assert(ADM_coreVdpau::funcs.fun); 
         
     GetMe(deviceDestroy,VDP_FUNC_ID_DEVICE_DESTROY);
     GetMe(getErrorString,VDP_FUNC_ID_GET_ERROR_STRING);
@@ -90,6 +135,7 @@ bool admVdpau::init(GUI_WindowInfo *x)
 
     GetMe(createSurface,VDP_FUNC_ID_VIDEO_SURFACE_CREATE);
     GetMe(destroySurface,VDP_FUNC_ID_VIDEO_SURFACE_DESTROY);
+    GetMe(querySurfaceCapabilities,VDP_FUNC_ID_VIDEO_SURFACE_QUERY_CAPABILITIES)
     GetMe(getDataSurface,VDP_FUNC_ID_VIDEO_SURFACE_GET_BITS_Y_CB_CR);
     GetMe(surfacePutBitsYCbCr,VDP_FUNC_ID_VIDEO_SURFACE_PUT_BITS_Y_CB_CR);
 
@@ -102,7 +148,7 @@ bool admVdpau::init(GUI_WindowInfo *x)
     GetMe(putBitsYV12OutputSurface,VDP_FUNC_ID_OUTPUT_SURFACE_PUT_BITS_Y_CB_CR);
     GetMe(putBitsCapsOutputSurface,VDP_FUNC_ID_OUTPUT_SURFACE_QUERY_PUT_BITS_Y_CB_CR_CAPABILITIES);
     GetMe(getBitsNativeOutputSurface,VDP_FUNC_ID_OUTPUT_SURFACE_GET_BITS_NATIVE);
-    
+    GetMe(queryDecoderCapabilities,VDP_FUNC_ID_DECODER_QUERY_CAPABILITIES);
     
 
     GetMe(presentationQueueDestroy,VDP_FUNC_ID_PRESENTATION_QUEUE_DESTROY);
@@ -135,6 +181,12 @@ bool admVdpau::init(GUI_WindowInfo *x)
         ADM_coreVdpau::funcs.getApiVersion(&version);
         ADM_info("[VDPAU] API : 0x%x, info : %s\n",version,versionString);
 
+    if(false==querySurfaceCapabilities(&videoSurfaceMaxWidth, &videoSurfaceMaxHeight) || !videoSurfaceMaxWidth || !videoSurfaceMaxHeight)
+    {
+        ADM_warning("Cannot get a valid maximum video surface size.\n");
+        return false;
+    }
+    ADM_info("VDPAU max supported video surface size: %d x %d\n", videoSurfaceMaxWidth, videoSurfaceMaxHeight);
     coreVdpWorking=true;
     myWindowInfo=*x;
 #if 0
@@ -197,42 +249,95 @@ bool admVdpau::isOperationnal(void)
     return coreVdpWorking;
 }
 /**
-    \fn
-    \brief
+    \fn getVideoSurfaceMaxWidth
 */
-VdpStatus admVdpau::decoderCreate( VdpDecoderProfile profile,    uint32_t  width,uint32_t  height,
-            uint32_t  max_references,VdpDecoder *      decoder)
+int admVdpau::getVideoSurfaceMaxWidth(void)
 {
-    CHECK(ADM_coreVdpau::funcs.decoderCreate(ADM_coreVdpau::vdpDevice,profile,width,height,max_references,decoder));
+    return videoSurfaceMaxWidth;
 }
 /**
-    \fn
+    \fn getVideoSurfaceMaxHeight
+*/
+int admVdpau::getVideoSurfaceMaxHeight(void)
+{
+    return videoSurfaceMaxHeight;
+}
+/**
+    \fn decoderCreate
+    \brief
+*/
+VdpStatus admVdpau::decoderCreate(  VdpDevice dev,VdpDecoderProfile profile,    uint32_t  width,uint32_t  height,
+            uint32_t  max_references,VdpDecoder *      decoder)
+{
+    VDP_TRACK("Creating decoder\n");
+    CHECK(ADM_coreVdpau::funcs.decoderCreate(dev,profile,dimensionRoundUp(width),dimensionRoundUp(height),max_references,decoder));
+}
+/**
+    \fn decoderDestroy
     \brief
 */
 VdpStatus  admVdpau::decoderDestroy(VdpDecoder decoder)
 {
+    VDP_TRACK("Destroying decoder\n");
     CHECK(ADM_coreVdpau::funcs.decoderDestroy(decoder));
 }
 /**
-    \fn
+    \fn surfaceCreate
     \brief
 */
 
 VdpStatus  admVdpau::surfaceCreate(uint32_t width,uint32_t height,VdpVideoSurface *surface)
 {
-    CHECK(ADM_coreVdpau::funcs.createSurface(ADM_coreVdpau::vdpDevice,VDP_CHROMA_TYPE_420,width,height,surface));
+    
+    if(!isOperationnal()) 
+        {ADM_error("vdpau is not operationnal\n");return VDP_STATUS_ERROR;}
+    VDP_TRACK("[VDPAU] Creating surface %d x %d\n",(int)width,(int)height);
+
+    int widthToUse=dimensionRoundUp(width);
+    int heightToUse=dimensionRoundUp(height);
+
+    if(widthToUse > videoSurfaceMaxWidth)
+    {
+        ADM_warning("Width %d exceeds max supported %d\n",widthToUse,videoSurfaceMaxWidth);
+        return VDP_STATUS_ERROR;
+    }
+    if(heightToUse > videoSurfaceMaxHeight)
+    {
+        ADM_warning("Height %d exceeds max supported %d\n",heightToUse,videoSurfaceMaxHeight);
+        return VDP_STATUS_ERROR;
+    }
+    VdpStatus r=ADM_coreVdpau::funcs.createSurface(ADM_coreVdpau::vdpDevice,VDP_CHROMA_TYPE_420,widthToUse,heightToUse,surface);
+    if(VDP_STATUS_OK!=r) 
+    {
+        ADM_warning("ADM_coreVdpau::funcs.createSurface(ADM_coreVdpau::vdpDevice,VDP_CHROMA_TYPE_420,width,height,surface) call failed with error=%s\n",getErrorString(r));
+       return r;
+    }
+    myIterator already = listOfAllocatedSurface.find(*surface);
+    if(already!=listOfAllocatedSurface.end())
+    {
+        ADM_assert("Doubly used vdpau surface\n");
+    }
+    listOfAllocatedSurface[*surface]=true;
+    return VDP_STATUS_OK;
 }
 /**
-    \fn
+    \fn surfaceDestroy
     \brief
 */
 
 VdpStatus  admVdpau::surfaceDestroy(VdpVideoSurface surface)
 {
+    // De we have it ?
+     myIterator already = listOfAllocatedSurface.find(surface);
+    if(already==listOfAllocatedSurface.end())
+    {
+        ADM_assert("Trying to destroy unallocated vdpau surface\n");
+    }
+    listOfAllocatedSurface.erase(already);
     CHECK(ADM_coreVdpau::funcs.destroySurface(surface));
 }
 /**
-    \fn
+    \fn getDataSurface
     \brief
 */
 
@@ -273,10 +378,11 @@ VdpStatus admVdpau::decoderRender(
     uint32_t                   bitstream_buffer_count,
     VdpBitstreamBuffer const * bitstream_buffers)
 {
+    VDP_TRACK("Calling VDPAU decoder , target surface=%d\n",target);
     CHECK(ADM_coreVdpau::funcs.decoderRender(decoder, target, (void * const *)info,bitstream_buffer_count, bitstream_buffers));
 }
 /**
-    \fn
+    \fn outputSurfaceCreate
     \brief
 */
 
@@ -298,7 +404,7 @@ VdpStatus admVdpau::outputSurfaceDestroy(    VdpOutputSurface surface)
     CHECK(ADM_coreVdpau::funcs.destroyOutputSurface(surface));
 }
 /**
-    \fn
+    \fn outPutSurfacePutBitsYV12
     \brief
 */
 
@@ -341,7 +447,7 @@ VdpStatus admVdpau::outputSurfaceGetBitsNative_FieldWeave(VdpOutputSurface     s
 
 
 /**
-    \fn
+    \fn outputSurfaceGetParameters
     \brief
 */
 VdpStatus admVdpau::outputSurfaceGetParameters(  VdpOutputSurface surface,    VdpRGBAFormat *  rgba_format,
@@ -357,7 +463,7 @@ VdpStatus admVdpau::surfaceGetParameters(VdpVideoSurface surface,VdpChromaType *
 }
 
 /**
-    \fn
+    \fn presentationQueueCreate
     \brief
 */
 
@@ -367,7 +473,7 @@ VdpStatus admVdpau::presentationQueueCreate(VdpPresentationQueue *queue)
 
 }
 /**
-    \fn
+    \fn presentationQueueDestroy
     \brief
 */
 
