@@ -19,6 +19,7 @@
 #include "ADM_ffmp43.h"
 #include "DIA_coreToolkit.h"
 #include "ADM_hwAccel.h"
+#include "prefs.h"
 
 #ifdef ADM_DEBUG
     #define LAV_VERBOSITY_LEVEL AV_LOG_DEBUG
@@ -42,14 +43,15 @@ extern "C"
     \fn clonePic
     \brief Convert AvFrame to ADMImage
 */
-uint8_t decoderFF::clonePic (AVFrame * src, ADMImage * out)
+uint8_t decoderFF::clonePic (AVFrame * src, ADMImage * out, bool swap)
 {
   uint32_t    u,v;
   ADM_assert(out->isRef());
   ADMImageRef *ref=out->castToRef();
   ref->_planes[0] = (uint8_t *) src->data[0];
   ref->_planeStride[0] = src->linesize[0];
-  if (decoderFF_params.swapUv)
+  swap = swap != decoderFF_params.swapUv;
+  if (swap)
     {
       u = 1;
       v = 2;
@@ -59,16 +61,16 @@ uint8_t decoderFF::clonePic (AVFrame * src, ADMImage * out)
       u = 2;
       v = 1;
     }
-  ref->_planes[1] = (uint8_t *) src->data[u];
-  ref->_planeStride[1] = src->linesize[u];
+  ref->_planes[1] = (uint8_t *) src->data[v];
+  ref->_planeStride[1] = src->linesize[v];
 
-  ref->_planes[2] = (uint8_t *) src->data[v];
-  ref->_planeStride[2] = src->linesize[v];
-  
-  _lastQ = 0;			//_context->quality;
-  out->_Qp = (src->quality * 32) / FF_LAMBDA_MAX;
+  ref->_planes[2] = (uint8_t *) src->data[u];
+  ref->_planeStride[2] = src->linesize[u];
+
+  // out->_Qp = (src->quality * 32) / FF_LAMBDA_MAX;
   out->flags = frameType ();
 
+#if 0 /* deprecated and dead, removed upstream */
   // Quant ?
   if (src->qstride && src->qscale_table && codecId != AV_CODEC_ID_H264)
     {
@@ -78,6 +80,7 @@ uint8_t decoderFF::clonePic (AVFrame * src, ADMImage * out)
       out->_qSize *= (_h + 15) >> 4;	// FixME?
     }
   else
+#endif
     {
       out->_qSize = out->_qStride = 0;
       out->quant = NULL;
@@ -86,6 +89,126 @@ uint8_t decoderFF::clonePic (AVFrame * src, ADMImage * out)
     //printf("[LAVC] Old pts :%"PRId64" new pts :%"PRId64"\n",out->Pts, pts_opaque);
     //printf("[LAVC] pts: %"PRIu64"\n",src->pts);
     out->Pts= (uint64_t)(pts_opaque);
+    out->_range=(src->color_range==AVCOL_RANGE_JPEG)? ADM_COL_RANGE_JPEG : ADM_COL_RANGE_MPEG;
+    {
+    ADM_colorPrimaries cp = admColPriFromLav(src->color_primaries);
+    if (cp != ADM_COL_PRI_UNSPECIFIED)
+        out->_colorPrim = cp;
+    ADM_colorTrC tc = admColTrcFromLav(src->color_trc);
+    if (tc != ADM_COL_TRC_UNSPECIFIED)
+        out->_colorTrc = tc;
+    ADM_colorSpace cs = admColSpcFromLav(src->colorspace);
+    if (cs != ADM_COL_SPC_UNSPECIFIED)
+        out->_colorSpace = cs;
+    }
+#ifdef ADM_DEBUG
+    if (out->_colorPrim == ADM_COL_PRI_UNSPECIFIED)
+        ADM_warning("Unspecified color primaries\n");
+    if (out->_colorTrc == ADM_COL_TRC_UNSPECIFIED)
+        ADM_warning("Unspecified color transfer characteristic\n");
+    if (out->_colorSpace == ADM_COL_SPC_UNSPECIFIED)
+        ADM_warning("Unspecified colorspace\n");
+#endif
+
+    // process side data
+    for (int i = 0; i < src->nb_side_data; i++)
+    {
+        AVFrameSideData *sd = src->side_data[i];
+        switch (sd->type)
+        {
+            case AV_FRAME_DATA_MASTERING_DISPLAY_METADATA:
+                {
+                    const AVMasteringDisplayMetadata *mastering_display;
+                    if (sd->size < sizeof(AVMasteringDisplayMetadata))
+                        break;
+                    mastering_display = (const AVMasteringDisplayMetadata *)sd->data;
+                    if (mastering_display->has_primaries)
+                    {
+                        for (int rgb=0; rgb<3; rgb++)
+                        {
+                            out->_hdrInfo.primaries[rgb][0] = av_q2d(mastering_display->display_primaries[rgb][0]);
+                            out->_hdrInfo.primaries[rgb][1] = av_q2d(mastering_display->display_primaries[rgb][1]);
+                        }
+                        out->_hdrInfo.whitePoint[0] = av_q2d(mastering_display->white_point[0]);
+                        out->_hdrInfo.whitePoint[1] = av_q2d(mastering_display->white_point[1]);
+#ifdef ADM_DEBUG
+                        ADM_info("HDR primaries: [%f,%f], [%f,%f], [%f,%f]\n",out->_hdrInfo.primaries[0][0],out->_hdrInfo.primaries[0][1], out->_hdrInfo.primaries[1][0],out->_hdrInfo.primaries[1][1], out->_hdrInfo.primaries[2][0],out->_hdrInfo.primaries[2][1]);
+                        ADM_info("HDR white point: [%f,%f]\n",out->_hdrInfo.whitePoint[0],out->_hdrInfo.whitePoint[1]);
+#endif
+                    }
+                    if (mastering_display->has_luminance)
+                    {
+                        out->_hdrInfo.maxLuminance = av_q2d(mastering_display->max_luminance);	// nits
+                        out->_hdrInfo.minLuminance = av_q2d(mastering_display->min_luminance);
+#ifdef ADM_DEBUG
+                        ADM_info("HDR max luminance: %f\n",out->_hdrInfo.maxLuminance);
+                        ADM_info("HDR min luminance: %f\n",out->_hdrInfo.minLuminance);
+#endif
+                    }
+                }
+                break;
+            case AV_FRAME_DATA_CONTENT_LIGHT_LEVEL:
+                {
+                    const AVContentLightMetadata *content_light;
+                    if (sd->size < sizeof(AVContentLightMetadata))
+                        break;
+                    content_light = (const AVContentLightMetadata *)sd->data;
+                    out->_hdrInfo.maxCLL = content_light->MaxCLL;	// integer, nits
+                    out->_hdrInfo.maxFALL = content_light->MaxFALL;
+#ifdef ADM_DEBUG
+                    ADM_info("HDR maxCLL: %f\n",out->_hdrInfo.maxCLL);
+                    ADM_info("HDR maxFALL: %f\n",out->_hdrInfo.maxFALL);
+#endif
+                }
+                break;
+            case AV_FRAME_DATA_DYNAMIC_HDR_PLUS:
+                {
+                    const AVDynamicHDRPlus *hdr_plus;
+                    if (sd->size < sizeof(AVDynamicHDRPlus))
+                        break;
+                    hdr_plus = (const AVDynamicHDRPlus *)sd->data;
+                    if (hdr_plus->num_windows > 0)
+                    {
+                        // CHEATING: assume num_windows==1; ATSC S34-301r2 A/341 Amendment – 2094-40 says: "The value of num_windows shall be 1."
+                        //           ignore bullshit parameters, like "elliptical pixel selector"; hope it's ok :)
+                        for (int rgb=0; rgb<3; rgb++)
+                        {
+                            out->_hdrInfo.maxSCL[rgb] = av_q2d(hdr_plus->params[0].maxscl[rgb]);
+                        }
+                        out->_hdrInfo.avgMaxRGB = av_q2d(hdr_plus->params[0].average_maxrgb);
+                        if (hdr_plus->params[0].tone_mapping_flag)
+                        {
+                            out->_hdrInfo.kneePoint[0] = av_q2d(hdr_plus->params[0].knee_point_x);
+                            out->_hdrInfo.kneePoint[1] = av_q2d(hdr_plus->params[0].knee_point_y);
+                        }
+                        if (hdr_plus->params[0].num_bezier_curve_anchors > 0)
+                        {
+                            for (int b=0; b<hdr_plus->params[0].num_bezier_curve_anchors; b++)
+                            {
+                                if (b >= 15)
+                                    break;
+                                out->_hdrInfo.bezierCurveAnchors[b] = av_q2d(hdr_plus->params[0].bezier_curve_anchors[b]);
+                            }
+                        }
+                        if (hdr_plus->params[0].color_saturation_mapping_flag)
+                        {
+                            out->_hdrInfo.colorSaturationWeight = av_q2d(hdr_plus->params[0].color_saturation_weight);
+                        }
+                    }
+                    out->_hdrInfo.targetMaxLuminance = av_q2d(hdr_plus->targeted_system_display_maximum_luminance);
+#ifdef ADM_DEBUG
+                    ADM_info("HDR maxSCL: %f, %f, %f\n",out->_hdrInfo.maxSCL[0],out->_hdrInfo.maxSCL[1],out->_hdrInfo.maxSCL[2]);
+                    ADM_info("HDR avgMaxRGB: %f\n",out->_hdrInfo.avgMaxRGB);
+                    ADM_info("HDR kneePoint: %f, %f\n",out->_hdrInfo.kneePoint[0],out->_hdrInfo.kneePoint[1]);
+                    ADM_info("HDR colorSaturationWeight: %f\n",out->_hdrInfo.colorSaturationWeight);
+                    ADM_info("HDR targetMaxLuminance: %f\n",out->_hdrInfo.targetMaxLuminance);
+#endif
+
+                    //other fields are not implemented as of ATSC S34-301r2 A/341 Amendment – 2094-40 
+                }
+                break;
+        }
+    }
 
     return 1;
 }
@@ -95,23 +218,31 @@ uint8_t decoderFF::clonePic (AVFrame * src, ADMImage * out)
 */
 void decoderFF::decoderMultiThread (void)
 {
-  uint32_t threads = 0;
+    static uint32_t sessionThreads = 0;
+    uint32_t threads = 1;
 
- // prefs->get(FEATURE_THREADING_LAVC, &threads);
-//#warning Fixme
-    threads=1;
-  if (threads == 0)
-	  threads = ADM_cpu_num_processors();
-
-  if (threads == 1)
-	  threads = 0;
-
-  if (threads)
-  {
-      printf ("[lavc] Enabling MT decoder with %u threads\n", threads);
-      _threads = threads;
-      _usingMT = 1;
-  }
+    if(false == prefs->get(FEATURES_THREADING_LAVC, &threads))
+        threads = 1;
+    if(!threads)
+        threads = ADM_cpu_num_processors();
+    if(threads > LAVC_MAX_SAFE_THREAD_COUNT)
+        threads = LAVC_MAX_SAFE_THREAD_COUNT;
+    if(!sessionThreads)
+    {
+        sessionThreads = threads;
+    }else
+    {
+        if((threads > 1) != (sessionThreads > 1))
+            ADM_warning("Restart application to %s multithreaded decoding.\n",(threads>1)? "enable" : "disable");
+        else
+            sessionThreads = threads;
+    }
+    if(sessionThreads > 1)
+    {
+        printf ("[lavc] Enabling MT decoder with %u threads\n", sessionThreads);
+        _threads = sessionThreads;
+        _usingMT = 1;
+    }
 }
 uint8_t decoderFF::getPARWidth (void)
 {
@@ -162,8 +293,8 @@ decoderFF::decoderFF (uint32_t w, uint32_t h,uint32_t fcc, uint32_t extraDataLen
             :decoders (w, h,fcc,extraDataLen,extraData,bpp)
 {
 	resetConfiguration();
-
   hurryUp=false;
+  _initCompleted=false;
   _drain=false;
   _done=false;
   _keepFeeding=false;
@@ -183,9 +314,11 @@ decoderFF::decoderFF (uint32_t w, uint32_t h,uint32_t fcc, uint32_t extraDataLen
 
   _frame=av_frame_alloc();
   if(!_frame)
-  {
-      ADM_assert (_frame);
-  }
+      return;
+
+  _packet = av_packet_alloc();
+  if(!_packet)
+      return;
 
   printf ("[lavc] Build: %d\n", LIBAVCODEC_BUILD);
   _extraDataCopy=NULL;
@@ -217,13 +350,10 @@ decoderFF::~decoderFF ()
         _context=NULL;
         printf ("[lavc] Destroyed\n");
     }
-  
-  if(_frame)
-  {
-      av_frame_free(&_frame);
-      _frame=NULL;
-  }
-  
+  // NULL-safe and will set the pointer to NULL
+  av_frame_free(&_frame);
+  av_packet_free(&_packet);
+
   if(_extraDataCopy)
   {
       delete [] _extraDataCopy;
@@ -232,6 +362,211 @@ decoderFF::~decoderFF ()
   delete hwDecoder;
   hwDecoder=NULL;
 }
+/**
+ * \fn initialized
+ */
+bool decoderFF::initialized(void)
+{
+    return _initCompleted;
+}
+
+
+ADM_pixelFormat decoderFF::admPixFrmtFromLav(AVPixelFormat pix_fmt, bool * swap)
+{
+  switch (pix_fmt)
+    {
+    case AV_PIX_FMT_YUV411P:
+      aprintf("pixel format is AV_PIX_FMT_YUV411P --> ADM_PIXFRMT_YUV411\n");
+      return ADM_PIXFRMT_YUV411;
+    case AV_PIX_FMT_YUYV422:
+      aprintf("pixel format is AV_PIX_FMT_YUYV422 --> ADM_PIXFRMT_YUV422\n");
+      return ADM_PIXFRMT_YUV422;
+    case AV_PIX_FMT_YUV422P:
+    case AV_PIX_FMT_YUVJ422P:
+      aprintf("pixel format is AV_PIX_FMT_YUV422P or AV_PIX_FMT_YUVJ422P --> ADM_PIXFRMT_YUV422P\n");
+      return ADM_PIXFRMT_YUV422P;
+    case AV_PIX_FMT_GRAY8:
+      aprintf("pixel format is AV_PIX_FMT_GRAY8 --> ADM_PIXFRMT_Y8\n");
+      return ADM_PIXFRMT_Y8;
+    case AV_PIX_FMT_YUV444P:
+    case AV_PIX_FMT_YUVJ444P:
+      aprintf("pixel format is AV_PIX_FMT_YUV444P or AV_PIX_FMT_YUVJ444P --> ADM_PIXFRMT_YUV444\n");
+      return ADM_PIXFRMT_YUV444;
+    case AV_PIX_FMT_YUV420P:
+    case AV_PIX_FMT_YUVJ420P:
+    case AV_PIX_FMT_YUVA420P:
+      // Default is YV12 or I420
+      // In that case depending on swap u/v
+      // we do it or not
+      aprintf("pixel format is AV_PIX_FMT_YUV420P, AV_PIX_FMT_YUVJ420P or AV_PIX_FMT_YUVA420P --> ADM_PIXFRMT_YV12\n");
+      if (swap)
+          *swap = true;
+      return ADM_PIXFRMT_YV12;
+    case AV_PIX_FMT_BGR24:
+      aprintf("pixel format is AV_PIX_FMT_BGR24 --> ADM_PIXFRMT_BGR24\n");
+      return ADM_PIXFRMT_BGR24;
+    case AV_PIX_FMT_RGB24:
+      aprintf("pixel format is AV_PIX_FMT_RGB24 --> ADM_PIXFRMT_RGB24\n");
+      return ADM_PIXFRMT_RGB24;
+    case AV_PIX_FMT_GBRP:
+      aprintf("pixel format is AV_PIX_FMT_GBRP --> ADM_PIXFRMT_GBR24P\n");
+      return ADM_PIXFRMT_GBR24P;
+    case AV_PIX_FMT_BGR0:
+    case AV_PIX_FMT_BGRA:
+      aprintf("pixel format is AV_PIX_FMT_BGR0 or AV_PIX_FMT_BGRA --> ADM_PIXFRMT_BGR32A\n");
+      return ADM_PIXFRMT_BGR32A;
+    case AV_PIX_FMT_RGBA: // ???PIX_FMT_RGBA32:
+      aprintf("pixel format is AV_PIX_FMT_RGBA --> ADM_PIXFRMT_RGB32A\n");
+      return ADM_PIXFRMT_RGB32A;
+    case AV_PIX_FMT_RGB555:
+      aprintf("pixel format is AV_PIX_FMT_RGB555 --> ADM_PIXFRMT_RGB555\n");
+      return ADM_PIXFRMT_RGB555;
+#if 0
+    case AV_PIX_FMT_VDPAU_MPEG1:
+    case AV_PIX_FMT_VDPAU_MPEG2:
+    case AV_PIX_FMT_VDPAU_WMV3:
+    case AV_PIX_FMT_VDPAU_VC1:
+    case AV_PIX_FMT_VDPAU_H264:
+    case AV_PIX_FMT_VDPAU:
+        return ADM_PIXFRMT_VDPAU;
+    case AV_PIX_FMT_VAAPI:
+        return ADM_PIXFRMT_LIBVA;
+        
+#ifdef USE_XVBA        
+    case AV_PIX_FMT_XVBA_VLD:
+        return ADM_PIXFRMT_XVBA;
+#endif        
+#endif
+  case AV_PIX_FMT_YUV444P10LE:
+        return ADM_PIXFRMT_YUV444_10BITS;
+  case AV_PIX_FMT_YUV422P10LE:
+        return ADM_PIXFRMT_YUV422_10BITS;
+  case   AV_PIX_FMT_P010LE:      
+        return ADM_PIXFRMT_NV12_10BITS;
+  case  AV_PIX_FMT_YUV420P10LE:
+        return ADM_PIXFRMT_YUV420_10BITS;
+  case  AV_PIX_FMT_YUV420P12LE:
+        return ADM_PIXFRMT_YUV420_12BITS;
+  case  AV_PIX_FMT_YUV444P12LE:
+        return ADM_PIXFRMT_YUV444_12BITS;
+    default:
+        return ADM_PIXFRMT_INVALID;
+    }
+}
+
+ADM_colorPrimaries decoderFF::admColPriFromLav(AVColorPrimaries color_primaries)
+{
+    switch(color_primaries)
+    {
+        case AVCOL_PRI_BT709:
+            return ADM_COL_PRI_BT709;
+        case AVCOL_PRI_BT470M:
+            return ADM_COL_PRI_BT470M;
+        case AVCOL_PRI_BT470BG:
+            return ADM_COL_PRI_BT470BG;
+        case AVCOL_PRI_SMPTE170M:
+            return ADM_COL_PRI_SMPTE170M;
+        case AVCOL_PRI_SMPTE240M:
+            return ADM_COL_PRI_SMPTE240M;
+        case AVCOL_PRI_FILM:
+            return ADM_COL_PRI_FILM;
+        case AVCOL_PRI_BT2020:
+            return ADM_COL_PRI_BT2020;
+        case AVCOL_PRI_SMPTE428:
+        //case AVCOL_PRI_SMPTEST428_1:
+            return ADM_COL_PRI_SMPTE428;
+        case AVCOL_PRI_SMPTE431:
+            return ADM_COL_PRI_SMPTE431;
+        case AVCOL_PRI_SMPTE432:
+            return ADM_COL_PRI_SMPTE432;
+        case AVCOL_PRI_EBU3213:
+        //case AVCOL_PRI_JEDEC_P22:
+            return ADM_COL_PRI_EBU3213;
+        default:
+            return ADM_COL_PRI_UNSPECIFIED;
+    }
+}
+
+ADM_colorTrC decoderFF::admColTrcFromLav(AVColorTransferCharacteristic color_trc)
+{
+    switch(color_trc)
+    {
+        case AVCOL_TRC_BT709:
+            return ADM_COL_TRC_BT709;
+        case AVCOL_TRC_GAMMA22:
+            return ADM_COL_TRC_GAMMA22;
+        case AVCOL_TRC_GAMMA28:
+            return ADM_COL_TRC_GAMMA28;
+        case AVCOL_TRC_SMPTE170M:
+            return ADM_COL_TRC_SMPTE170M;
+        case AVCOL_TRC_SMPTE240M:
+            return ADM_COL_TRC_SMPTE240M;
+        case AVCOL_TRC_LINEAR:
+            return ADM_COL_TRC_LINEAR;
+        case AVCOL_TRC_LOG:
+            return ADM_COL_TRC_LOG;
+        case AVCOL_TRC_LOG_SQRT:
+            return ADM_COL_TRC_LOG_SQRT;
+        case AVCOL_TRC_IEC61966_2_4:
+            return ADM_COL_TRC_IEC61966_2_4;
+        case AVCOL_TRC_BT1361_ECG:
+            return ADM_COL_TRC_BT1361_ECG;
+        case AVCOL_TRC_IEC61966_2_1:
+            return ADM_COL_TRC_IEC61966_2_1;
+        case AVCOL_TRC_BT2020_10:
+            return ADM_COL_TRC_BT2020_10;
+        case AVCOL_TRC_BT2020_12:
+            return ADM_COL_TRC_BT2020_12;
+        case AVCOL_TRC_SMPTE2084:
+        //case AVCOL_TRC_SMPTEST2084:
+            return ADM_COL_TRC_SMPTE2084;
+        case AVCOL_TRC_SMPTE428:
+        //case AVCOL_TRC_SMPTEST428_1:
+            return ADM_COL_TRC_SMPTE428;
+        case AVCOL_TRC_ARIB_STD_B67:
+            return ADM_COL_TRC_ARIB_STD_B67;
+        default:
+            return ADM_COL_TRC_UNSPECIFIED;
+    }
+}
+
+ADM_colorSpace decoderFF::admColSpcFromLav(AVColorSpace colorspace)
+{
+    switch(colorspace)
+    {
+        case AVCOL_SPC_RGB:
+            return ADM_COL_SPC_sRGB;
+        case AVCOL_SPC_BT709:
+            return ADM_COL_SPC_BT709;
+        case AVCOL_SPC_FCC:
+            return ADM_COL_SPC_FCC;
+        case AVCOL_SPC_BT470BG:
+            return ADM_COL_SPC_BT470BG;
+        case AVCOL_SPC_SMPTE170M:
+            return ADM_COL_SPC_SMPTE170M;
+        case AVCOL_SPC_SMPTE240M:
+            return ADM_COL_SPC_SMPTE240M;
+        case AVCOL_SPC_YCGCO:
+        //case AVCOL_SPC_YCOCG:
+            return ADM_COL_SPC_YCGCO;
+        case AVCOL_SPC_BT2020_NCL:
+            return ADM_COL_SPC_BT2020_NCL;
+        case AVCOL_SPC_BT2020_CL:
+            return ADM_COL_SPC_BT2020_CL;
+        case AVCOL_SPC_SMPTE2085:
+            return ADM_COL_SPC_SMPTE2085;
+        case AVCOL_SPC_CHROMA_DERIVED_NCL:
+            return ADM_COL_SPC_CHROMA_DERIVED_NCL;
+        case AVCOL_SPC_CHROMA_DERIVED_CL:
+            return ADM_COL_SPC_CHROMA_DERIVED_CL;
+        case AVCOL_SPC_ICTCP:
+            return ADM_COL_SPC_ICTCP;
+        default:
+            return ADM_COL_SPC_UNSPECIFIED;
+    }
+}
+
+
 /**
  * 
  * @param pic
@@ -252,10 +587,10 @@ uint32_t decoderFF::admFrameTypeFromLav (AVFrame *pic)
                 SET (AVI_KEY_FRAME);
                 if (!pic->key_frame)
                   {
-                    if (codecId == AV_CODEC_ID_H264)
+                    if (codecId == AV_CODEC_ID_H264 || codecId == AV_CODEC_ID_FFV1)
                         SET (AVI_P_FRAME)
                     else
-                      ADM_info ("\n But keyframe is not set\n");
+                        ADM_info("Picture type is I, but keyframe is not set\n");
                   }
                 break;
         case AV_PICTURE_TYPE_S:
@@ -348,8 +683,8 @@ bool decoderFF::decodeErrorHandler(int code)
                 return false;
             default:
             {
-                char er[2048]={0};
-                av_make_error_string(er, sizeof(er)-1, code);
+                char er[AV_ERROR_MAX_STRING_SIZE]={0};
+                av_make_error_string(er, AV_ERROR_MAX_STRING_SIZE, code);
                 ADM_warning("Error %d in lavcodec (%s)\n",code,er);
                 return false;
             }
@@ -368,12 +703,13 @@ bool   decoderFF::uncompress (ADMCompressedImage * in, ADMImage * out)
 {
   int ret = 0;
   out->_noPicture = 0;
-  if(hwDecoder)
+  out->_Qp = ADM_IMAGE_UNKNOWN_QP;
+  if(hwDecoder && !_usingMT)
         return hwDecoder->uncompress(in,out);
  
   //printf("Frame size : %d\n",in->dataLength);
 
-    if (in->dataLength == 0 && !_allowNull) // Null frame, silently skipped
+    if (!_drain && in->dataLength == 0 && !_allowNull) // Null frame, silently skipped
     {
         printf ("[Codec] null frame\n");
         out->_noPicture = 1;
@@ -395,34 +731,49 @@ bool   decoderFF::uncompress (ADMCompressedImage * in, ADMImage * out)
         }
     }else
     {
-        AVPacket pkt;
-        av_init_packet(&pkt);
-        pkt.data=in->data;
-        pkt.size=in->dataLength;
-        if(in->flags&AVI_KEY_FRAME)
-            pkt.flags=AV_PKT_FLAG_KEY;
-        else
-            pkt.flags=0;
+        _packet->data = in->data;
+        _packet->size = in->dataLength;
 
-        avcodec_send_packet(_context, &pkt);
+        if(in->flags&AVI_KEY_FRAME)
+            _packet->flags = AV_PKT_FLAG_KEY;
+        else
+            _packet->flags = 0;
+
+        ret = avcodec_send_packet(_context, _packet);
+
+        if(ret)
+        {
+            char er[AV_ERROR_MAX_STRING_SIZE]={0};
+            av_make_error_string(er, AV_ERROR_MAX_STRING_SIZE, ret);
+            ADM_warning("Ignoring error %d submitting packet to decoder (\"%s\")\n",ret,er);
+        }
+
+        av_packet_unref(_packet);
+
+        // HW accel may be setup by now if this has been the very first packet. In this case
+        // ask the hw decoder to collect the decoded picture.
+        if(hwDecoder)
+        {
+            hwDecoder->skipSendFrame();
+            return hwDecoder->uncompress(in,out);
+        }
     }
 
     ret = avcodec_receive_frame(_context, _frame);
 
     out->_qStride = 0; // Default = no quant
 
-    if (ret && !hurryUp)
+    if (!_drain && ret && !hurryUp)
     {
         // Some encoder code a vop header with the
         // vop flag set to 0
         // it is meant to mean frame skipped but very dubious
 #define MPEG4_EMPTY_FRAME_THRESHOLD 19 // MAX_NVOP_SIZE
 #define FRAPS_EMPTY_FRAME_THRESHOLD  8 // assumption
-        if ((in->dataLength <= MPEG4_EMPTY_FRAME_THRESHOLD && codecId == AV_CODEC_ID_MPEG4) ||
-            (in->dataLength <= FRAPS_EMPTY_FRAME_THRESHOLD && codecId == AV_CODEC_ID_MPEG4))
+        if ((codecId == AV_CODEC_ID_MPEG4 && in->dataLength <= MPEG4_EMPTY_FRAME_THRESHOLD) ||
+            (codecId == AV_CODEC_ID_FRAPS && in->dataLength <= FRAPS_EMPTY_FRAME_THRESHOLD))
         {
             printf ("[lavc] Probably placeholder frame (data length: %u)\n",in->dataLength);
-            out->_Qp = 2;
             out->Pts=ADM_NO_PTS; // not sure
             out->_noPicture = 1;
             return true;
@@ -456,97 +807,25 @@ bool   decoderFF::uncompress (ADMCompressedImage * in, ADMImage * out)
         _frame->reordered_opaque=(int64_t)in->demuxerPts;
     }
 
-  // We have an image....
-  switch (_context->pix_fmt)
+    // We have an image....
+    bool swap = false;
+    ADM_pixelFormat pix_fmt;
+    pix_fmt = admPixFrmtFromLav(_context->pix_fmt, &swap);
+    if (pix_fmt == ADM_PIXFRMT_INVALID)
     {
-    case AV_PIX_FMT_YUV411P:
-      out->_colorspace = ADM_COLOR_YUV411;
-      aprintf("colorspace is AV_PIX_FMT_YUV411P --> ADM_COLOR_YUV411\n");
-      break;
-    case AV_PIX_FMT_YUYV422:
-      out->_colorspace = ADM_COLOR_YUV422;
-      aprintf("colorspace is AV_PIX_FMT_YUYV422 --> ADM_COLOR_YUV422\n");
-      break;
-    case AV_PIX_FMT_YUV422P:
-    case AV_PIX_FMT_YUVJ422P:
-      out->_colorspace = ADM_COLOR_YUV422P;
-      aprintf("colorspace is AV_PIX_FMT_YUV422P or AV_PIX_FMT_YUVJ422P --> ADM_COLOR_YUV422P\n");
-      break;
-    case AV_PIX_FMT_GRAY8:
-      out->_colorspace = ADM_COLOR_Y8;
-      aprintf("colorspace is AV_PIX_FMT_GRAY8 --> ADM_COLOR_Y8\n");
-      break;
-    case AV_PIX_FMT_YUV444P:
-    case AV_PIX_FMT_YUVJ444P:
-      out->_colorspace = ADM_COLOR_YUV444;
-      aprintf("colorspace is AV_PIX_FMT_YUV444P or AV_PIX_FMT_YUVJ444P --> ADM_COLOR_YUV444\n");
-      break;
-    case AV_PIX_FMT_YUV420P:
-    case AV_PIX_FMT_YUVJ420P:
-    case AV_PIX_FMT_YUVA420P:
-      // Default is YV12 or I420
-      // In that case depending on swap u/v
-      // we do it or not
-      out->_colorspace = ADM_COLOR_YV12;
-      aprintf("colorspace is AV_PIX_FMT_YUV420P, AV_PIX_FMT_YUVJ420P or AV_PIX_FMT_YUVA420P --> ADM_COLOR_YV12\n");
-      break;
-    case AV_PIX_FMT_BGR24:
-      out->_colorspace = ADM_COLOR_BGR24;
-      aprintf("colorspace is AV_PIX_FMT_BGR24 --> ADM_COLOR_BGR24\n");
-      break;
-    case AV_PIX_FMT_RGB24:
-      out->_colorspace = ADM_COLOR_RGB24;
-      aprintf("colorspace is AV_PIX_FMT_RGB24 --> ADM_COLOR_RGB24\n");
-      break;
-    case AV_PIX_FMT_BGR0:
-    case AV_PIX_FMT_BGRA:
-      out->_colorspace = ADM_COLOR_BGR32A;
-      aprintf("colorspace is AV_PIX_FMT_BGR0 or AV_PIX_FMT_BGRA --> ADM_COLOR_BGR32A\n");
-      break;
-    case AV_PIX_FMT_RGBA: // ???PIX_FMT_RGBA32:
-      out->_colorspace = ADM_COLOR_RGB32A;
-      aprintf("colorspace is AV_PIX_FMT_RGBA --> ADM_COLOR_RGB32A\n");
-      break;
-    case AV_PIX_FMT_RGB555:
-      out->_colorspace = ADM_COLOR_RGB555;
-      aprintf("colorspace is AV_PIX_FMT_RGB555 --> ADM_COLOR_RGB555\n");
-      break;
-#if 0
-    case AV_PIX_FMT_VDPAU_MPEG1:
-    case AV_PIX_FMT_VDPAU_MPEG2:
-    case AV_PIX_FMT_VDPAU_WMV3:
-    case AV_PIX_FMT_VDPAU_VC1:
-    case AV_PIX_FMT_VDPAU_H264:
-    case AV_PIX_FMT_VDPAU:
-        out->_colorspace=ADM_COLOR_VDPAU;
-        break;        
-    case AV_PIX_FMT_VAAPI:
-        out->_colorspace=ADM_COLOR_LIBVA;
-        break;
-        
-#ifdef USE_XVBA        
-    case AV_PIX_FMT_XVBA_VLD:
-        out->_colorspace=ADM_COLOR_XVBA;
-        break;
-#endif        
-#endif
-  case AV_PIX_FMT_YUV444P10LE:
-        out->_colorspace=ADM_COLOR_YUV444_10BITS;
-        break;      
-  case AV_PIX_FMT_YUV422P10LE:
-        out->_colorspace=ADM_COLOR_YUV422_10BITS;
-        break;
-  case   AV_PIX_FMT_P010LE:      
-        out->_colorspace= ADM_COLOR_NV12_10BITS;
-        break;
-  case  AV_PIX_FMT_YUV420P10LE      :  
-        out->_colorspace=ADM_COLOR_YV12_10BITS;
-        break;
-    default:
-      printf ("[lavc] Unhandled colorspace: %d (AV_PIX_FMT_YUV444P10BE=%d)\n", _context->pix_fmt,AV_PIX_FMT_YUV444P10BE);
-      return 0;
+        printf ("[lavc] Unhandled pixel format: %d (AV_PIX_FMT_YUV444P10BE=%d)\n", _context->pix_fmt,AV_PIX_FMT_YUV444P10BE);
+        return 0;
     }
-    clonePic (_frame, out);
+    out->_pixfrmt = pix_fmt;
+
+    // make sure the output is not marked as a hw image
+    int count = 0;
+    while(out->refType != ADM_HW_NONE && count < 32 /* arbitrary limit */)
+    {
+        out->hwDecRefCount();
+        count++;
+    }
+    clonePic(_frame, out, swap);
     //printf("[AvCodec] Pts : %"PRIu64" Out Pts:%"PRIu64" \n",_frame.pts,out->Pts);
 
   return 1;
@@ -571,8 +850,12 @@ decoderFF (w, h,fcc,extraDataLen,extraData,bpp)
   _refCopy = 1;			// YUV420 only
   _setFcc=true;
   decoderMultiThread ();
+  if(_usingMT && _threads > 2)
+  {
+        ADM_warning("%u threads requested, reducing to 2\n",_threads);
+        _threads=2; // else we cannot handle placeholder frames following a keyframe. FIXME
+  }
   WRAP_Open (AV_CODEC_ID_MPEG4);
-  
 }
 bool decoderFFMpeg4::uncompress(ADMCompressedImage *in, ADMImage *out)
 {
@@ -609,6 +892,14 @@ bool decoderFFficv::uncompress(ADMCompressedImage *in, ADMImage *out)
     return false;
 }
 //************************************
+decoderFFMpg1::decoderFFMpg1(uint32_t w, uint32_t h, uint32_t fcc, uint32_t extraDataLen, uint8_t *extraData, uint32_t bpp):
+                decoderFF(w,h,fcc,extraDataLen,extraData,bpp)
+{
+    _refCopy = 1;
+    decoderMultiThread();
+    WRAP_Open(AV_CODEC_ID_MPEG1VIDEO)
+}
+
 decoderFFMpeg12::decoderFFMpeg12 (uint32_t w, uint32_t h,uint32_t fcc, uint32_t extraDataLen, uint8_t *extraData,uint32_t bpp):
                 decoderFF (w, h,fcc,extraDataLen,extraData,bpp)
 {
