@@ -13,6 +13,7 @@
  ***************************************************************************/
 #include "ADM_cpp.h"
 #include <math.h>
+#include <climits>
 
 #include "ADM_default.h"
 #include "ADM_Video.h"
@@ -63,7 +64,6 @@ uint8_t mkvHeader::videoIndexer(ADM_ebml_file *parser)
     {
         parser->seek(_clusters[clusters].pos);
         ADM_ebml_file cluster(parser,_clusters[clusters].size);
-        int thiscluster=0;
         while(!cluster.finished())
         {
             if(!work->isAlive())
@@ -72,10 +72,11 @@ uint8_t mkvHeader::videoIndexer(ADM_ebml_file *parser)
                 break;
             }
             work->update(clusters,nbClusters);
-            cluster.readElemId(&id,&len);
+            if(!cluster.readElemId(&id,&len))
+                break;
             if(!ADM_searchMkvTag( (MKV_ELEM_ID)id,&ss,&type))
             {
-                printf("[MKV] Tag 0x%" PRIx64" not found (len %" PRIu64")\n",id,len);
+                printf("[MKV] Cluster %d, tag 0x%" PRIx64" at 0x%llx not found (len %" PRIu64")\n",clusters,id,cluster.tell()-2,len);
                 cluster.skip(len);
                 continue;
             }
@@ -96,10 +97,11 @@ uint8_t mkvHeader::videoIndexer(ADM_ebml_file *parser)
                     ADM_ebml_file blockGroup(parser,len);
                     while(!blockGroup.finished())
                     {
-                        blockGroup.readElemId(&id,&len);
+                        if(!blockGroup.readElemId(&id,&len))
+                            break;
                         if(!ADM_searchMkvTag( (MKV_ELEM_ID)id,&ss,&type))
                         {
-                            printf("[MKV] Tag 0x%" PRIx64" not found (len %" PRIu64")\n",id,len);
+                            printf("[MKV] Block group in cluster %d, tag 0x%" PRIx64" at 0x%llx not found (len %" PRIu64")\n",clusters,id,blockGroup.tell()-2,len);
                             blockGroup.skip(len);
                             continue;
                         }
@@ -114,14 +116,13 @@ uint8_t mkvHeader::videoIndexer(ADM_ebml_file *parser)
                                 break;
                         }
                     }
-                    thiscluster++;
                 }
                 break; // Block Group
             }
         }
    // printf("[MKV] ending cluster at 0x%llx\n",segment.tell());
     }
-    printf("Found %" PRIu32" images in this cluster\n",(uint32_t)VIDEO.index.size());
+    printf("Found %" PRIu32" images in this video\n",(uint32_t)VIDEO.index.size());
     delete work;
     delete [] readBuffer;
     readBuffer=NULL;
@@ -183,6 +184,7 @@ static bool canRederiveFrameType(uint32_t fcc)
 {
     if(isMpeg4Compatible(fcc) || isVC1Compatible(fcc)) return true;
     if(isH264Compatible(fcc)) return true;
+    if(isH265Compatible(fcc)) return true;
     if(isMpeg12Compatible(fcc)) return true;
     return false;
 
@@ -205,6 +207,7 @@ uint8_t mkvHeader::addIndexEntry(uint32_t track,ADM_ebml_file *parser,uint64_t w
     ix.Pts=timecodeMS*_timeBase;
     //printf("Track=%d, timecode=%d timeBase=%d, Pts=%d\n",track,(int)timecodeMS,(int)_timeBase,ix.Pts);
     uint32_t rpt=_tracks[0].headerRepeatSize;
+    uint32_t frameNo = Track->index.size();
 
     // expand buffer if needed
     if(size>readBufferSize)
@@ -230,10 +233,27 @@ uint8_t mkvHeader::addIndexEntry(uint32_t track,ADM_ebml_file *parser,uint64_t w
             parser->readBin(readBuffer+rpt,size-3);
 
             // Search the frame type...
-            uint32_t nb,timeinc=16;
+            uint32_t timeIncBits=0;
+            const uint32_t u32 = sizeof(uint32_t);
+
+            if(_tracks[0].paramCache && _tracks[0].paramCacheSize == u32)
+                memcpy(&timeIncBits,_tracks[0].paramCache,u32);
+
             ADM_vopS vops[10];
             vops[0].type=AVI_KEY_FRAME;
-            ADM_searchVop(readBuffer,readBuffer+rpt+size-3,&nb,vops, &timeinc);
+
+            if(ADM_searchVop(readBuffer,readBuffer+rpt+size-3,10,vops,&timeIncBits))
+            {
+                if(!_tracks[0].paramCache)
+                {
+                    _tracks[0].paramCache = new uint8_t[u32];
+                    _tracks[0].paramCacheSize = u32;
+                }
+                memcpy(_tracks[0].paramCache,&timeIncBits,u32);
+            }else
+            {
+                ADM_warning("No VOP at index entry %u, corrupted data? Size: %d\n",frameNo,rpt+size-3);
+            }
             ix.flags=vops[0].type;
 
         }else if(isH264Compatible(_videostream.fccHandler))
@@ -297,7 +317,7 @@ uint8_t mkvHeader::addIndexEntry(uint32_t track,ADM_ebml_file *parser,uint64_t w
                         MATCH(refFrames)
                         if(!match)
                         {
-                            ADM_warning("Codec parameters change on the fly at frame %u, expect problems.\n",Track->index.size());
+                            ADM_warning("Codec parameters change on the fly at frame %u, expect problems.\n",frameNo);
                             // Nevertheless, update the cached info
                             memcpy(_tracks[0].infoCache, &info2, sz);
                         }
@@ -310,33 +330,73 @@ uint8_t mkvHeader::addIndexEntry(uint32_t track,ADM_ebml_file *parser,uint64_t w
                     _tracks[0].paramCacheSize=inBandSpsLen;
                 }
             }
+            bool r = false;
             if(AnnexB)
-                extractH264FrameType_startCode(readBuffer, rpt+size-3, &flags, NULL, info, &_H264Recovery);
+                r = extractH264FrameType_startCode(readBuffer, rpt+size-3, &flags, NULL, info, &_H264Recovery);
             else
-                extractH264FrameType(readBuffer, rpt+size-3, nalSize, &flags, NULL, info, &_H264Recovery);
-            if(flags & AVI_KEY_FRAME)
+                r = extractH264FrameType(readBuffer, rpt+size-3, nalSize, &flags, NULL, info, &_H264Recovery);
+            if(r)
             {
-                printf("[MKV/H264] Frame %" PRIu32" is a keyframe\n",(uint32_t)Track->index.size());
+                if(flags & AVI_KEY_FRAME)
+                {
+                    if(flags & AVI_FIELD_STRUCTURE)
+                    {
+                        if(Track->_secondField)
+                        {
+                            printf("[MKV/H264] Clearing keyframe flag from second field at index entry %" PRIu32"\n",frameNo);
+                            flags &= ~AVI_KEY_FRAME;
+                        }
+                        Track->_secondField = !Track->_secondField;
+                    }
+                    setFlag(frameNo,flags);
+                    if(flags & AVI_KEY_FRAME)
+                        printf("[MKV/H264] Frame %" PRIu32" is a keyframe\n",frameNo);
+                }else
+                    Track->_secondField = false;
+                ix.flags=flags;
+                if(Track->index.size()) ix.Dts=ADM_NO_PTS;
             }
-            ix.flags=flags;
-            if(Track->index.size()) ix.Dts=ADM_NO_PTS;
-
-        }/*else if(isH265Compatible(_videostream.fccHandler))
+        }else if(isH265Compatible(_videostream.fccHandler))
         {
             uint32_t flags=AVI_KEY_FRAME;
 
             if(rpt)
                 memcpy(readBuffer,_tracks[0].headerRepeat,rpt);
             parser->readBin(readBuffer+rpt,size-3);
-            extractH265FrameType(2,readBuffer,rpt+size-3,&flags); // Nal size is not used in that case
-            if(flags & AVI_KEY_FRAME)
-            {
-                printf("[MKV/H265] Frame %" PRIu32" is a keyframe\n",(uint32_t)Track->index.size());
-            }
-            ix.flags=flags;
-            if(Track->index.size()) ix.Dts=ADM_NO_PTS;
 
-        }*/else if(isMpeg12Compatible(_videostream.fccHandler))
+            // Deal with Matroska files containing Annex-B type of HEVC stream
+            bool AnnexB=false;
+            if(!_tracks[0].extraDataLen && rpt+size > 3 && !readBuffer[0] && !readBuffer[1])
+            {
+                uint32_t mark=(readBuffer[2] << 8) + readBuffer[3];
+                if(mark == 1 || (mark > 0xFF && mark < 0x200 && rpt+size-3 != mark+4))
+                    AnnexB=true;
+            }
+
+            uint32_t nalSize=0;
+            if(_tracks[0].extraDataLen)
+                nalSize = ADM_getNalSizeH265(_tracks[0].extraData, _tracks[0].extraDataLen);
+
+            ADM_SPSinfoH265 info; // TODO: check for inband VPS/PPS/SPS changing on the fly
+            int poc = INT_MIN;
+
+            bool r = false;
+            if(AnnexB)
+                r = extractH265FrameType_startCode(readBuffer, rpt+size-3, &info, &flags, &poc);
+            else
+                r = extractH265FrameType(readBuffer, rpt+size-3, nalSize, &info, &flags, &poc);
+
+            if(r)
+            {
+                if(flags & AVI_KEY_FRAME)
+                {
+                    printf("[MKV/H265] Frame %" PRIu32" is a keyframe",(uint32_t)Track->index.size());
+                    printf("%s\n",(flags & AVI_IDR_FRAME)? " (IDR)" : " (non-IDR)");
+                }
+                ix.flags=flags;
+                if(Track->index.size()) ix.Dts=ADM_NO_PTS;
+            }
+        }else if(isMpeg12Compatible(_videostream.fccHandler))
         {
             if(rpt)
                 memcpy(readBuffer,_tracks[0].headerRepeat,rpt);
@@ -462,10 +522,11 @@ bool mkvHeader::readCue(ADM_ebml_file *parser)
     ADM_ebml_file cues(parser,vlen);
     while(!cues.finished())
     {
-        cues.readElemId(&id,&len);
+        if(!cues.readElemId(&id,&len))
+            continue;
         if(!ADM_searchMkvTag( (MKV_ELEM_ID)id,&ss,&type))
         {
-            printf("[MKV] Tag 0x%" PRIx64" not found (len %" PRIu64")\n",id,len);
+            printf("[MKV] Tag 0x%" PRIx64" in CUES not found (len %" PRIu64")\n",id,len);
             cues.skip(len);
             continue;
         }
@@ -477,7 +538,8 @@ bool mkvHeader::readCue(ADM_ebml_file *parser)
         }
         ADM_ebml_file cue(&cues,len);
         // Cue TIME normally
-        cue.readElemId(&id,&len);
+        if(!cue.readElemId(&id,&len))
+            continue;
         if(id!=MKV_CUE_TIME)
         {
             ADM_searchMkvTag( (MKV_ELEM_ID)id,&ss,&type);
@@ -487,7 +549,8 @@ bool mkvHeader::readCue(ADM_ebml_file *parser)
         }
         time=cue.readUnsignedInt(len);
 
-        cue.readElemId(&id,&len);
+        if(!cue.readElemId(&id,&len))
+            continue;
         if(id!=MKV_CUE_TRACK_POSITION)
         {
             ADM_searchMkvTag( (MKV_ELEM_ID)id,&ss,&type);
@@ -501,7 +564,8 @@ bool mkvHeader::readCue(ADM_ebml_file *parser)
         uint64_t cue_position=0;
         while(!trackPos.finished())
         {
-            trackPos.readElemId(&id,&len);
+            if(!trackPos.readElemId(&id,&len))
+                continue;
             switch(id)
             {
                 case MKV_CUE_TRACK: tid=trackPos.readUnsignedInt(len);break;
@@ -584,7 +648,8 @@ uint8_t mkvHeader::indexClusters(ADM_ebml_file *parser)
      // Normally the timecode is the 1st one following
 
 tryAgain:
-        segment.readElemId(&id,&len);
+        if(!segment.readElemId(&id,&len))
+            continue;
         switch(id)
         {
             case MKV_CRC32:
