@@ -152,6 +152,7 @@ bool muxerMp4v2::open(const char *file, ADM_videoStream *s,uint32_t nbAudioTrack
         audioDelay=s->getVideoDelay();
         vStream=s;
         nbAStreams=nbAudioTrack;
+        setOutputFileName(file);
         aStreams=a;
         videoBufferSize=vStream->getWidth()*vStream->getHeight()*3;
         videoBuffer[0]=new uint8_t[videoBufferSize];
@@ -177,7 +178,7 @@ bool muxerMp4v2::open(const char *file, ADM_videoStream *s,uint32_t nbAudioTrack
                 case WAV_MP2:case WAV_MP3:case WAV_AAC:case WAV_AC3:
                             continue;
                 default:
-                    GUI_Error_HIG("Audio","Audio format not supported, only AAC/MP3/AC3");
+                    GUI_Error_HIG(QT_TRANSLATE_NOOP("mp4v2muxer","Audio"),QT_TRANSLATE_NOOP("mp4v2muxer","Audio format not supported, only AAC/MP3/AC3"));
                     return false;
             }            
         }
@@ -223,7 +224,7 @@ bool muxerMp4v2::save(void)
    
 
     initUI("Saving MP4V2");
-    encoding->setPhasis(QT_TRANSLATE_NOOP("adm","Saving"));
+    encoding->setPhase(ADM_ENC_PHASE_LAST_PASS,QT_TRANSLATE_NOOP("mp4v2muxer","Saving"));
     encoding->setContainer("MP4 (libmp4v2)");
     uint64_t lastSentDts=0;
     
@@ -237,7 +238,7 @@ bool muxerMp4v2::save(void)
         ADM_assert(in[nextWrite].dts!=ADM_NO_PTS)
         if(in[other].pts==ADM_NO_PTS || in[other].pts==ADM_NO_PTS)
         {
-            GUI_Error_HIG("Video","Video does not have enough timing information. Are you copying from AVI ?");
+            GUI_Error_HIG(QT_TRANSLATE_NOOP("mp4v2muxer","Video"),QT_TRANSLATE_NOOP("mp4v2muxer","Video does not have enough timing information. Are you copying from AVI?"));
             goto theEnd;
         }
 
@@ -246,7 +247,7 @@ bool muxerMp4v2::save(void)
         uint64_t nextDts=in[nextWrite].dts;   // Delta between dts=duration of the frame (sort of)     
         uint64_t myDts=in[other].dts;
         uint64_t myPts=in[other].pts;        
-        cprintf(">>next DTS=%"PRIu64", last DTS=%"PRIu64"delta=%"PRIu64"\n",nextDts,lastSentDts,nextDts-lastSentDts);
+        cprintf(">>next DTS=%" PRIu64", last DTS=%" PRIu64"delta=%" PRIu64"\n",nextDts,lastSentDts,nextDts-lastSentDts);
 
         encoding->pushVideoFrame(in[other].len,in[other].out_quantizer,myDts);
         uint64_t delta=myPts-lastSentDts; // composition time...
@@ -256,7 +257,7 @@ bool muxerMp4v2::save(void)
         uint64_t scaled_duration=timeScale(duration);
         duration=inverseTimeScale(scaled_duration); // handle rounding error
         nbFrame++;
-        cprintf("Sending frame duration=%"PRIu64", pts/dts=%"PRIu64"\n",lastSentDts,delta);
+        cprintf("Sending frame duration=%" PRIu64", pts/dts=%" PRIu64"\n",lastSentDts,delta);
         if(false==MP4WriteSample(handle,videoTrackId,in[other].data,in[other].len,
                         scaled_duration, // duration
                         delta, // pts/dts offset
@@ -270,7 +271,7 @@ bool muxerMp4v2::save(void)
         // update lastSentDts
         lastSentDts+=duration; // beginning of next frame...
         //
-        cprintf("lastSendDts=%"PRIu64", next Dts=%"PRIu64", skew=%"PRId64"\n",lastSentDts,nextDts,
+        cprintf("lastSendDts=%" PRIu64", next Dts=%" PRIu64", skew=%" PRId64"\n",lastSentDts,nextDts,
                         (int64_t)nextDts-(int64_t)lastSentDts);
         //
         fillAudio(lastSentDts);
@@ -298,18 +299,19 @@ theEnd:
     
     if(muxerConfig.optimize && result==true)
     {
-        encoding->setPhasis("Optimizing");
+        encoding->setPhase(ADM_ENC_PHASE_OTHER,QT_TRANSLATE_NOOP("mp4v2muxer","Optimizing"));
         string tmpTargetFileName=targetFileName+string(".tmp");
         if(!ADM_renameFile(targetFileName.c_str(),tmpTargetFileName.c_str()))
         {
-            GUI_Error_HIG("","Cannot rename file (optimize)");
+            GUI_Error_HIG("",QT_TRANSLATE_NOOP("mp4v2muxer","Cannot rename file (optimize)"));
             return false;
         }
         // Optimize
         ADM_info("Optimizing...\n");
         MP4Optimize( tmpTargetFileName.c_str(), targetFileName.c_str() );
         // delete
-        unlink(tmpTargetFileName.c_str());
+        if(!ADM_eraseFile(tmpTargetFileName.c_str()))
+            ADM_warning("Could not delete %s\n",tmpTargetFileName.c_str());
     }
     closeUI();
     return result;
