@@ -46,6 +46,9 @@ ADM_ffMpeg2Encoder::ADM_ffMpeg2Encoder(ADM_coreVideoFilter *src,bool globalHeade
 */
 bool ADM_ffMpeg2Encoder::configureContext(void)
 {
+    presetContext(&Settings);
+    _context->rc_max_rate = Mp2Settings.lavcSettings.maxBitrate * 1000;
+
     switch(Settings.params.mode)
     {
       case COMPRESS_2PASS:
@@ -62,12 +65,48 @@ bool ADM_ffMpeg2Encoder::configureContext(void)
             _context->bit_rate = 0;
             break;
       case COMPRESS_CBR:
-              _context->bit_rate=Settings.params.bitrate*1000; // kb->b;
+            _context->bit_rate =
+            _context->rc_max_rate =
+            _context->rc_min_rate = Settings.params.bitrate*1000; // kb->b;
             break;
      default:
             return false;
     }
-    presetContext(&Settings);
+    if(Settings.lavcSettings.interlaced)
+        _context->flags |= (AV_CODEC_FLAG_INTERLACED_DCT | AV_CODEC_FLAG_INTERLACED_ME);
+    if(!Settings.lavcSettings.widescreen)
+    { // we handle only default resolutions
+        if(getHeight()==480) // NTSC
+        {
+            switch(getWidth())
+            {
+                case 704:
+                    _context->sample_aspect_ratio.num=10;
+                    _context->sample_aspect_ratio.den=11;
+                    break;
+                case 720:
+                    _context->sample_aspect_ratio.num=8;
+                    _context->sample_aspect_ratio.den=9;
+                    break;
+                default:break;
+            }
+        }
+        if(getHeight()==576) // PAL
+        {
+            switch(getWidth())
+            {
+                case 704:
+                    _context->sample_aspect_ratio.num=12;
+                    _context->sample_aspect_ratio.den=11;
+                    break;
+                case 720:
+                    _context->sample_aspect_ratio.num=16;
+                    _context->sample_aspect_ratio.den=15;
+                    break;
+                default:break;
+            }
+        }
+    }
     // Override some parameters specific to this codec
     // Set matrix if any...
 #define MX(a,b,c) case a: _context->intra_matrix=b,_context->inter_matrix=c;break;
@@ -83,12 +122,11 @@ bool ADM_ffMpeg2Encoder::configureContext(void)
                 break;
     }
     _context->rc_buffer_size=Mp2Settings.lavcSettings.bufferSize*8*1024;
-    _context->rc_buffer_size_header=Mp2Settings.lavcSettings.bufferSize*8*1024;
+    //_context->rc_buffer_size_header=Mp2Settings.lavcSettings.bufferSize*8*1024; // needs patched avcodec.h
     _context->rc_initial_buffer_occupancy=_context->rc_buffer_size;
-    _context->rc_max_rate=Mp2Settings.lavcSettings.maxBitrate*1000;
-    _context->rc_max_rate_header=Mp2Settings.lavcSettings.maxBitrate*1000;
+    _context->bit_rate_tolerance=Mp2Settings.lavcSettings.vratetol*1000;
+    //_context->rc_max_rate_header=Mp2Settings.lavcSettings.maxBitrate*1000; // needs patched avcodec.h
     // /Override some parameters specific to this codec
-    
     return true;
 }
 
@@ -97,7 +135,8 @@ bool ADM_ffMpeg2Encoder::configureContext(void)
 */
 bool ADM_ffMpeg2Encoder::setup(void)
 {
-
+    if(Settings.lavcSettings.interlaced)
+        av_dict_set(&_options, "alternate_scan", "1", 0);
     if(false== ADM_coreVideoEncoderFFmpeg::setup(AV_CODEC_ID_MPEG2VIDEO))
         return false;
     printf("[ffMpeg] Setup ok\n");
@@ -121,27 +160,27 @@ ADM_ffMpeg2Encoder::~ADM_ffMpeg2Encoder()
 bool         ADM_ffMpeg2Encoder::encode (ADMBitstream * out)
 {
 int sz,q;
-int gotData;
 int r;
 again:
     sz=0;
     if(false==preEncode()) // Pop - out the frames stored in the queue due to B-frames
     {
-        AVPacket pkt;
-        av_init_packet(&pkt);
-        pkt.data=out->data;
-        pkt.size=out->bufferSize;
-
         r=encodeWrapper(NULL,out);
+
+        if(encoderState == ADM_ENCODER_STATE_FLUSHED)
+        {
+            ADM_info("[ffMpeg2] End of stream.\n");
+            return false;
+        }
         if(r<0)
         {
             ADM_warning("[ffMpeg2] Error %d encoding video\n",r);
             return false;
         }
         sz=r;
+        if(!sz) return false;
         ADM_info("[ffMpeg2] Popping delayed bframes (%d)\n",sz);
         goto link;
-        return false;
     }
     q=image->_Qp;
     
@@ -179,7 +218,15 @@ again:
                                      _context->bit_rate,  _frame->quality, _frame->quality/ FF_QP2LAMBDA,q);     
     
     _frame->reordered_opaque=image->Pts;
+    _frame->interlaced_frame=Settings.lavcSettings.interlaced;
+    _frame->top_field_first=!Settings.lavcSettings.bff;
     r=encodeWrapper(_frame,out);
+
+    if(encoderState == ADM_ENCODER_STATE_FLUSHED)
+    {
+        ADM_info("[ffMpeg2] End of stream.\n");
+        return false;
+    }
     if(r<0)
     {
         ADM_warning("[ffMpeg2] Error %d encoding video\n",r);
@@ -214,37 +261,37 @@ bool         ffMpeg2Configure(void)
 
 diaMenuEntry  arE[]=
 {
-    {0,QT_TRANSLATE_NOOP("ffmpeg2","Normal (4:3)")},
-    {1,QT_TRANSLATE_NOOP("ffmpeg2","Wide (16:9)")}
+    {0,QT_TRANSLATE_NOOP("ffmpeg2","Normal (4:3)"),NULL},
+    {1,QT_TRANSLATE_NOOP("ffmpeg2","Wide (16:9)"),NULL}
 };
       
 diaMenuEntry  matrixE[]=
 {
-    {MPEG2_MATRIX_DEFAULT,QT_TRANSLATE_NOOP("ffmpeg2","Default")},
-    {MPEG2_MATRIX_TMPGENC,QT_TRANSLATE_NOOP("ffmpeg2","Tmpgenc")},
-    {MPEG2_MATRIX_ANIME,QT_TRANSLATE_NOOP("ffmpeg2","Animes")},
-    {MPEG2_MATRIX_KVCD,QT_TRANSLATE_NOOP("ffmpeg2","KVCD")},
+    {MPEG2_MATRIX_DEFAULT,QT_TRANSLATE_NOOP("ffmpeg2","Default"),NULL},
+    {MPEG2_MATRIX_TMPGENC,QT_TRANSLATE_NOOP("ffmpeg2","Tmpgenc"),NULL},
+    {MPEG2_MATRIX_ANIME,QT_TRANSLATE_NOOP("ffmpeg2","Animes"),NULL},
+    {MPEG2_MATRIX_KVCD,QT_TRANSLATE_NOOP("ffmpeg2","KVCD"),NULL}
 };
       
 diaMenuEntry rdE[]={
-  {0,QT_TRANSLATE_NOOP("ffmpeg2","MB comparison")},
-  {1,QT_TRANSLATE_NOOP("ffmpeg2","Fewest bits (vhq)")},
-  {2,QT_TRANSLATE_NOOP("ffmpeg2","Rate distortion")}
+  {0,QT_TRANSLATE_NOOP("ffmpeg2","MB comparison"),NULL},
+  {1,QT_TRANSLATE_NOOP("ffmpeg2","Fewest bits (vhq)"),NULL},
+  {2,QT_TRANSLATE_NOOP("ffmpeg2","Rate distortion"),NULL}
 };     
 diaMenuEntry threads[]={
-  {0,QT_TRANSLATE_NOOP("ffmpeg2","One thread")},
-  {2,QT_TRANSLATE_NOOP("ffmpeg2","Two threads)")},
-  {3,QT_TRANSLATE_NOOP("ffmpeg2","Three threads")},
-  {99,QT_TRANSLATE_NOOP("ffmpeg2","Auto (#cpu)")}
+  {0,QT_TRANSLATE_NOOP("ffmpeg2","One thread"),NULL},
+  {2,QT_TRANSLATE_NOOP("ffmpeg2","Two threads"),NULL},
+  {3,QT_TRANSLATE_NOOP("ffmpeg2","Three threads"),NULL},
+  {99,QT_TRANSLATE_NOOP("ffmpeg2","Auto (#cpu)"),NULL}
 };     
    
 diaMenuEntry interE[]={
-  {0,QT_TRANSLATE_NOOP("ffmpeg2","Progressive")},
-  {1,QT_TRANSLATE_NOOP("ffmpeg2","Interlaced")},
+  {0,QT_TRANSLATE_NOOP("ffmpeg2","Progressive"),NULL},
+  {1,QT_TRANSLATE_NOOP("ffmpeg2","Interlaced"),NULL}
 };     
 diaMenuEntry foE[]={
-  {0,QT_TRANSLATE_NOOP("ffmpeg2","Top Field First")},
-  {1,QT_TRANSLATE_NOOP("ffmpeg2","Bottom Field First")},
+  {0,QT_TRANSLATE_NOOP("ffmpeg2","Top Field First"),NULL},
+  {1,QT_TRANSLATE_NOOP("ffmpeg2","Bottom Field First"),NULL}
 };     
 
         mpeg2_encoder *conf=&Mp2Settings;
@@ -279,6 +326,8 @@ diaMenuEntry foE[]={
         diaElemMenu     interlaced(&(iinterlaced),QT_TRANSLATE_NOOP("ffmpeg2","_Interlaced:"),2,interE);
         diaElemMenu     fieldOrder(&(bff),QT_TRANSLATE_NOOP("ffmpeg2","Field Order:"),2,foE);
 
+        interlaced.link(interE+1,1,&fieldOrder);
+
           /* First Tab : encoding mode */
         diaElem *diamode[]={&arM,&threadM,&bitrate};
         diaElemTabs tabMode(QT_TRANSLATE_NOOP("ffmpeg2","Basic Settings"),3,diamode);
