@@ -23,6 +23,7 @@
 #include "DIA_coreToolkit.h"
 #include "ADM_vidMisc.h"
 #include "ADM_coreUtils.h"
+#include <climits>
 
 #if 1
     #define aprintf(...) {}
@@ -62,17 +63,17 @@ bool x264Encoder::setup(void)
   if(!x264Settings.useAdvancedConfiguration)
   {
         std::string tune;
-        if(x264Settings.general.tuning != std::string("none"))
+        if(x264Settings.general.tuning != "none")
             tune = x264Settings.general.tuning;
         if(x264Settings.general.fast_decode)
         {
-            tune += std::string(",");
-            tune += std::string("fastdecode");
+            tune += ",";
+            tune += "fastdecode";
         }
         if(x264Settings.general.zero_latency)
         {
-            tune += std::string(",");
-            tune += std::string("zerolatency");
+            tune += ",";
+            tune += "zerolatency";
         }
         if(tune.empty())
             x264_param_default_preset(&param, x264Settings.general.preset.c_str(), NULL);
@@ -98,11 +99,23 @@ bool x264Encoder::setup(void)
   param.i_log_level=X264_LOG_INFO; //DEBUG; //INFO;
 
   //Framerate
-  int n,d;    
+  int n,d;
   uint64_t f=source->getInfo()->frameIncrement;
-  usSecondsToFrac(f,&n,&d);
+  usSecondsToFrac(f,&n,&d,INT_MAX);
   param.i_fps_num = d;
   param.i_fps_den = n;
+  n = source->getInfo()->timeBaseNum & 0x7FFFFFFF;
+  d = source->getInfo()->timeBaseDen & 0x7FFFFFFF;
+  ADM_assert(d);
+  param.i_timebase_num = n;
+  param.i_timebase_den = d;
+#if 0
+  bool constantFps = (n * 1000 * 1000 + (d / 2))/ d >= f;
+  param.b_vfr_input = !constantFps;
+#else
+// We do pseudo cfr ...
+  param.b_vfr_input=0;
+#endif
 
   // -------------- vui------------
   #undef MKPARAM
@@ -113,6 +126,13 @@ bool x264Encoder::setup(void)
   #define MKPARAMB(x,y) {param.vui.x = x264Settings.vui.y ;aprintf("[x264] vui."#x" = %s\n",TrueFalse[param.vui.x&1]);}
   MKPARAM (i_sar_width,sar_width) 
   MKPARAM (i_sar_height,sar_height) 
+  MKPARAM (i_overscan,overscan)
+  MKPARAM (i_vidformat,vidformat)
+  MKPARAMB(b_fullrange,fullrange)
+  MKPARAM (i_colorprim,colorprim)
+  MKPARAM (i_transfer,transfer)
+  MKPARAM (i_colmatrix,colmatrix)
+  MKPARAM (i_chroma_loc,chroma_loc)
 
   // -------------- rate control------------
   switch(x264Settings.general.params.mode)
@@ -194,25 +214,34 @@ bool x264Encoder::setup(void)
   else
       param.b_repeat_headers=1;
 
-  // We do pseudo cfr ...
-  param.b_vfr_input=0;
-
-  if(x264Settings.useAdvancedConfiguration)
-  {  
-
   #undef MKPARAM
   #undef MKPARAMF
   #undef MKPARAMB
   #define MKPARAM(x,y) {param.x = x264Settings.y;aprintf("[x264] "#x" = %d\n",param.x);}
   #define MKPARAMF(x,y) {param.x = (float)x264Settings.y; aprintf("[x264] "#x" = %.2f\n",param.x);}
   #define MKPARAMB(x,y) {param.x = x264Settings.y ;aprintf("[x264] "#x" = %s\n",TrueFalse[param.x&1]);}
-    MKPARAM(i_frame_reference,MaxRefFrames);
-    MKPARAM(i_keyint_min,MinIdr);
-    MKPARAM(i_keyint_max,MaxIdr);
-    MKPARAM(i_scenecut_threshold,i_scenecut_threshold);
-    MKPARAMB(b_intra_refresh,intra_refresh);
-    MKPARAM(i_bframe,MaxBFrame);
 
+#ifdef SIMPLE_MODE_EX
+  MKPARAMB(b_interlaced,interlaced)
+  MKPARAMB(b_tff,tff)
+  MKPARAM(i_keyint_min,MinIdr)
+  MKPARAM(i_keyint_max,MaxIdr)
+  MKPARAM(i_scenecut_threshold,i_scenecut_threshold)
+  MKPARAMB(b_intra_refresh,intra_refresh)
+#endif
+
+  if(x264Settings.useAdvancedConfiguration)
+  {
+    MKPARAM(i_frame_reference,MaxRefFrames);
+#ifndef SIMPLE_MODE_EX
+    MKPARAMB(b_interlaced,interlaced)
+    MKPARAMB(b_tff,tff)
+    MKPARAM(i_keyint_min,MinIdr)
+    MKPARAM(i_keyint_max,MaxIdr)
+    MKPARAM(i_scenecut_threshold,i_scenecut_threshold)
+    MKPARAMB(b_intra_refresh,intra_refresh)
+#endif
+    MKPARAM(i_bframe,MaxBFrame);
     MKPARAM(i_bframe_adaptive,i_bframe_adaptive);
     MKPARAM(i_bframe_bias,i_bframe_bias);
     MKPARAM(i_bframe_pyramid,i_bframe_pyramid);
@@ -223,9 +252,7 @@ bool x264Encoder::setup(void)
       MKPARAM(i_deblocking_filter_beta,i_deblocking_filter_beta);
     }
     MKPARAMB(b_cabac,cabac);
-    MKPARAMB(b_interlaced,interlaced);
     MKPARAMB(b_constrained_intra,constrained_intra);
-    MKPARAMB(b_tff,tff);
     MKPARAMB(b_fake_interlaced,fake_interlaced);
 
     // -------------- analyze------------
@@ -326,6 +353,8 @@ bool x264Encoder::setup(void)
     x264_param_apply_profile(&param, x264Settings.general.profile.c_str());
   }
 
+  setConstraintsByLevel();
+
   dumpx264Setup(&param);
   ADM_info("Creating x264 encoder\n");
   handle = x264_encoder_open (&param);
@@ -346,137 +375,234 @@ bool x264Encoder::setup(void)
     
   return true;
 }
+/**
+    \fn setConstraintsByLevel
+    \brief Copied from Handbrake code
+*/
+bool x264Encoder::setConstraintsByLevel(void)
+{
+    if(param.i_level_idc < 0)
+        return true;
+    if(param.i_width <= 0 || param.i_height <= 0)
+    {
+        ADM_warning("Invalid dimensions: %d:%d\n",param.i_width,param.i_height);
+        return false;
+    }
+    const x264_level_t *level = NULL;
+    int val = param.i_level_idc;
+    for(int i=0; x264_levels[i].level_idc; i++)
+    {
+        if(x264_levels[i].level_idc == val)
+        {
+            level = &x264_levels[i];
+            break;
+        }
+    }
+    if(!level)
+    {
+        ADM_warning("Invalid level %d\n",val);
+        return false;
+    }
+    if(level->frame_only && (param.b_interlaced || param.b_fake_interlaced))
+    {
+        ADM_warning("Interlaced flag not supported for level %d, disabling",val);
+        param.b_interlaced = param.b_fake_interlaced = 0;
+    }
+    // Calculate frame dimensions and rate (in macroblocks)
+    int mbW = (param.i_width + 15) >> 4;
+    int mbH = (param.i_height + 15) >> 4;
+    if(param.b_interlaced || param.b_fake_interlaced)
+    {
+        // interlaced: encoded height must divide cleanly by 32
+        mbH = (mbH + 1) & ~1;
+    }
+    int nbMb = mbW * mbH;
+    int mbRate;
+    if(param.i_fps_den <= 0 || param.i_fps_num <= 0)
+        mbRate = 0;
+    else
+        mbRate = (int64_t)nbMb * param.i_fps_num / param.i_fps_den;
+    // Sanitize frame_reference
+    if(param.i_keyint_max != 1)
+    {
+        int decBufSize = level->dpb / nbMb;
+        if(decBufSize < 1) decBufSize = 1;
+        if(decBufSize > 16) decBufSize = 16;
+        if(decBufSize < param.i_frame_reference)
+        {
+            ADM_warning("Number of ref frames %d too high for the IDC level, setting to %d\n",param.i_frame_reference,decBufSize);
+            param.i_frame_reference = decBufSize;
+        }
+        // Now disable B-frames and pyramid if necessary.
+        if(decBufSize < 2)
+        {
+            if(param.i_bframe)
+            {
+                ADM_warning("B-frames forbidden by the IDC level, disabling.\n");
+                param.i_bframe = 0;
+            }
+        }else if(decBufSize < 4 && param.i_bframe_pyramid != X264_B_PYRAMID_NONE)
+        {
+            ADM_warning("B-frame pyramid forbidden by the IDC level, disabling.\n");
+            param.i_bframe_pyramid = X264_B_PYRAMID_NONE;
+        }
+    }
+    // Sanitize VBV depending on profile, simple mode only
+    if(!x264Settings.useAdvancedConfiguration && x264Settings.general.profile != std::string("high444"))
+    {
+        int codedPicBufFactor = 4;
+        if(x264Settings.general.profile == "high")
+            codedPicBufFactor = 5;
+        int maxVbvBitrate = (level->bitrate * codedPicBufFactor) >> 2;
+        if(!param.rc.i_vbv_max_bitrate || param.rc.i_vbv_max_bitrate > maxVbvBitrate)
+        {
+            param.rc.i_vbv_max_bitrate = maxVbvBitrate;
+        }
+        int maxVbvBufSize = (level->cpb * codedPicBufFactor) >> 2;
+        if(!param.rc.i_vbv_buffer_size || param.rc.i_vbv_buffer_size > maxVbvBitrate)
+        {
+            param.rc.i_vbv_buffer_size = maxVbvBitrate;
+        }
+    }
+    // Log warnings about constraints violation which are impossible to correct
+    if(level->frame_size < nbMb)
+        ADM_warning("Too many macroblocks per frame for the IDC level: %d (max: %d)\n",nbMb,level->frame_size);
+    if(level->mbps < mbRate)
+    {
+        ADM_warning("Framerate %d/%d too high for IDC level (mb/s: %d, max: %d)\n",param.i_fps_num,param.i_fps_den,mbRate,level->mbps);
+    }
+    return true;
+}
 /**
     \fn dumpx264Setup
 */
-void dumpx264Setup(x264_param_t *param)
+static void dumpx264Setup(x264_param_t *param)
 {
-#define PI(x) printf(#x"\t:%d\n",(int)param->x);
-    PI(cpu);
-    PI(i_threads);
-    PI(b_sliced_threads);
-    PI(b_deterministic);
-    PI(i_sync_lookahead);
+#define STR(x) #x
+#define MKSTRING(x) STR(x)
+#define PI(x) printf("%-24s : %d\n",MKSTRING(x),(int)param->x);
+    PI(cpu)
+    PI(i_threads)
+    PI(b_sliced_threads)
+    PI(b_deterministic)
+    PI(i_sync_lookahead)
 
-    PI(i_width); 
-    PI(i_height); 
-    PI(i_width); 
-    PI(i_level_idc); 
-    PI(i_frame_total);
+    PI(i_width)
+    PI(i_height)
+    PI(i_width)
+    PI(i_level_idc)
+    PI(i_frame_total)
 
-#define VI(x) printf(#x"\t:%d\n",(int)param->vui.x);
+#define VI(x) printf("%-24s : %d\n",MKSTRING(x),(int)param->vui.x);
+    VI(i_sar_height)
+    VI(i_sar_width)
+    VI(i_overscan)
+    VI(i_vidformat)
+    VI(b_fullrange)
+    VI(i_colorprim)
+    VI(i_transfer)
+    VI(i_colmatrix)
+    VI(i_chroma_loc)
 
-    VI(i_sar_height);
-    VI(i_sar_width);
-    VI(i_overscan);
-    VI(i_vidformat);
-    VI(b_fullrange);
-    VI(i_colorprim);
-    VI(i_transfer);
-    VI(i_colmatrix);
-    VI(i_chroma_loc);
+    PI(i_fps_num)
+    PI(i_fps_den)
 
-    PI(i_fps_num);
-    PI(i_fps_den);
+    PI(i_timebase_num)
+    PI(i_timebase_den)
+    PI(b_vfr_input)
 
-    PI(i_timebase_num);
-    PI(i_timebase_den);
+    PI(i_frame_reference)
+    PI(i_keyint_max)
+    PI(i_keyint_min)
+    PI(i_scenecut_threshold)
+    PI(b_intra_refresh)
 
-        
-    PI(i_frame_reference);
-    PI(i_keyint_max);
-    PI(i_keyint_min);
-    PI(i_scenecut_threshold);
-    PI(b_intra_refresh);
+    PI(i_bframe)
+    PI(i_bframe_adaptive)
+    PI(i_bframe_bias)
+    PI(i_bframe_pyramid)
 
-    PI(i_bframe);
-    PI(i_bframe_adaptive);
-    PI(i_bframe_bias);
-    PI(i_bframe_pyramid);
+    PI(b_deblocking_filter)
+    PI(i_deblocking_filter_alphac0)
+    PI(i_deblocking_filter_beta)
 
-    PI(b_deblocking_filter);
-    PI(i_deblocking_filter_alphac0);
-    PI(i_deblocking_filter_beta);
+    PI(b_cabac)
+    PI(i_cabac_init_idc)
 
-    PI(b_cabac);
-    PI(i_cabac_init_idc);
+    PI(b_interlaced)
+    PI(b_tff)
+    PI(b_fake_interlaced)
+    PI(b_bluray_compat)
+    PI(b_constrained_intra)
 
-    PI(b_interlaced);
-    PI(b_tff);
-    PI(b_fake_interlaced);
-    PI(b_bluray_compat);
-    PI(b_constrained_intra);
-
-#define AI(x) printf(#x"\t:%d\n",(int)param->analyse.x);
-#define AF(x) printf(#x"\t:%f\n",(float)param->analyse.x);
+#define AI(x) printf("%-24s : %d\n",MKSTRING(x),(int)param->analyse.x);
+#define AF(x) printf("%-24s : %f\n",MKSTRING(x),(float)param->analyse.x);
     printf("*************************************\n");
     printf("*********     Analyse       *********\n");
     printf("*************************************\n");
 
-    AI(intra);
-    AI(inter);
+    AI(intra)
+    AI(inter)
 
-    AI(b_transform_8x8);
-    AI(i_weighted_pred);
-    AI(b_weighted_bipred);
-    AI(i_chroma_qp_offset);
-    
-    AI(i_me_method);
-    AI(i_me_range);
-    AI(i_mv_range);
-    AI(i_mv_range_thread);
-    AI(i_subpel_refine);
-    AI(b_chroma_me);
-    AI(b_mixed_references);
-    AI(i_trellis);
-    AI(b_fast_pskip);
+    AI(b_transform_8x8)
+    AI(i_weighted_pred)
+    AI(b_weighted_bipred)
+    AI(i_chroma_qp_offset)
 
-    AI(b_dct_decimate);
-    AI(i_noise_reduction);
-    AF(f_psy_rd);
-    AF(f_psy_trellis);
-    AI(b_psy);
+    AI(i_me_method)
+    AI(i_me_range)
+    AI(i_mv_range)
+    AI(i_mv_range_thread)
+    AI(i_subpel_refine)
+    AI(b_chroma_me)
+    AI(b_mixed_references)
+    AI(i_trellis)
+    AI(b_fast_pskip)
 
-    PI(b_aud);
-    PI(b_repeat_headers);
-    PI(b_annexb);
-    PI(b_vfr_input);
-    
-    AI(i_luma_deadzone[0]);
-    AI(i_luma_deadzone[1]);
+    AI(b_dct_decimate)
+    AI(i_noise_reduction)
+    AF(f_psy_rd)
+    AF(f_psy_trellis)
+    AI(b_psy)
 
-    PI(i_sps_id);
+    PI(b_aud)
+    PI(b_repeat_headers)
+    PI(b_annexb)
 
-    PI(i_slice_max_size);
-    PI(i_slice_max_mbs);
-    PI(i_slice_count);
+    AI(i_luma_deadzone[0])
+    AI(i_luma_deadzone[1])
 
-#define RI(x) printf(#x"\t:%d\n",(int)param->rc.x)
-#define RF(x) printf(#x"\t:%f\n",(float)param->rc.x)
+    PI(i_sps_id)
+
+    PI(i_slice_max_size)
+    PI(i_slice_max_mbs)
+    PI(i_slice_count)
+
+#define RI(x) printf("%-24s : %d\n",MKSTRING(x),(int)param->rc.x);
+#define RF(x) printf("%-24s : %f\n",MKSTRING(x),(float)param->rc.x);
     printf("*************************************\n");
     printf("*********     RC            *********\n");
     printf("*************************************\n");
-    RI(i_rc_method);
-    RI(i_qp_constant);
-    RF(f_rf_constant);
-    RI(i_qp_min);
-    RI(i_qp_max);
-    RI(i_qp_step);
+    RI(i_rc_method)
+    RI(i_qp_constant)
+    RF(f_rf_constant)
+    RI(i_qp_min)
+    RI(i_qp_max)
+    RI(i_qp_step)
 
-   
-    RI(i_bitrate);
-    RI(i_qp_constant);
-    RF(f_rate_tolerance);
-    RI(i_vbv_max_bitrate);
-    RI(i_vbv_buffer_size);
-    RF(f_vbv_buffer_init);
-    RF(f_ip_factor);
-    RF(f_pb_factor);
+    RI(i_bitrate)
+    RI(i_qp_constant)
+    RF(f_rate_tolerance)
+    RI(i_vbv_max_bitrate)
+    RI(i_vbv_buffer_size)
+    RF(f_vbv_buffer_init)
+    RF(f_ip_factor)
+    RF(f_pb_factor)
 
-    RI(i_aq_mode);
-    RF(f_aq_strength);
-    RI(b_mb_tree);
-    RI(i_lookahead);
+    RI(i_aq_mode)
+    RF(f_aq_strength)
+    RI(b_mb_tree)
+    RI(i_lookahead)
 
 }
 /**
