@@ -31,6 +31,57 @@ static void dumpx265Setup(x265_param *param);
 #define MMSET(x) memset(&(x),0,sizeof(x))
 
 extern x265_settings x265Settings;
+
+/**
+    \fn x265ProbeBitDepth
+*/
+bool x265ProbeBitDepth(int depth)
+{
+    static uint32_t cachedQueries=0;
+
+    uint32_t already=1;
+    uint32_t match=0;
+
+#define BIT_DEPTH_8_BITS 1
+#define BIT_DEPTH_10_BITS 2
+#define BIT_DEPTH_12_BITS 4
+
+    switch(depth)
+    {
+        case 0:
+            return true;
+        case 8:
+            match = BIT_DEPTH_8_BITS;
+            break;
+        case 10:
+            match = BIT_DEPTH_10_BITS;
+            break;
+        case 12:
+            match = BIT_DEPTH_12_BITS;
+            break;
+        default:
+            return false;
+    }
+    already <<= depth;
+    if(cachedQueries & already)
+    {
+        if((cachedQueries & BIT_DEPTH_8_BITS) && depth == 8)
+            return true;
+        if((cachedQueries & BIT_DEPTH_10_BITS) && depth == 10)
+            return true;
+        if((cachedQueries & BIT_DEPTH_12_BITS) && depth == 12)
+            return true;
+        return false;
+    }
+    cachedQueries |= already;
+    if(x265_api_get(depth))
+    {
+        cachedQueries |= match;
+        return true;
+    }
+    return false;
+}
+
 /**
     \fn setup
 */
@@ -39,19 +90,33 @@ bool x265Encoder::setup(void)
   ADM_info("=============x265, setting up==============\n");
   MMSET(param);
 
-  x265_param_default( &param);
+  if(x265Settings.useAdvancedConfiguration)
+  {
+        api = x265_api_get(x265Settings.general.output_bit_depth);
+  }else if(x265Settings.general.profile == "main10")
+  {
+        api = x265_api_get(10);
+  }else
+  {
+        api = x265_api_get(8);
+  }
+  if(!api)
+        api = x265_api_get(0);
+  ADM_assert(api);
+
+  api->param_default(&param);
   firstIdr=true;
   image=new ADMImageDefault(getWidth(),getHeight());
 
   // -------------- preset, tune, idc ------------
   if(!x265Settings.useAdvancedConfiguration)
   {
-    if(x265Settings.general.tuning == std::string("none"))
+    if(x265Settings.general.tuning == "none")
     {
-        x265_param_default_preset(&param, x265Settings.general.preset.c_str(), NULL);
+        api->param_default_preset(&param, x265Settings.general.preset.c_str(), NULL);
     }else
     {
-        x265_param_default_preset(&param, x265Settings.general.preset.c_str(), x265Settings.general.tuning.c_str());
+        api->param_default_preset(&param, x265Settings.general.preset.c_str(), x265Settings.general.tuning.c_str());
     }
   }
   param.logLevel=x265Settings.level; 
@@ -75,7 +140,7 @@ bool x265Encoder::setup(void)
   param.sourceWidth = getWidth();
   param.sourceHeight = getHeight();
   param.internalCsp = X265_CSP_I420;
-  param.internalBitDepth = 8;
+  param.internalBitDepth = api->bit_depth;
   param.logLevel=X265_LOG_INFO; //DEBUG; //INFO;
 
   //Framerate
@@ -92,8 +157,19 @@ bool x265Encoder::setup(void)
   #define MKPARAM(x,y) {param.vui.x = x265Settings.vui.y;aprintf("[x265] vui."#x" = %d\n",param.vui.x);}
   #define MKPARAMD(x,y) {param.vui.x = (double)x265Settings.vui.y; aprintf("[x265] vui."#x" = %.2f\n",param.vui.x);}
   #define MKPARAMB(x,y) {param.vui.x = x265Settings.vui.y ;aprintf("[x265] vui."#x" = %s\n",TrueFalse[param.vui.x&1]);}
-  MKPARAM (sarWidth,sar_width) 
-  MKPARAM (sarHeight,sar_height) 
+
+    MKPARAM(aspectRatioIdc,sar_idc);
+    MKPARAM(sarWidth,sar_width);
+    MKPARAM(sarHeight,sar_height);
+
+    if(x265Settings.vui.color_primaries != 2 || x265Settings.vui.matrix_coeffs != 2 || x265Settings.vui.transfer_characteristics != 2)
+    {
+	param.vui.bEnableVideoSignalTypePresentFlag = true;
+	param.vui.bEnableColorDescriptionPresentFlag = true;
+	MKPARAM(colorPrimaries,color_primaries);
+	MKPARAM(matrixCoeffs,matrix_coeffs);
+	MKPARAM(transferCharacteristics,transfer_characteristics);
+    }
 
   // -------------- rate control------------
   switch(x265Settings.general.params.mode)
@@ -170,48 +246,65 @@ bool x265Encoder::setup(void)
   else
       param.bRepeatHeaders=1;
 
-  if(x265Settings.useAdvancedConfiguration)
-  {  
-
   #undef MKPARAM
   #undef MKPARAMD
   #undef MKPARAMB
   #define MKPARAM(x,y) {param.x = x265Settings.y;aprintf("[x265] "#x" = %d\n",param.x);}
   #define MKPARAMD(x,y) {param.x = (double)x265Settings.y; aprintf("[x265] "#x" = %.2f\n",param.x);}
   #define MKPARAMB(x,y) {param.x = x265Settings.y ;aprintf("[x265] "#x" = %s\n",TrueFalse[param.x&1]);}
-    MKPARAM(maxNumReferences,MaxRefFrames);
-    MKPARAM(keyframeMin,MinIdr);
-    MKPARAM(keyframeMax,MaxIdr);
-    MKPARAM(scenecutThreshold,i_scenecut_threshold);
-    MKPARAM(bframes,MaxBFrame);
 
-    MKPARAM(bFrameAdaptive,i_bframe_adaptive);
-    MKPARAM(bFrameBias,i_bframe_bias);
-    MKPARAM(bBPyramid,i_bframe_pyramid);
-    MKPARAMB(bEnableLoopFilter,b_deblocking_filter);
-    MKPARAMB(interlaceMode,interlaced_mode);
-    MKPARAMB(bEnableConstrainedIntra,constrained_intra);
-    MKPARAM(lookaheadDepth,lookahead);
+#ifdef SIMPLE_MODE_EX
+  MKPARAM(interlaceMode,interlaced_mode)
+  MKPARAM(maxNumReferences,MaxRefFrames)
+  MKPARAMB(bOpenGOP,b_open_gop)
+  MKPARAM(keyframeMin,MinIdr)
+  MKPARAM(keyframeMax,MaxIdr)
+  MKPARAM(scenecutThreshold,i_scenecut_threshold)
+#endif
 
-    MKPARAMB(bEnableWeightedBiPred,weighted_bipred) 
-    MKPARAM (bEnableWeightedPred,weighted_pred) 
+  if(x265Settings.useAdvancedConfiguration)
+  {
+#ifndef SIMPLE_MODE_EX
+    MKPARAM(interlaceMode,interlaced_mode)
+    MKPARAM(maxNumReferences,MaxRefFrames)
+    MKPARAMB(bOpenGOP,b_open_gop)
+    MKPARAM(keyframeMin,MinIdr)
+    MKPARAM(keyframeMax,MaxIdr)
+    MKPARAM(scenecutThreshold,i_scenecut_threshold)
+#endif
+    MKPARAM(bframes,MaxBFrame)
+    MKPARAM(bFrameAdaptive,i_bframe_adaptive)
+    MKPARAM(bBPyramid,i_bframe_pyramid)
+    MKPARAM(bFrameBias,i_bframe_bias)
+
+    MKPARAM(lookaheadDepth,lookahead)
+    MKPARAMB(bEnableConstrainedIntra,constrained_intra)
+    MKPARAMB(bIntraInBFrames,b_intra)
+
+    MKPARAM(searchMethod,me_method)
+    MKPARAM(subpelRefine,subpel_refine)
+    MKPARAM(searchRange,me_range)
+    MKPARAM(limitReferences,limit_refs)
+
+    MKPARAM(bEnableWeightedPred,weighted_pred)
+    MKPARAMB(bEnableWeightedBiPred,weighted_bipred)
+    MKPARAMB(bEnableRectInter,rect_inter)
+    MKPARAMB(bEnableAMP,amp_inter)
+    MKPARAMB(limitModes,limit_modes)
+
+    MKPARAMB(bEnableLoopFilter,b_deblocking_filter)
+    MKPARAMB(bEnableEarlySkip,fast_pskip)
+    MKPARAMB(bEnableTSkipFast,dct_decimate)
+
+    MKPARAM (rdLevel,rd_level)
+    MKPARAMD(psyRd,psy_rd)
+    MKPARAM (rdoqLevel,rdoq_level)
+    MKPARAMD(psyRdoq,psy_rdoq)
     MKPARAM (cbQpOffset,cb_chroma_offset)
     MKPARAM (crQpOffset,cr_chroma_offset)
 
-    MKPARAM (searchMethod,me_method) 
-    MKPARAM (searchRange,me_range)
-    MKPARAM (subpelRefine,subpel_refine) 
-    MKPARAM (bFrameAdaptive,trellis) 
-    MKPARAMB(bEnableEarlySkip,fast_pskip) 
-    MKPARAMB(bEnableTSkipFast,dct_decimate) 
-    MKPARAMD(psyRd,psy_rd)
-            
-#if X265_BUILD >= 40
     MKPARAM (noiseReductionIntra,noise_reduction_intra)
     MKPARAM (noiseReductionInter,noise_reduction_inter)
-#else
-    MKPARAM (noiseReduction,noise_reduction)
-#endif
 
     MKPARAMB(bEnableStrongIntraSmoothing,strong_intra_smoothing)
 
@@ -225,11 +318,7 @@ bool x265Encoder::setup(void)
 
       MKPARAM(qpStep,qp_step);
     
-#if X265_BUILD >= 41
       MKPARAMB(bStrictCbr,strict_cbr);
-#else
-      MKPARAM(rateTolerance,rate_tolerance);
-#endif
       
       MKPARAM(ipFactor,ip_factor);
       MKPARAM(pbFactor,pb_factor);
@@ -253,12 +342,12 @@ bool x265Encoder::setup(void)
 
   if(!x265Settings.useAdvancedConfiguration)
   {
-    x265_param_apply_profile(&param, x265Settings.general.profile.c_str());
+    api->param_apply_profile(&param, x265Settings.general.profile.c_str());
   }
 
   dumpx265Setup(&param);
   ADM_info("Creating x265 encoder\n");
-  handle = x265_encoder_open (&param);
+  handle = api->encoder_open (&param);
   if (!handle)
   {
     ADM_error("Cannot initialize x265\n");
@@ -281,13 +370,15 @@ bool x265Encoder::setup(void)
 */
 void dumpx265Setup(x265_param *param)
 {
-#define PI(x) printf(#x"\t:%d\n",(int)param->x)
-#define PD(x) printf(#x"\t:%f\n",(double)param->x)
-#define PS(x) printf(#x"\t:%s\n",param->x)
-    printf("*************************************\n");
-    printf("***      Encoder Environment      ***\n");
-    printf("*************************************\n");
-    
+#define STR(x) #x
+#define MKSTRING(x) STR(x)
+#define PI(x) printf("%-32s : %d\n",MKSTRING(x),(int)param->x)
+#define PD(x) printf("%-32s : %f\n",MKSTRING(x),(double)param->x)
+#define PS(x) printf("%-32s : %s\n",MKSTRING(x),param->x)
+    printf("*********************************************\n");
+    printf("***          Encoder Environment          ***\n");
+    printf("*********************************************\n");
+
     PI(cpuid);
     PI(bEnableWavefront);
 #if X265_BUILD >= 47
@@ -301,11 +392,11 @@ void dumpx265Setup(x265_param *param)
     PI(bEnablePsnr); 
     PI(bEnableSsim); 
     PI(decodedPictureHashSEI);
-    
-    printf("*************************************\n");
-    printf("** Internal Picture Specification  **\n");
-    printf("*************************************\n");
-    
+
+    printf("*********************************************\n");
+    printf("***    Internal Picture Specification     ***\n");
+    printf("*********************************************\n");
+
     PI(internalBitDepth);
     PI(internalCsp);
     PI(fpsNum);
@@ -317,19 +408,19 @@ void dumpx265Setup(x265_param *param)
     PI(bRepeatHeaders);
     PI(bEnableAccessUnitDelimiters);
     PI(bEmitHRDSEI);
-    
-    printf("*************************************\n");
-    printf("*** Coding Unit (CU) Definitions  ***\n");
-    printf("*************************************\n");
-    
+
+    printf("*********************************************\n");
+    printf("***     Coding Unit (CU) Definitions      ***\n");
+    printf("*********************************************\n");
+
     PI(maxCUSize);
     PI(tuQTMaxInterDepth);
     PI(tuQTMaxIntraDepth);
-    
-    printf("*************************************\n");
-    printf("***  GOP Structure and Lookahead  ***\n");
-    printf("*************************************\n");
-    
+
+    printf("*********************************************\n");
+    printf("***      GOP Structure and Lookahead      ***\n");
+    printf("*********************************************\n");
+
     PI(bOpenGOP);
     PI(keyframeMin);
     PI(keyframeMax);
@@ -341,31 +432,34 @@ void dumpx265Setup(x265_param *param)
     PI(lookaheadDepth);
     PI(bFrameBias);
     PI(scenecutThreshold);
-    
-    printf("*************************************\n");
-    printf("***      Intra Coding Tools       ***\n");
-    printf("*************************************\n");
-    
+
+    printf("*********************************************\n");
+    printf("***          Intra Coding Tools           ***\n");
+    printf("*********************************************\n");
+
     PI(bEnableConstrainedIntra);
+    PI(bIntraInBFrames);
     PI(bEnableStrongIntraSmoothing);
-    
-    printf("*************************************\n");
-    printf("***      Inter Coding Tools       ***\n");
-    printf("*************************************\n");
-    
+
+    printf("*********************************************\n");
+    printf("***          Inter Coding Tools           ***\n");
+    printf("*********************************************\n");
+
     PI(searchMethod);
     PI(subpelRefine);
     PI(searchRange);
     PI(maxNumMergeCand);
+    PI(limitReferences);
     PI(bEnableWeightedPred);
     PI(bEnableWeightedBiPred);
-    
-    printf("*************************************\n");
-    printf("***        Analysis Tools         ***\n");
-    printf("*************************************\n");
-    
-    PI(bEnableAMP);
     PI(bEnableRectInter);
+    PI(bEnableAMP);
+    PI(limitModes);
+
+    printf("*********************************************\n");
+    printf("***            Analysis Tools             ***\n");
+    printf("*********************************************\n");
+
 #if X265_BUILD < 45
     PI(bEnableCbfFastMode);
 #endif
@@ -373,11 +467,13 @@ void dumpx265Setup(x265_param *param)
     PI(rdPenalty);
     PI(rdLevel);
     PD(psyRd);
-    
-    printf("*************************************\n");
-    printf("***         Coding Tools          ***\n");
-    printf("*************************************\n");
-    
+    PI(rdoqLevel);
+    PD(psyRdoq);
+
+    printf("*********************************************\n");
+    printf("***             Coding Tools              ***\n");
+    printf("*********************************************\n");
+
     PI(bEnableSignHiding);
     PI(bEnableTransformSkip);
     PI(bEnableTSkipFast);
@@ -396,33 +492,24 @@ void dumpx265Setup(x265_param *param)
     
     PI(cbQpOffset);
     PI(crQpOffset);
-    PI(bIntraInBFrames);
     
-#if X265_BUILD >= 40
     PI(noiseReductionIntra);
     PI(noiseReductionInter);
-#else
-    PI(noiseReduction);
-#endif
     
     PI(bLossless);
     PI(bCULossless);
 
-#define RI(x) printf(#x"\t:%d\n",(int)param->rc.x)
-#define RD(x) printf(#x"\t:%f\n",(double)param->rc.x)
-    printf("*************************************\n");
-    printf("***         Rate Control          ***\n");
-    printf("*************************************\n");
-    
+#define RI(x) printf("%-32s : %d\n",MKSTRING(x),(int)param->rc.x)
+#define RD(x) printf("%-32s : %f\n",MKSTRING(x),(double)param->rc.x)
+    printf("*********************************************\n");
+    printf("***             Rate Control              ***\n");
+    printf("*********************************************\n");
+
     RI(rateControlMode);
     RI(qp);
     RI(bitrate);
     
-#if X265_BUILD >= 41
     RI(bStrictCbr);
-#else
-    RD(rateTolerance);
-#endif
     
     RD(qCompress);
     RD(ipFactor);
@@ -442,11 +529,11 @@ void dumpx265Setup(x265_param *param)
     RD(qblur);
     RD(complexityBlur);
 
-#define VI(x) printf(#x"\t:%d\n",(int)param->vui.x)
-    printf("*************************************\n");
-    printf("***  Video Usability Information  ***\n");
-    printf("*************************************\n");
-    
+#define VI(x) printf("%-32s : %d\n",MKSTRING(x),(int)param->vui.x)
+    printf("*********************************************\n");
+    printf("***      Video Usability Information      ***\n");
+    printf("*********************************************\n");
+
     VI(aspectRatioIdc);
     VI(sarWidth);
     VI(sarHeight);
