@@ -10,7 +10,7 @@ class flyBlacken : public ADM_flyDialogRgb
    uint8_t    upload(void);
    uint8_t    autocrop(void);
               flyBlacken (QDialog *parent,uint32_t width,uint32_t height,ADM_coreVideoFilter *in,
-                                    ADM_QCanvas *canvas, QSlider *slider) 
+                                    ADM_QCanvas *canvas, ADM_QSlider *slider)
                 : ADM_flyDialogRgb(parent,width, height,in,canvas, slider,RESIZE_LAST) {};
 };
 
