@@ -17,6 +17,7 @@
 #include "Q_eq2.h"
 #include "ADM_toolkitQt.h"
 #include <QSignalMapper>
+#include <QAction>
 
 //
 //	Video is in YV12 Colorspace
@@ -37,14 +38,22 @@ Ui_eq2Window::Ui_eq2Window(QWidget *parent, eq2 *param,ADM_coreVideoFilter *in)
 
     canvas=new ADM_QCanvas(ui.graphicsView,width,height);
 
-    myCrop=new flyEq2(this, width, height,in,canvas,ui.horizontalSlider);
+    scene=new QGraphicsScene(this);
+    scene->setSceneRect(0,0,256,128);
+    ui.graphicsViewHistogram->setScene(scene);
+    ui.graphicsViewHistogram->scale(1.0,1.0);
+
+    myCrop=new flyEq2(this, width, height,in,canvas,ui.horizontalSlider,scene);
     memcpy(&(myCrop->param),param,sizeof(eq2));
     myCrop->_cookie=&ui;
-    myCrop->addControl(ui.toolboxLayout);
+    myCrop->addControl(ui.toolboxLayout, ControlOption::PeekOriginalBtn);
+    myCrop->setTabOrder();
     myCrop->upload();
     myCrop->sliderChanged();
     myCrop->update();
 
+    ui.horizontalSliderContrast->setFocus();
+
     QSignalMapper *signalMapper = new QSignalMapper(this);
     connect( signalMapper,SIGNAL(mapped(QWidget*)),this,SLOT(resetSlider(QWidget*)));
     connect( ui.horizontalSlider,SIGNAL(valueChanged(int)),this,SLOT(sliderUpdate(int)));
@@ -66,11 +75,10 @@ Ui_eq2Window::Ui_eq2Window(QWidget *parent, eq2 *param,ADM_coreVideoFilter *in)
     SPINNER(Blue);
     SPINNER(Green);
 
+
     setResetSliderEnabledState();
 
-    show();
-    myCrop->adjustCanvasPosition();
-    canvas->parentWidget()->setMinimumSize(30,30); // allow resizing after the dialog has settled
+    setModal(true);
 }
 
 /**
@@ -158,6 +166,7 @@ Ui_eq2Window::~Ui_eq2Window()
     myCrop=NULL;
     if(canvas) delete canvas;
     canvas=NULL;
+    scene=NULL;
 }
 
 /**
@@ -186,6 +195,16 @@ void Ui_eq2Window::resizeEvent(QResizeEvent *event)
     myCrop->adjustCanvasPosition();
 }
 
+/**
+    \fn showEvent
+*/
+void Ui_eq2Window::showEvent(QShowEvent *event)
+{
+    QDialog::showEvent(event);
+    myCrop->adjustCanvasPosition();
+    canvas->parentWidget()->setMinimumSize(30,30); // allow resizing both ways after the dialog has settled
+}
+
 #define sliderSet(x,y) w->horizontalSlider##x->setValue((int)(param.y*100));
 #define sliderGet(x,y) param.y=w->horizontalSlider##x->value()/100.;
 //************************
@@ -208,6 +227,8 @@ uint8_t flyEq2::upload(void)
     sliderSet(Green,ggamma);
     sliderSet(Blue,bgamma);
 
+    tablesDone = false;
+
     return 1;
 }
 
@@ -229,9 +250,43 @@ uint8_t flyEq2::download(void)
     sliderGet(Green,ggamma);
     sliderGet(Blue,bgamma);
 
+    tablesDone = false;
+
     return 1;
 }
 
+/**
+    \fn setTabOrder
+*/
+void flyEq2::setTabOrder(void)
+{
+    Ui_eq2Dialog *w=(Ui_eq2Dialog *)_cookie;
+    std::vector<QWidget *> controls;
+
+#define SLDR(x) controls.push_back(w->horizontalSlider##x);
+    SLDR(Contrast)
+    SLDR(Brightness)
+    SLDR(Saturation)
+    SLDR(Initial)
+    SLDR(Red)
+    SLDR(Green)
+    SLDR(Blue)
+    SLDR(Weight)
+
+    controls.insert(controls.end(), buttonList.begin(), buttonList.end());
+    controls.push_back(w->horizontalSlider);
+
+    QWidget *first, *second;
+
+    for(std::vector<QWidget *>::iterator tor = controls.begin(); tor != controls.end(); ++tor)
+    {
+        if(tor+1 == controls.end()) break;
+        first = *tor;
+        second = *(tor+1);
+        _parent->setTabOrder(first,second);
+        //ADM_info("Tab order: %p (%s) --> %p (%s)\n",first,first->objectName().toUtf8().constData(),second,second->objectName().toUtf8().constData());
+    }
+}
 /**
       \fn     DIA_getEQ2Param
       \brief  Handle MPlayer EQ2 flyDialog
