@@ -20,6 +20,8 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <QPushButton>
+
 #include "Q_hue.h"
 #include "ADM_toolkitQt.h"
 
@@ -39,18 +41,23 @@
         canvas=new ADM_QCanvas(ui.graphicsView,width,height);
         
         myCrop=new flyHue( this,width, height,in,canvas,ui.horizontalSlider);
-        memcpy(&(myCrop->param),param,sizeof(hue));
+        myCrop->setParam(param);
         myCrop->_cookie=&ui;
-        myCrop->addControl(ui.toolboxLayout);
+        myCrop->addControl(ui.toolboxLayout, ControlOption::PeekOriginalBtn);
+        myCrop->setTabOrder();
         myCrop->upload();
         myCrop->sliderChanged();
 
-
+        ui.horizontalSliderHue->setFocus();
+        ui.horizontalSliderSaturation->setScale(1,10,1);
         connect( ui.horizontalSlider,SIGNAL(valueChanged(int)),this,SLOT(sliderUpdate(int)));
 #define SPINNER(x) connect( ui.horizontalSlider##x,SIGNAL(valueChanged(int)),this,SLOT(valueChanged(int))); 
           SPINNER(Hue);
           SPINNER(Saturation);
 
+        QPushButton *resetButton = ui.buttonBox->button(QDialogButtonBox::Reset);
+        connect(resetButton,SIGNAL(clicked()),this,SLOT(reset()));
+
         setModal(true);
   }
   void Ui_hueWindow::sliderUpdate(int foo)
@@ -59,9 +66,8 @@
   }
   void Ui_hueWindow::gather(hue *param)
   {
-    
         myCrop->download();
-        memcpy(param,&(myCrop->param),sizeof(hue));
+        myCrop->getParam(param);
   }
 Ui_hueWindow::~Ui_hueWindow()
 {
@@ -79,6 +85,16 @@ void Ui_hueWindow::valueChanged( int f )
   lock--;
 }
 
+void Ui_hueWindow::reset(void)
+{
+    if(lock) return;
+    lock++;
+    myCrop->reset();
+    myCrop->upload();
+    myCrop->sameImage();
+    lock--;
+}
+
 void Ui_hueWindow::resizeEvent(QResizeEvent *event)
 {
     if(!canvas->height())
@@ -101,20 +117,62 @@ void Ui_hueWindow::showEvent(QShowEvent *event)
 //************************
 uint8_t flyHue::upload(void)
 {
-      Ui_hueDialog *w=(Ui_hueDialog *)_cookie;
+    Ui_hueDialog *w=(Ui_hueDialog *)_cookie;
 
-        MYSPIN(Saturation)->setValue((int)(param.saturation*10));
-        MYSPIN(Hue)->setValue((int)param.hue);
-        return 1;
+    MYSPIN(Saturation)->blockSignals(true);
+    MYSPIN(Hue)->blockSignals(true);
+
+    MYSPIN(Saturation)->setValue((int)(flyset.param.saturation*10));
+    MYSPIN(Hue)->setValue((int)flyset.param.hue);
+
+    MYSPIN(Saturation)->blockSignals(false);
+    MYSPIN(Hue)->blockSignals(false);
+
+    update();
+
+    return 1;
 }
 uint8_t flyHue::download(void)
 {
-       Ui_hueDialog *w=(Ui_hueDialog *)_cookie;
-         param.hue=MYSPIN(Hue)->value();
-         param.saturation=MYSPIN(Saturation)->value()/10.;
-return 1;
+    Ui_hueDialog *w=(Ui_hueDialog *)_cookie;
+
+    flyset.param.hue=MYSPIN(Hue)->value();
+    flyset.param.saturation=MYSPIN(Saturation)->value()/10.;
+
+    update();
+
+    return 1;
 }
 
+void flyHue::setTabOrder(void)
+{
+    Ui_hueDialog *w=(Ui_hueDialog *)_cookie;
+    std::vector<QWidget *> controls;
+    controls.push_back(MYSPIN(Hue));
+    controls.push_back(MYSPIN(Saturation));
+    controls.insert(controls.end(), buttonList.begin(), buttonList.end());
+    controls.push_back(w->horizontalSlider);
+    //controls.push_back(w->graphicsView);
+#if 0 /* don't mess with button box */
+    controls.push_back(w->buttonBox->button(QDialogButtonBox::Reset));
+    { // button box stuff
+    QList<QAbstractButton *> buttonBoxList = w->buttonBox->buttons();
+    QList<QAbstractButton *>::reverse_iterator bit;
+    for (bit = buttonBoxList.rbegin(); bit != buttonBoxList.rend(); ++bit)
+        controls.push_back(*bit);
+    }
+#endif
+    QWidget *first, *second;
+
+    for(std::vector<QWidget *>::iterator tor = controls.begin(); tor != controls.end(); ++tor)
+    {
+        if(tor+1 == controls.end()) break;
+        first = *tor;
+        second = *(tor+1);
+        _parent->setTabOrder(first,second);
+        //ADM_info("Tab order: %p (%s) --> %p (%s)\n",first,first->objectName().toUtf8().constData(),second,second->objectName().toUtf8().constData());
+    }
+}
 /**
       \fn     DIA_getCropParams
       \brief  Handle crop dialog
