@@ -125,9 +125,9 @@ bool vdpauVideoFilter::setupVdpau(void)
         }
     }
     {
-    int paddedHeight=(previousFilter->getInfo()->height+15)&~15;
-    if(VDP_STATUS_OK!=admVdpau::mixerCreate(previousFilter->getInfo()->width,
-                                            paddedHeight,&mixer)) 
+    int paddedWidth = admVdpau::dimensionRoundUp(previousFilter->getInfo()->width);
+    int paddedHeight = admVdpau::dimensionRoundUp(previousFilter->getInfo()->height);
+    if(VDP_STATUS_OK!=admVdpau::mixerCreate(paddedWidth,paddedHeight,&mixer))
     {
         ADM_error("Cannot create mixer\n");
         goto badInit;
@@ -135,7 +135,7 @@ bool vdpauVideoFilter::setupVdpau(void)
     }
     setIdentityCSC();
     tempBuffer=new uint8_t[info.width*info.height*4];
-    scaler=new ADMColorScalerSimple( info.width,info.height, ADM_COLOR_RGB32A,ADM_COLOR_YV12);
+    scaler=new ADMColorScalerSimple( info.width,info.height, ADM_PIXFRMT_RGB32A,ADM_PIXFRMT_YV12);
     ADM_info("VDPAU setup ok\n");
     return true;
 badInit:
@@ -198,10 +198,8 @@ vdpauVideoFilter::~vdpauVideoFilter()
 */
 bool vdpauVideoFilter::configure( void) 
 {
-    
-     
-     diaElemUInteger  tWidth(&(configuration.targetWidth),QT_TRANSLATE_NOOP("vdpresize","Width :"),16,2048);
-     diaElemUInteger  tHeight(&(configuration.targetHeight),QT_TRANSLATE_NOOP("vdpresize","Height :"),16,2048);
+     diaElemUInteger  tWidth(&(configuration.targetWidth),QT_TRANSLATE_NOOP("vdpresize","Width:"),16,MAXIMUM_SIZE);
+     diaElemUInteger  tHeight(&(configuration.targetHeight),QT_TRANSLATE_NOOP("vdpresize","Height:"),16,MAXIMUM_SIZE);
      
      diaElem *elems[]={&tWidth,&tHeight};
      
@@ -307,7 +305,7 @@ bool vdpauVideoFilter::getNextFrame(uint32_t *fn,ADMImage *image)
         
         struct ADM_vdpauRenderState *rndr = (struct ADM_vdpauRenderState *)next->refDescriptor.refHwImage;
         tmpSurface=rndr->surface;
-        printf("image is already vdpau %d\n",(int)tmpSurface);
+        //printf("image is already vdpau %d\n",(int)tmpSurface);
     }else
     {
         //printf("Uploading image to vdpau\n");
